/*
** Extras. These routines are implementations of standard functions
** which are not always available. This file is meant to be added
** to distributions which may be interested in one or more of the
** routines. Ifdefs are needed to activate each routine.
**
** NEED_TEMPNAM
** NEED_DIRNAME
*/

/* -------------------------------------------------------------*/

#include <stdio.h>

/* -------------------------------------------------------------*/

#ifdef NEED_TEMPNAM
/*
** tempnam is a fairly standard routine, however it is not available
** all systems. This implementation attempts to adhere to the standard
** API. The following deviations apply:
**
**    - Fitness of a given directory is not determined.
**    - P_Tmpdir is not considered.
*/
char *tempnam( const char* dir, const char *prefix )
{
    char *p;
    char tbuf[1023+1];
    unsigned pid = (unsigned)getpid();
    static unsigned count = 0;

    if( dir == NULL )
    {
	if( (dir=(const char*)getenv("TMPDIR")) == NULL )
	    dir = "/tmp";
    }

    if( prefix == NULL )
	prefix = "p";

    sprintf( tbuf, "%s/%s%u%u", dir, prefix, pid, count );

    if( (p=(char*)malloc(strlen(tbuf)+1)) == NULL )
	return NULL;

    ++count;

    strcpy( p, tbuf );

    return p;
}
#endif

#if 0
main()
{
    char *s;
    int i;

    s = tempnam(NULL,NULL);
    fprintf( stdout, "tempnam(NULL,NULL) \"%s\".\n", s );
    free( s );

    s = tempnam("/foo",NULL);
    fprintf( stdout, "tempnam(\"/foo\",NULL) \"%s\".\n", s );
    free( s );

    s = tempnam(NULL,"pfx");
    fprintf( stdout, "tempnam(NULL,\"pfx\") \"%s\".\n", s );
    free( s );

    s = tempnam("/usr/lib","pfx");
    fprintf( stdout, "tempnam(\"/usr/lib\",\"pfx\") \"%s\".\n", s );
    free( s );

    fputs( "Five files using tempnam(\"/foo/bar\",\"pfx\").\n", stdout );
    for( i=0; i<5; ++i )
    {
	fprintf( stdout, "   \"%s\".\n", tempnam("/foo/bar","pfx") );
    }
}
#endif

/* -------------------------------------------------------------*/

#ifdef NEED_DIRNAME
/*
** This is an implementation of the standard (but not universally available)
** dirname routine. Like the original, this routine writes in the given
** string.
*/

char  *dirname( char *path )
{
    char *p = path;
    char *curr = ".";

    if( path == NULL )
	return curr;

    /* Go to end of string. */
    for( p=path; *p!='\0'; ++p );

    /* Find last slash. */
    for( ; (p!=path)&&(*p!='/'); --p );

    /* Simple filename or root. */
    if( p == path )
    {
	if( *path == '/' )
	    return "/";
	return curr;
    }

    *p = '\0';

    if( *path == '\0' )
	return curr;

    return path;
}
#endif

#if 0
main()
{
    fprintf( stdout, "/ \"%s\"\n", dirname("/") );
    fprintf( stdout, ". \"%s\"\n", dirname(".") );
    fprintf( stdout, ".. \"%s\"\n", dirname("..") );
    fprintf( stdout, "/usr/lib \"%s\"\n", dirname("/usr/lib") );
    fprintf( stdout, "/foo.c \"%s\"\n", dirname("/") );
}
#endif

/* -------------------------------------------------------------*/
