/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

/*
** Glob objects are semi-persistant representations of Glob files.
** Data is read from the file system, but changes made in memory are
** not written out.
** 
** When an instance is created, a Glob filename must be specified. Once
** created an instance must be verified for validity.
*/

#ifndef GLOB_H
#define GLOB_H 1

#include<sys/stat.h>
#include<stdio.h>
#include<string.h>
#include<stdlib.h>

/*
** All ints are big-endian (i.e. NOT Intel style).
*/

#define GLOB_MAX_COMPONENTS 256
#define GLOB_MAX_NAME 16
#define GLOB_MAX_TYPE 32

struct globidx
{
    char type[GLOB_MAX_TYPE];
    char name[GLOB_MAX_NAME];
    int begin;
    int length;
};

/*
** Header in a Glob file.
*/
struct globhdr
{
    char magic1;
    char magic2;
    unsigned short version;
    unsigned int count;        /* Number of indices. */
    /*
    ** Above is followed by a variable length number of struct globidx.
    */
};

#ifdef __cplusplus
class Glob
{
    int fd;
    int version;
    unsigned int count;
    struct globidx *idxs;
public:
    Glob( const char* );
    ~Glob( void );
    int initialized( void );
    char *extract( const char*, int* );
    void dump( int );
};
#endif

#endif /* GLOB_H */
