#include <stdio.h>

/*
** tempnam is a fairly standard routine, however it is not available
** all systems. This implementation attempts to adhere to the standard
** API. The following deviations apply:
**
**    - Fitness of a given directory is not determined.
**    - P_Tmpdir is not considered.
*/
char *tempnam( const char* dir, const char *prefix )
{
    char *p;
    char tbuf[1023+1];
    unsigned pid = (unsigned)getpid();
    static unsigned count = 0;

    if( dir == NULL )
    {
	if( (dir=(const char*)getenv("TMPDIR")) == NULL )
	    dir = "/tmp";
    }

    if( prefix == NULL )
	prefix = "p";

    sprintf( tbuf, "%s/%s%u%u", dir, prefix, pid, count );

    if( (p=(char*)malloc(strlen(tbuf)+1)) == NULL )
	return NULL;

    ++count;

    strcpy( p, tbuf );

    return p;
}

#if 0
main()
{
    char *s;
    int i;

    s = tempnam(NULL,NULL);
    fprintf( stdout, "tempnam(NULL,NULL) \"%s\".\n", s );
    free( s );

    s = tempnam("/foo",NULL);
    fprintf( stdout, "tempnam(\"/foo\",NULL) \"%s\".\n", s );
    free( s );

    s = tempnam(NULL,"pfx");
    fprintf( stdout, "tempnam(NULL,\"pfx\") \"%s\".\n", s );
    free( s );

    s = tempnam("/usr/lib","pfx");
    fprintf( stdout, "tempnam(\"/usr/lib\",\"pfx\") \"%s\".\n", s );
    free( s );

    fputs( "Five files using tempnam(\"/foo/bar\",\"pfx\").\n", stdout );
    for( i=0; i<5; ++i )
    {
	fprintf( stdout, "   \"%s\".\n", tempnam("/foo/bar","pfx") );
    }
}
#endif
