/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

/*
** extractor.
**
** extractor <glob> <component>
**
** Extract a copy of the named component from the named glob.
*/
#include<stdio.h>
#include<unistd.h>
#include<stdlib.h>
#include<fcntl.h>
#include<errno.h>
#include"glob.h"

extern char *sys_errlist[];

void usage( void )
{
    fputs( "usage: extract <glob> <component>\n", stdout );
}

main( int argc, char *argv[] )
{
    char *buf;
    char *outfile, *component, *globfile;
    int i;
    int len;
    int fd;
    Glob *glob;

    outfile = NULL;
    component = NULL;
    globfile = NULL;

    for( i=1; i<argc; ++i )
    {
	if( argv[i][0] == '-' )
	{
	    fprintf( stdout, "ERROR. Unrecognized option \"%s\".\n",
		argv[i] );
	    usage();
	    exit( 1 );
	}
	if( globfile == NULL )
	{
	    globfile = argv[i];
	    continue;
	}
	if( component == NULL )
	{
	    component = argv[i];
	    continue;
	}
	fprintf( stdout, "ERROR. Unrecognized argument \"%s\".\n",
	    argv[i] );
	usage();
	exit( 1 );
    }

    if( globfile == NULL )
    {
	fputs( "ERROR. Specify a glob file.\n", stdout );
	usage();
	exit( 1 );
    }

    if( component == NULL )
    {
	fputs( "ERROR. Specify a component.\n", stdout );
	usage();
	exit( 1 );
    }

    outfile = component;

    glob = new Glob( globfile );
    if( !glob->initialized() )
    {
	fputs( "ERROR. Could not create Glob object.\n", stdout );
	exit( 1 );
    }

    if( (buf=glob->extract(component,&len)) == NULL )
    {
	fputs( "ERROR. Glob extract failed.\n", stdout );
	delete glob;
	exit( 1 );
    }

    if( (fd=open(outfile,O_WRONLY|O_CREAT,0664)) == -1 )
    {
	fprintf( stdout, "ERROR. Bad open of \"%s\"\". %s\n",
	    outfile, sys_errlist[errno] );
	delete glob;
	free( buf );
	exit( 1 );
    }

    if( write(fd,buf,len) == -1 )
    {
	fprintf( stdout, "ERROR. Bad write of \"%s\"\". %s\n",
	    outfile, sys_errlist[errno] );
	free( buf );
	delete glob;
	exit( 1 );
    }

#if 0
    if( (fp=fopen(outfile,"w")) == NULL )
    {
	fprintf( stdout, "ERROR. Bad fopen of \"%s\"\".\n", outfile );
	free( buf );
	exit( 1 );
    }

    fputs( buf, fp );

    fclose( fp );
#endif

    free( buf );
    close( fd );
    delete glob;

    exit( 0 );
}
