/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

#include <stdio.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include "ButtonP.h"

/*
** Widget hooks.
*/
static void hook_initialize( Widget, Widget, ArgList, Cardinal* );
static void hook_destroy( Widget w );
static void hook_resize( Widget w );
static void hook_redisplay( Widget, XEvent*, Region );
static XtGeometryResult hook_queryGeometry( Widget, XtWidgetGeometry*, 
    XtWidgetGeometry* );

static void draw_button( Widget w );

static Boolean hook_setValues( Widget, Widget, Widget, ArgList, Cardinal* );

/*
** Action procedures.
*/
static void action_press( Widget, XEvent*, char**, Cardinal* );
static void action_release( Widget, XEvent*, char**, Cardinal* );
static void action_leave( Widget, XEvent*, char**, Cardinal* );

static XtResource resources[] =
{
    {
	XljpNactivateCallback,
	XljpCActivateCallback,
	XtRCallback,
	sizeof(XtCallbackList),
	XtOffsetOf(LjpButtonRec,button.activate_callback),
	XtRImmediate, NULL
    },
    {
	XljpNarmColor,
	XljpCArmColor,
	XtRPixel,
	sizeof(Pixel),
	XtOffsetOf(LjpButtonRec,button.arm_color),
	XtRString,
	XtDefaultForeground
    }
};
	  
/* Action table - where procedures are mapped to strings */
static XtActionsRec button_actions[] =
{
    { "press",		action_press },
    { "release",	action_release },
    { "leave",		action_leave }
};

/* Translation table - where events are mapped to action names */
static char defaultTranslations[] = "\
<Btn1Down>:	press()\n\
<Btn1Up>:	release()\n\
<LeaveNotify>:	leave()";

LjpButtonClassRec ljpButtonClassRec =
{
   {
      (WidgetClass) &ljpLabelClassRec,		/* superclass         */
      "LjpButton",                            	/* class_name         */
      sizeof(LjpButtonRec),                   	/* widget_size        */
      NULL,                          		/* class_initialize   */
      NULL,		        		/* class_part_initiali*/
      FALSE,                                    /* class_inited       */
      hook_initialize,                          /* initialize         */
      NULL,                                     /* initialize_hook    */
      XtInheritRealize,                         /* realize            */
      button_actions,                           /* actions            */
      XtNumber(button_actions),                 /* num_actions        */
      resources,                                /* resources          */
      XtNumber(resources),                      /* num_resources      */
      NULLQUARK,                                /* xrm_class          */
      TRUE,                                     /* compress_motion    */
      XtExposeCompressMaximal,                  /* compress_exposure  */
      TRUE,                                     /* compress_enterleave*/
      FALSE,                                    /* visible_interest   */
      hook_destroy,                             /* destroy            */
      hook_resize,                              /* resize             */
      hook_redisplay,                 		/* expose             */
      hook_setValues,                       /* set_values         */
      NULL,					/* set_values_hook    */
      NULL,                            		/* set_values_almost  */
      NULL,                                     /* get_values_hook    */
      NULL,                                     /* accept_focus       */
      XtVersion,                                /* version            */
      NULL,                                     /* callback_private   */
      defaultTranslations,                      /* tm_table           */
      hook_queryGeometry,             		/* query_geometry     */
      XtInheritDisplayAccelerator,              /* display accel      */
      NULL,                                     /* extension          */
    },
    /* Primitve class. */
    {
	XljpInheritDrawFrame,			/* draw_frame.		*/
	XljpInheritCalcInterior,		/* calc_interior.	*/
	NULL                                    /* extension		*/
    },
    /* Label. */
    {
	XljpInheritDrawLabel,			/* draw_label.		*/
	NULL                                    /* extension          */
    },
    /* Button. */
    {
	NULL                                    /* extension          */
    }
};

WidgetClass ljpButtonWidgetClass = (WidgetClass)&ljpButtonClassRec;

static void hook_initialize( Widget w, Widget neww, ArgList args,
    Cardinal *num_args )
{
    LjpButtonWidget bw = (LjpButtonWidget)w;
    LjpButtonWidget new_bw = (LjpButtonWidget)neww;
    LjpButtonWidgetClass bc = (LjpButtonWidgetClass)XtClass(neww);

    /*
    fprintf( stdout, "LjpButton %x hook init\n", (unsigned int)neww );
    */

    /*
    fprintf( stdout, "LjpButton %x hook init w%d h%d, w%d h%d\n",
	new_bw, bw->core.width, bw->core.height,
	new_bw->core.width, new_bw->core.height );
    */

    new_bw->button.activate_callback = NULL;

    new_bw->button.armed = False;
}

static void hook_destroy( Widget w )
{
    LjpButtonWidget bw = (LjpButtonWidget)w;

    if( bw->button.activate_callback != NULL )
	XtRemoveAllCallbacks( w, XljpNactivateCallback );
}

static void hook_resize( Widget w )
{
    LjpButtonWidget bw = (LjpButtonWidget)w;

    /*
    fprintf( stdout, "LjpButton hook resize ml%d mr%d\n",
	bw->primitive.margin_left,
	bw->primitive.margin_right );
    */
    draw_button( w );
}

static void hook_redisplay( Widget w, XEvent *event, Region region )
{
    LjpButtonWidget bw = (LjpButtonWidget)w;

    /*
    fprintf( stdout, "LjpButton hook redisp ml%d mr%d\n",
	bw->primitive.margin_left,
	bw->primitive.margin_right );
    */

    draw_button( w );
}

static XtGeometryResult hook_queryGeometry( Widget w,
    XtWidgetGeometry *request, XtWidgetGeometry *reply )
{
    LjpButtonWidget dw = (LjpButtonWidget)w;

    reply->request_mode = CWWidth|CWHeight;

    if( ((request->request_mode&CWWidth) &&
	request->width==reply->width) &&
	((request->request_mode & CWHeight) &&
	request->height==reply->height) )
    {
	return XtGeometryYes;
    }

    if( (reply->width==dw->core.width) &&
	(reply->height==dw->core.height) )
    {
	return XtGeometryNo; 
    }

    return XtGeometryAlmost; 
}

static void draw_button( Widget aw )
{
    LjpButtonWidget bw = (LjpButtonWidget)aw;
    LjpButtonWidgetClass bc = (LjpButtonWidgetClass)XtClass(aw);
    Pixel color;

    if( bw->button.armed == True )
    {
	bw->primitive.shadow_type = XljpShadowIn;
	color = bw->button.arm_color;
    }
    else
    {
	bw->primitive.shadow_type = XljpShadowOut;
	color = bw->core.background_pixel;
    }
    /*
    fprintf( stdout, "LjpButton draw ml%d mr%d\n",
	bw->primitive.margin_left,
	bw->primitive.margin_right );
    */

    bc->label_class.draw_label( aw, color );
}

static void action_leave( Widget w, XEvent *event, char **params,
    Cardinal *num_params )
{
    LjpButtonWidget dw = (LjpButtonWidget)w;
    struct ljpButtonCallbackStruct rec;
    
    if( dw->button.armed == False )
	return;

    dw->button.armed = False;
    draw_button( w );
}

static void action_press( Widget w, XEvent *event, char **params,
    Cardinal *num_params )
{
    LjpButtonWidget dw = (LjpButtonWidget)w;
    struct ljpButtonCallbackStruct rec;

    if( dw->button.armed == True )
	return;

    dw->button.armed = True;
    draw_button( w );

    rec.reason = XljpCR_PRESS;
    rec.event = event;
    rec.value = 1;
    if( dw->button.activate_callback )
	XtCallCallbackList( (Widget)dw, dw->button.activate_callback, &rec );
}

static void action_release( Widget w, XEvent *event, char **params,
    Cardinal *num_params )
{
    LjpButtonWidget bw = (LjpButtonWidget)w;
    struct ljpButtonCallbackStruct rec;

    if( bw->button.armed == False )
	return;

    bw->button.armed = False;
    draw_button( w );

    rec.reason = XljpCR_RELEASE;
    rec.event = event;
    rec.value = 0;
    if( bw->button.activate_callback )
	XtCallCallbackList( (Widget)bw, bw->button.activate_callback, &rec );
}
static Boolean hook_setValues( Widget a, Widget b, Widget c, ArgList d, Cardinal* e )
{
}
