/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

#include <stdio.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include "FrameP.h"

/*
** Widget hooks.
*/
static void hook_initialize( Widget, Widget, ArgList, Cardinal* );
static Boolean hook_setValues( Widget, Widget, Widget, ArgList, Cardinal* );
static void hook_redisplay( Widget, XEvent*, Region );
static void hook_resize( Widget );
static void hook_changeManaged( Widget );
static XtGeometryResult hook_geometryManager( Widget, XtWidgetGeometry*,
    XtWidgetGeometry* );
static XtGeometryResult hook_queryGeometry( Widget, XtWidgetGeometry*, 
    XtWidgetGeometry* );
static void hook_destroy( Widget );
static void get_sizes( LjpFrameWidget, Dimension*, Dimension*, Dimension* );

static void mkGC( LjpFrameWidget );
static void layout( Widget, Boolean );
static void draw_frame( Widget );
static print_request( const char*, XtWidgetGeometry* );

static XtResource resources[] =
{
    {
	XljpNmarginWidth,
	XljpCMarginWidth,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpFrameRec,frame.margin_width),
	XtRImmediate,
	(XtPointer)0
    },
    {
	XljpNmarginHeight,
	XljpCMarginHeight,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpFrameRec,frame.margin_height),
	XtRImmediate,
	(XtPointer)0
    },
    {
	XljpNmarginTop,
	XljpCMarginTop,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpFrameRec,frame.margin_top),
	XtRImmediate,
	(XtPointer)2
    },
    {
	XljpNmarginBottom,
	XljpCMarginBottom,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpFrameRec,frame.margin_bottom),
	XtRImmediate,
	(XtPointer)2
    },
    {
	XljpNmarginLeft,
	XljpCMarginLeft,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpFrameRec,frame.margin_left),
	XtRImmediate,
	(XtPointer)2
    },
    {
	XljpNmarginRight,
	XljpCMarginRight,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpFrameRec,frame.margin_right),
	XtRImmediate,
	(XtPointer)2
    },
    {
	XljpNshadowThickness,
	XljpCShadowThickness,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpFrameRec,frame.shadow_thickness),
	XtRImmediate,
	(XtPointer)1
    },
    {
	XljpNshadowType,
	XljpCShadowType,
	XtRInt,
	sizeof(enum XljpShadowType),
	XtOffsetOf(LjpFrameRec,frame.shadow_type),
	XtRImmediate,
	(XtPointer)XljpShadowOut
    },
    {
	XljpNtopShadowColor,
	XljpCTopShadowColor,
	XtRPixel,
	sizeof(Pixel),
	XtOffsetOf(LjpFrameRec,frame.pixelTopShadow),
	XtRString,
	XtDefaultForeground
    },
    {
	XljpNbottomShadowColor,
	XljpCBottomShadowColor,
	XtRPixel,
	sizeof(Pixel),
	XtOffsetOf(LjpFrameRec,frame.pixelBottomShadow),
	XtRString,
	XtDefaultBackground
    }
};
	  
LjpFrameClassRec ljpFrameClassRec =
{
   {
      (WidgetClass) &compositeClassRec,         /* superclass         */
      "LjpFrame",                            	/* class_name         */
      sizeof(LjpFrameRec),                   	/* widget_size        */
      NULL,                          		/* class_initialize   */
      NULL,                            		/* class_part_initiali*/
      FALSE,                                    /* class_inited       */
      hook_initialize,                          /* initialize         */
      NULL,                                     /* initialize_hook    */
      XtInheritRealize,                         /* realize            */
      NULL, 	                                /* actions            */
      0,                                        /* num_actions        */
      resources,                                /* resources          */
      XtNumber(resources),                      /* num_resources      */
      NULLQUARK,                                /* xrm_class          */
      TRUE,                                     /* compress_motion    */
      XtExposeCompressMaximal,                  /* compress_exposure  */
      TRUE,                                     /* compress_enterleave*/
      FALSE,                                    /* visible_interest   */
      hook_destroy,                             /* destroy            */
      hook_resize,                              /* resize             */
      hook_redisplay,                 		/* expose             */
      hook_setValues,                           /* set_values         */
      NULL,                                     /* set_values_hook    */
      NULL,                            		/* set_values_almost  */
      NULL,                                     /* get_values_hook    */
      NULL,                                     /* accept_focus       */
      XtVersion,                                /* version            */
      NULL,                                     /* callback_private   */
      NULL, 	    	                        /* tm_table           */
      hook_queryGeometry,             		/* query_geometry     */
      XtInheritDisplayAccelerator,              /* display accel      */
      NULL,                                     /* extension          */
    },
    /* Composite class. */
    {
    /* geometry_manager   */    hook_geometryManager,
    /* change_managed     */    hook_changeManaged,
    /* insert_child	  */	XtInheritInsertChild,
    /* delete_child	  */	XtInheritDeleteChild,
    /* extension          */	NULL
    },
    /* Frame. */
    {
	NULL                                    /* extension          */
    }
};

WidgetClass ljpFrameWidgetClass = (WidgetClass)&ljpFrameClassRec;

static void hook_initialize( Widget w, Widget neww, ArgList args,
    Cardinal *num_args )
{
    LjpFrameWidget dw = (LjpFrameWidget)w;
    LjpFrameWidget new_fw = (LjpFrameWidget)neww;
    Dimension defaultWidth=0, defaultHeight=0;
    XGCValues values;
    unsigned long valuemask;

    /*
    fprintf( stdout, "LjpFrame 0x%x hook initialize\n", neww );
    */

    /*
    ** Override Core class default. App will have to change
    ** it after widget creation if it desires a different
    ** value.
    */
    new_fw->core.border_width = 0;

    if( neww->core.width == 0 )
	neww->core.width = 60;
    else
	neww->core.width = neww->core.width;

    if( neww->core.height == 0 )
	neww->core.height = 40;
    else
	neww->core.height = neww->core.height;

    if( new_fw->frame.shadow_thickness == 0 )
	new_fw->frame.shadow_thickness = 1;

    mkGC( new_fw );
}

static void hook_redisplay( Widget w, XEvent *event, Region region )
{
    LjpFrameWidget fw = (LjpFrameWidget)w;

    /*
    fprintf( stdout, "TMP LjpFrame %x hook redisp\n", w );
    */

    if( !XtIsRealized((Widget)fw) )
    {
	return;
    }

    /*fprintf( stdout, "TMP LjpFrame clear and draw\n" );*/

    draw_frame( w );
}

static Boolean hook_setValues( Widget oldw, Widget w, Widget neww,
    ArgList args, Cardinal *num_args )
{
    LjpFrameWidget old_fw = (LjpFrameWidget)oldw;
    LjpFrameWidget new_fw = (LjpFrameWidget)neww;
    Boolean do_redisplay = False;
    Boolean do_gc = False;

    /*
    fprintf( stdout, "LjpFrame %x hook setValues\n", w );
    */

    if( new_fw->core.background_pixel != old_fw->core.background_pixel ||
        new_fw->frame.pixelTopShadow != old_fw->frame.pixelTopShadow ||
        new_fw->frame.pixelBottomShadow != old_fw->frame.pixelBottomShadow )
    {
	do_gc = True;
    }

    if( new_fw->frame.margin_height != old_fw->frame.margin_height ||
        new_fw->frame.margin_width != old_fw->frame.margin_width ||
        new_fw->core.height != old_fw->core.height ||
        new_fw->core.width != old_fw->core.width ||
        new_fw->frame.shadow_thickness != old_fw->frame.shadow_thickness )
    {
	do_redisplay = True;
    }

    if( new_fw->frame.shadow_type != old_fw->frame.shadow_type )
	do_redisplay = True;

    if( do_gc )
    {
	XtReleaseGC( oldw, old_fw->frame.gcTop );
	XtReleaseGC( oldw, old_fw->frame.gcBottom );
	XtReleaseGC( oldw, old_fw->frame.gcClear );
	mkGC( new_fw );
    }

    return do_redisplay;
}

static void hook_destroy( Widget w )
{
    LjpFrameWidget fw = (LjpFrameWidget)w;
   
    XtReleaseGC( w, fw->frame.gcTop );
    XtReleaseGC( w, fw->frame.gcBottom );
    XtReleaseGC( w, fw->frame.gcClear );
}

static void hook_resize( Widget w )
{
    LjpFrameWidget fw = (LjpFrameWidget)w;
    Dimension nw, nh, thick;
    Widget cw;

    /*
    fprintf( stdout, "LjpFrame %x hook resize w%d h%d\n",
	w, w->core.width, w->core.height );
    */

    if( fw->composite.num_children < 1 )
	return;

    get_sizes( fw, &thick, &nw, &nh );

    /*
    fprintf( stdout, "LjpFrame %x resize extraw%d extrah%d\n", fw, nw, nh );
    */
    cw = fw->composite.children[0];
    nw = (fw->core.width-nw) - (2*cw->core.border_width);
    nh = (fw->core.height-nh) - (2*cw->core.border_width);
    XtConfigureWidget( cw,
	thick+fw->frame.margin_width+fw->frame.margin_left,
	thick+fw->frame.margin_height+fw->frame.margin_top,
	nw, nh, cw->core.border_width );
}

/*
** Parent wants to know our preffered size. Pass request along
** to child.
*/
static XtGeometryResult hook_queryGeometry( Widget w,
    XtWidgetGeometry *request,  XtWidgetGeometry *reply )
{
    LjpFrameWidget fw = (LjpFrameWidget)w;
    XtWidgetGeometry nrequest, nreply;
    XtGeometryResult result;
    Dimension nw, nh;

    /*
    fprintf( stdout, "LjpFrame %x hook queryGeometry\n", w );
    */

    if( fw->composite.num_children < 1 )
	return XtGeometryYes;

    /* Only pass to child certain requests, accept all others. */
    reply->request_mode =
        (CWWidth|CWHeight|CWStackMode) & request->request_mode;
    if( reply->request_mode == 0 )
        return XtGeometryYes;

    get_sizes( fw, NULL, &nw, &nh );
    nrequest.request_mode = reply->request_mode;
    nrequest.width = request->width - nw;
    nrequest.height = request->height - nh;
    nrequest.stack_mode = request->stack_mode;
    print_request( "pass parent req to child", &nrequest );
    result = XtQueryGeometry( fw->composite.children[0], &nrequest, &nreply );

    /* Give the child what it wants. */
    if( result == XtGeometryYes )
	return XtGeometryYes;
    if( result == XtGeometryNo )
	return XtGeometryNo;

    reply->request_mode &= nreply.request_mode;
    if( reply->request_mode == 0 )
	return XtGeometryYes;

    reply->width = nreply.width + nw;
    reply->height = nreply.height + nh;
    reply->sibling = nreply.sibling;
    reply->stack_mode = nreply.stack_mode;
    print_request( "child prefers", &nreply );
    return XtGeometryAlmost;
}

/*
** Requests from child are passed to the parent.
*/
static XtGeometryResult hook_geometryManager( Widget cw,
    XtWidgetGeometry *request, XtWidgetGeometry *reply )
{
    XtWidgetGeometry nrequest, nreply;
    XtGeometryResult result;
    Position xn, yn;
    Dimension nw, nh, extraw, extrah;
    LjpFrameWidget fw = (LjpFrameWidget)XtParent(cw);

    get_sizes( fw, NULL, &nw, &nh );

    /* Accept all but width and height without asking parent. */
    if( !(request->request_mode&(CWWidth|CWHeight)) )
	return XtGeometryYes;

    nrequest.request_mode = request->request_mode & (CWWidth|CWHeight);
    nrequest.width = request->width + nw;
    nrequest.height = request->height + nh;
    result = XtMakeGeometryRequest( (Widget)fw, &nrequest, &nreply );

    if( result == XtGeometryNo )
	return XtGeometryNo;
    if( result == XtGeometryYes )
	return XtGeometryYes;

    reply->request_mode = nreply.request_mode & (CWWidth|CWHeight);
    reply->width = nreply.width - nw;
    reply->height = nreply.height - nh;
    return XtGeometryAlmost;
}

static void hook_changeManaged( Widget w )
{
    LjpFrameWidget fw = (LjpFrameWidget)w;
    Widget cw;
    XtWidgetGeometry nrequest, nreply;
    XtGeometryResult result;
    Dimension nw, nh;

    /*
    fprintf( stdout, "LjpFrame %x hook changeManaged\n", w );
    */

    if( fw->composite.num_children == 0 )
	return;

    get_sizes( fw, NULL, &nw, &nh );
    cw = fw->composite.children[0];

    nrequest.request_mode = CWWidth | CWHeight;
    nrequest.width = cw->core.width + nw;
    nrequest.height = cw->core.height + nh;
    result = XtMakeGeometryRequest( (Widget)fw, &nrequest, &nreply );

    get_sizes( fw, NULL, &nw, &nh );
    XtConfigureWidget( cw,
	fw->frame.margin_width+nw+fw->frame.margin_left,
	fw->frame.margin_height+nh+fw->frame.margin_top,
	(fw->core.width-nw)-cw->core.border_width,
	(fw->core.height-nh)-cw->core.border_width,
	cw->core.border_width );
}

static void layout( Widget w, Boolean Doit )
{
    LjpFrameWidget fw = (LjpFrameWidget)w;
    Widget pw;
    Widget cw = NULL;
    Position nx, ny;

    /*fprintf( stdout, "TMP LjpFrame layout %d\n", Doit );*/

/*
    if( !XtIsRealized((Widget)w) )
    {
	Doit = False;
    } 
*/
    pw = XtParent(w);

    if( fw->composite.num_children > 1 )
    {
	XtAppError( XtWidgetToApplicationContext((Widget)fw),
	    "LjpFrame: Can only manage one widget." );
    }
    if( fw->composite.num_children == 1 )
	cw = fw->composite.children[0];

    /*
    ** Update child position.
    */
    if( Doit && (cw!=NULL) )
    {
	Position nx, ny;

	get_sizes( fw, NULL, (Dimension*)&nx, (Dimension*)&ny );

	if( (cw->core.x!=nx) || (cw->core.y!=ny) )
	{
	    XtMoveWidget( cw, nx, ny );
	    /*
	    fprintf( stdout, "TMP LjpFrame child %d,%d\n",
		cw->core.x, cw->core.y );
	    */
	}
    }
}

static void draw_frame( Widget w )
{
    int width, h, thick;
    XPoint tPoints[7], bPoints[7];
    GC gcTop, gcBottom;
    LjpFrameWidget fw = (LjpFrameWidget)w;

    switch( fw->frame.shadow_type )
    {
	case XljpShadowNone:
	    gcTop = gcBottom = fw->frame.gcClear;
	    break;
	case XljpShadowLine:
	    gcTop = gcBottom = fw->frame.gcTop;
	    break;
	case XljpShadowIn:
	case XljpShadowEngrave:
	    gcTop = fw->frame.gcBottom;
	    gcBottom = fw->frame.gcTop;
	    break;
	case XljpShadowOut:
	case XljpShadowRelief:
	    gcTop = fw->frame.gcTop;
	    gcBottom = fw->frame.gcBottom;
	    break;
    }

    width = fw->core.width;
    h = fw->core.height;
    thick = fw->frame.shadow_thickness;

    /* Upper left corner. */
    tPoints[0].x = 0;
    tPoints[0].y = 0;
    /* Right. */
    tPoints[1].x = width;
    tPoints[1].y = 0;
    /* Down-left. */
    tPoints[2].x = width-thick;
    tPoints[2].y = thick;
    /* Left. */
    tPoints[3].x = thick;
    tPoints[3].y = thick;
    /* Down. */
    tPoints[4].x = thick;
    tPoints[4].y = h-thick;
    /* Left-down. */
    tPoints[5].x = 0;
    tPoints[5].y = h;
    /* Up. */
    tPoints[6].x = 0;
    tPoints[6].y = 0;

    /* Lower right corner. */
    bPoints[0].x = width;
    bPoints[0].y = h;
    /* Left. */
    bPoints[1].x = 0;
    bPoints[1].y = h;
    /* Upper-right. */
    bPoints[2].x = thick;
    bPoints[2].y = h-thick;
    /* Right. */
    bPoints[3].x = width-thick;
    bPoints[3].y = h-thick;
    /* Up. */
    bPoints[4].x = width-thick;
    bPoints[4].y = thick;
    /* Up-right */
    bPoints[5].x = width;
    bPoints[5].y = 0;
    /* Down. */
    bPoints[6].x = width;
    bPoints[6].y = h;

    XFillPolygon( XtDisplay(fw), XtWindow(fw), gcTop, tPoints, 7,
	Nonconvex, CoordModeOrigin );
    XFillPolygon( XtDisplay(fw), XtWindow(fw), gcBottom, bPoints, 7,
	Nonconvex, CoordModeOrigin );

    switch( fw->frame.shadow_type )
    {
	case XljpShadowRelief:
	    gcTop = fw->frame.gcBottom;
	    gcBottom = fw->frame.gcTop;
	    break;
	case XljpShadowEngrave:
	    gcTop = fw->frame.gcTop;
	    gcBottom = fw->frame.gcBottom;
	    break;
	default:
	    return;
    }

    width -= (4*thick);
    h -= (4*thick);

    /* Upper left corner. */
    tPoints[0].x = 0+(2*thick)+1;
    tPoints[0].y = 0+(2*thick);
    /* Right. */
    tPoints[1].x = width+(2*thick);
    tPoints[1].y = 0+(2*thick);
    /* Left-down. */
    tPoints[2].x = (width-thick)+(2*thick);
    tPoints[2].y = thick+(2*thick);
    /* Left. */
    tPoints[3].x = thick+(2*thick);
    tPoints[3].y = thick+(2*thick);
    /* Down. */
    tPoints[4].x = thick+(2*thick);
    tPoints[4].y = (h-thick)+(2*thick);
    tPoints[5].x = 0+(2*thick);
    tPoints[5].y = h+(2*thick);
    tPoints[6].x = 0+(2*thick);
    tPoints[6].y = 0+(2*thick);

    /* Lower right corner. */
    bPoints[0].x = width+(2*thick);
    bPoints[0].y = h+(2*thick);
    /* Left. */
    bPoints[1].x = 0+(2*thick);
    bPoints[1].y = h+(2*thick);
    /* Up-right. */
    bPoints[2].x = thick+(2*thick);
    bPoints[2].y = (h-thick)+(2*thick);
    /* Right. */
    bPoints[3].x = width-thick+(2*thick);
    bPoints[3].y = (h-thick)+(2*thick);
    bPoints[4].x = width-thick+(2*thick);
    bPoints[4].y = thick+(2*thick);
    bPoints[5].x = width+(2*thick);
    bPoints[5].y = 0+(2*thick);
    bPoints[6].x = width+(2*thick);
    bPoints[6].y = h+(2*thick);

    XFillPolygon( XtDisplay(fw), XtWindow(fw), gcTop, tPoints, 7,
	Nonconvex, CoordModeOrigin );
    XFillPolygon( XtDisplay(fw), XtWindow(fw), gcBottom, bPoints, 7,
	Nonconvex, CoordModeOrigin );

#if 0
{
    tPoints[0].x = 12;
    tPoints[0].y = 0;
    tPoints[1].x = 11;
    tPoints[1].y = 1;
    tPoints[2].x = 10;
    tPoints[2].y = 2;
    XDrawPoints( XtDisplay(fw), XtWindow(fw), gcTop, tPoints, 3,
       CoordModeOrigin );
    tPoints[0].x = 12;
    tPoints[0].y = 3;
    tPoints[1].x = 11;
    tPoints[1].y = 4;
    tPoints[2].x = 10;
    tPoints[2].y = 5;
    XDrawPoints( XtDisplay(fw), XtWindow(fw), gcTop, tPoints, 3,
       CoordModeOrigin );
    tPoints[0].x = 15;
    tPoints[0].y = 6;
    tPoints[1].x = 15;
    tPoints[1].y = 7;
    tPoints[2].x = 15;
    tPoints[2].y = 8;
    XDrawPoints( XtDisplay(fw), XtWindow(fw), gcBottom, tPoints, 3,
       CoordModeOrigin );
    tPoints[0].x = 18;
    tPoints[0].y = 6;
    tPoints[1].x = 18;
    tPoints[1].y = 7;
    tPoints[2].x = 18;
    tPoints[2].y = 8;
    XDrawPoints( XtDisplay(fw), XtWindow(fw), gcTop, tPoints, 3,
       CoordModeOrigin );
}
#endif

}

static void mkGC( LjpFrameWidget dw )
{
    XGCValues values;
    XtGCMask mask;

    mask = GCForeground|GCBackground;
    values.foreground = dw->frame.pixelTopShadow;
    values.background = dw->core.background_pixel;

    dw->frame.gcTop = XCreateGC( XtDisplay(dw),
        RootWindow(XtDisplay(dw),DefaultScreen(XtDisplay(dw))),
        mask, &values );

    mask = GCForeground|GCBackground;
    values.foreground = dw->frame.pixelBottomShadow;
    values.background = dw->core.background_pixel;

    dw->frame.gcBottom = XCreateGC( XtDisplay(dw),
        RootWindow(XtDisplay(dw),DefaultScreen(XtDisplay(dw))),
        mask, &values );

    mask = GCForeground|GCBackground|GCFillStyle|GCFillRule;
    values.foreground = dw->core.background_pixel;
    values.background = dw->frame.pixelTopShadow;
    values.fill_style = FillSolid;
    values.fill_rule = EvenOddRule;

    dw->frame.gcClear = XCreateGC( XtDisplay(dw),
        RootWindow(XtDisplay(dw),DefaultScreen(XtDisplay(dw))),
        mask, &values );
}

/*
** Get shadow and margin sizes. The first is the width/height
** of a single side of the shadow. The final two arguments
** will contain the total width or height consumed by the
** shadows and margins (i.e. total-nw is interior width).
*/
static void get_sizes( LjpFrameWidget fw, Dimension *thick,
   Dimension *nw, Dimension *nh )
{
    Dimension w, h;

    w = fw->frame.shadow_thickness;
    h = fw->frame.shadow_thickness;

    /*
    ** Relief and Engrave shadow types are three times shadowThickness.
    */
    if( (fw->frame.shadow_type==XljpShadowRelief) ||
	(fw->frame.shadow_type==XljpShadowEngrave) )
    {
	w += (2*fw->frame.shadow_thickness);
	h += (2*fw->frame.shadow_thickness);
    }

    if( thick != NULL )
	*thick = w;

    if( nw != NULL )
    {
	*nw = (2*w) + (2*fw->frame.margin_width)+fw->frame.margin_left+
	    fw->frame.margin_right;
    }
    if( nh != NULL )
    {
	*nh = (2*h) + (2*fw->frame.margin_height)+fw->frame.margin_top+
	    fw->frame.margin_bottom;
    }
}

static print_request( const char *msg, XtWidgetGeometry *rec )
{
    fprintf( stdout,
	"LjpFrame %s w%d%c h%d%c\n",
	msg,
	rec->width, (rec->request_mode&CWWidth)?'*':' ',
	rec->height, (rec->request_mode&CWHeight)?'*':' ' );
}
