/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

#include <stdio.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include "GameWindoP.h"

/*
** Widget hooks.
*/
static void hook_initialize( Widget, Widget, ArgList, Cardinal* );
static Boolean hook_setValues( Widget, Widget, Widget, ArgList, Cardinal* );
static void hook_resize( Widget );
static void hook_changeManaged( Widget );
static void hook_insertChild( Widget, ArgList, Cardinal* );
static XtGeometryResult hook_geometryManager( Widget, XtWidgetGeometry*,
    XtWidgetGeometry* );
static XtGeometryResult hook_queryGeometry( Widget, XtWidgetGeometry*, 
    XtWidgetGeometry* );

static void mkGC( LjpGameWindowWidget );
static void get_sizes( LjpGameWindowWidget, Dimension*, Dimension* );
static void get_sizes_replace( LjpGameWindowWidget, Widget, Dimension,
    Dimension, Dimension*, Dimension* );
static void layout( LjpGameWindowWidget, Boolean );
static void placement( LjpGameWindowWidget, Boolean );
static void print_request( const char *msg, XtWidgetGeometry* );

static XtResource resources[] =
{
    {
	XljpNmarginTop,
	XljpCMarginTop,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpGameWindowRec,gamewindow.margin_top),
	XtRImmediate,
	(XtPointer)4
    },
    {
	XljpNmarginBottom,
	XljpCMarginBottom,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpGameWindowRec,gamewindow.margin_bottom),
	XtRImmediate,
	(XtPointer)4
    },
    {
	XljpNmarginWidth,
	XljpCMarginWidth,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpGameWindowRec,gamewindow.margin_width),
	XtRImmediate,
	(XtPointer)0
    },
    {
	XljpNmarginHeight,
	XljpCMarginHeight,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpGameWindowRec,gamewindow.margin_height),
	XtRImmediate,
	(XtPointer)0
    }
};
	  
LjpGameWindowClassRec ljpGameWindowClassRec =
{
   {
      (WidgetClass) &compositeClassRec,         /* superclass         */
      "LjpGameWindow",                         	/* class_name         */
      sizeof(LjpGameWindowRec),         	/* widget_size        */
      NULL,                          		/* class_initialize   */
      NULL,                            		/* class_part_initiali*/
      FALSE,                                    /* class_inited       */
      hook_initialize,                          /* initialize         */
      NULL,                                     /* initialize_hook    */
      XtInheritRealize,                         /* realize            */
      NULL, 	                                /* actions            */
      0,                                        /* num_actions        */
      resources,                                /* resources          */
      XtNumber(resources),                      /* num_resources      */
      NULLQUARK,                                /* xrm_class          */
      TRUE,                                     /* compress_motion    */
      XtExposeCompressMaximal,                  /* compress_exposure  */
      TRUE,                                     /* compress_enterleave*/
      FALSE,                                    /* visible_interest   */
      NULL,					/* destroy            */
      hook_resize,                              /* resize             */
      NULL,	                 		/* expose             */
      hook_setValues,                           /* set_values         */
      NULL,                                     /* set_values_hook    */
      XtInheritSetValuesAlmost,        		/* set_values_almost  */
      NULL,                                     /* get_values_hook    */
      NULL,                                     /* accept_focus       */
      XtVersion,                                /* version            */
      NULL,                                     /* callback_private   */
      NULL, 	    	                        /* tm_table           */
      hook_queryGeometry,             		/* query_geometry     */
      XtInheritDisplayAccelerator,              /* display accel      */
      NULL,                                     /* extension          */
    },
    /* Composite class. */
    {
    /* geometry_manager   */    hook_geometryManager,
	hook_changeManaged,			/* change_managed	*/
	hook_insertChild,			/* insert_child		*/
    /* delete_child	  */	XtInheritDeleteChild,
    /* extension          */	NULL
    },
    /* GameWindow. */
    {
	NULL                                    /* extension          */
    }
};

WidgetClass ljpGameWindowWidgetClass = (WidgetClass)&ljpGameWindowClassRec;

static void hook_initialize( Widget w, Widget neww, ArgList args,
    Cardinal *num_args )
{
    LjpGameWindowWidget gw = (LjpGameWindowWidget)w;
    LjpGameWindowWidget new_gw = (LjpGameWindowWidget)neww;
    Dimension defaultWidth=0, defaultHeight=0;

    /*
    fprintf( stdout, "LjpGameWindow 0x%x hook initialize\n", neww );
    */

    if( neww->core.width <= 0 )
	neww->core.width = 100;

    if( neww->core.height <= 0 )
	neww->core.height = 50;

    /* Set default size (possibly based on initial children). */
    layout( gw, False );
}

static Boolean hook_setValues( Widget oldw, Widget w, Widget neww,
    ArgList args, Cardinal *num_args )
{
    LjpGameWindowWidget old_gw = (LjpGameWindowWidget)oldw;
    LjpGameWindowWidget new_gw = (LjpGameWindowWidget)neww;
    Boolean do_redisplay = False;

    /*
    fprintf( stdout, "LjpGameWindow setValues\n" );
    */

    if( new_gw->gamewindow.margin_height !=
	    old_gw->gamewindow.margin_height ||
        new_gw->gamewindow.margin_width != old_gw->gamewindow.margin_width ||
	new_gw->gamewindow.margin_top != old_gw->gamewindow.margin_top ||
	new_gw->gamewindow.margin_bottom != old_gw->gamewindow.margin_bottom )
    {
	layout( new_gw, True );
    }

    return False;
}

static void hook_resize( Widget w )
{
    LjpGameWindowWidget gw = (LjpGameWindowWidget)w;

    /*
    fprintf( stdout, "LjpGameWindow hook resize\n" );
    fprintf( stdout, "LjpGameWindow resized to x%d y%d w%d h%d\n",
	gw->core.x, gw->core.y, gw->core.width, gw->core.height );
    */
    layout( gw, True );
}

static XtGeometryResult hook_queryGeometry( Widget w,
    XtWidgetGeometry *request, XtWidgetGeometry *reply )
{
    int i;
    Dimension nw, nh, nh1, nh2, nh3;
    LjpGameWindowWidget gw = (LjpGameWindowWidget)XtParent(w);
    XtWidgetGeometry desired;
    Widget cw;

    /*
    fprintf( stdout, "LjpGameWindow hook queryGeometry\n" );
    */

    if( request->request_mode == 0 )
	return XtGeometryYes;

    if( gw->composite.num_children < 3 )
	return XtGeometryYes;

    XtQueryGeometry( gw->composite.children[0], NULL, &desired );
    nh1 = desired.height;
    nw = desired.width;
    XtQueryGeometry( gw->composite.children[1], NULL, &desired );
    nh2 = desired.height;
    if( desired.width > nw )
	nw = desired.width;
    XtQueryGeometry( gw->composite.children[2], NULL, &desired );
    nh3 = desired.height;
    if( desired.width > nw )
	nw = desired.width;

    reply->width = nw + (2*gw->gamewindow.margin_width);
    reply->height = nh1+nh2+nh3+(2*gw->gamewindow.margin_height)+
	gw->gamewindow.margin_top+gw->gamewindow.margin_bottom;
    reply->request_mode = CWWidth|CWHeight;

    if( ((request->request_mode&CWWidth)&&(request->width==reply->width)) &&
	((request->request_mode&CWHeight)&&(request->height==reply->height)) )
    {
	return XtGeometryYes;
    }

    if( (reply->width==gw->core.width) &&
	(reply->height==gw->core.height) )
    {
	return XtGeometryNo;
    }

    return XtGeometryAlmost;
}

/*
** Child wants to change its size. We allow any of the children to do this.
*/
static XtGeometryResult hook_geometryManager( Widget cw,
    XtWidgetGeometry *request, XtWidgetGeometry *reply )
{
    Boolean flagAlmost = False;
    Boolean flagRequest = False;
    Dimension widthHold, heightHold, nw, nh, aw, ah;
    LjpGameWindowWidget gw = (LjpGameWindowWidget)XtParent(cw);
    XtGeometryResult result = XtGeometryYes;
    XtWidgetGeometry geomRequest, geomReply;
    XtGeometryMask mask = request->request_mode;

    /*
    fprintf( stdout, "LjpGameWindow hook geometryManager\n" );

    print_request( "child wants", request );
    */

    geomRequest.request_mode = 0;
    geomReply.request_mode = 0;
    reply->request_mode = 0;

    /* Only consider size requests. */
    if( !(mask&(CWWidth|CWHeight)) )
    	return XtGeometryNo;

    /* If child already has request height and width. */
    flagAlmost = True;
    if( (request->request_mode&CWWidth) &&
	(request->width!=gw->core.width) )
    {
	flagAlmost = False;
    }
    if( (request->request_mode&CWHeight) &&
	(request->height!=gw->core.height) )
    {
	flagAlmost = False;
    }
    if( flagAlmost == True )
	return XtGeometryNo;

    flagAlmost = False;

    /*
    ** If this is query only, only query parent.
    */
    if( (mask&XtCWQueryOnly) || flagAlmost )
	geomRequest.request_mode = XtCWQueryOnly;
    else
	geomRequest.request_mode = 0;

    if( request->request_mode & CWWidth )
	aw = request->width;
    else
	aw = cw->core.width;
    if( request->request_mode & CWHeight )
	ah = request->height;
    else
	ah = cw->core.height;
    get_sizes_replace( gw, cw, aw, ah, &nw, &nh );

    if( mask & CWWidth )
    {
	if( request->width != nw )
	{
	    flagRequest = True;
	    widthHold = cw->core.width;
	    cw->core.width = request->width;
	    geomRequest.request_mode |= CWWidth;
	    geomRequest.width = nw;
	}
    }

    if( mask & CWHeight )
    {
	if( request->height != nh )
	{
	    flagRequest = True;
	    heightHold = cw->core.height;
	    cw->core.height = request->height;
	    geomRequest.request_mode |= CWHeight;
	    geomRequest.width = nh;
	}
    }

    if( flagRequest )
    {
	/*
	print_request( "request", &geomRequest );
	*/

	result = XtMakeGeometryRequest( (Widget)gw, &geomRequest,
	    &geomReply );

	if( result == XtGeometryNo )
	{
	    /*
	    print_request( "reply no", &geomReply );
	    */
	    cw->core.width = widthHold;
	    cw->core.height = heightHold;
	    return result;
	}
	/*
	else
	    print_request( "reply yes", &geomReply );
	*/

	if( result == XtGeometryAlmost )
	{
	    reply->request_mode = mask;

	    /*
	    fprintf( stdout, "LjpGameWindow request almost\n" );
	    */
	    if( mask & CWWidth )
	    {
		if( geomRequest.request_mode & CWWidth )
		{
		    if( geomRequest.width == reply->width )
			reply->width = cw->core.width;
		    else
		    {
			reply->width = cw->core.width +
			    (reply->width-geomRequest.width);
		    }
		}
		else
		    reply->width = cw->core.width;
		cw->core.width = widthHold;
	    }

	    if( mask & CWHeight )
	    {
		if( geomRequest.request_mode & CWHeight )
		{
		    if( geomRequest.height == reply->height )
			reply->height = cw->core.height;
		    else
		    {
			reply->height = cw->core.height +
			    (reply->height-geomRequest.height);
		    }
		}
		else
		    reply->height = cw->core.height;
		cw->core.height = heightHold;
	    }
	}

	return XtGeometryAlmost;
    }

    if( flagAlmost )
	result = XtGeometryAlmost;

    if( mask & XtCWQueryOnly )
    {
	if( mask & CWWidth )
	    cw->core.width = widthHold;
	if( mask & CWHeight )
	    cw->core.height = heightHold;
    }
    else
	layout( gw, True );

    return result;
}

static void hook_changeManaged( Widget w )
{
    Boolean make_request = False;
    Dimension nw, nh;
    XtGeometryResult result = XtGeometryYes;
    XtWidgetGeometry geomRequest, geomReply;
    LjpGameWindowWidget gw = (LjpGameWindowWidget)w;

    /*
    fprintf( stdout, "LjpGameWindow hook changeManaged %d children\n",
       gw->composite.num_children );
    */

    geomRequest.request_mode = 0;

    get_sizes( gw, &nw, &nh );

    if( gw->core.width != nw )
    {
	make_request = True;
	geomRequest.request_mode |= CWWidth;
	geomRequest.width = nw;
    }

    if( gw->core.height != nh )
    {
	make_request = True;
	geomRequest.request_mode |= CWHeight;
	geomRequest.height = nh;
    }

    if( make_request )
    {
	/*
	print_request( "changeManaged request", &geomRequest );
	*/
	result = XtMakeGeometryRequest( w, &geomRequest, &geomReply );
	if( result != XtGeometryYes )
	{
	    if( result == XtGeometryAlmost )
	    {
		result = XtMakeGeometryRequest( w, &geomReply, NULL );
		print_request( "changeManaged reply almost", &geomReply );
	    }
	    else
		print_request( "changeManaged reply no", &geomReply );
	}
	/*
	else
	    print_request( "changeManaged reply yes", &geomReply );
	*/
    }

    {
	Widget cw, bw;

	bw = gw->composite.children[1];

	cw = gw->composite.children[0];
	if( cw->core.width != bw->core.width )
	{
	    XtConfigureWidget( cw, cw->core.x, cw->core.y,
		bw->core.width, cw->core.height, cw->core.border_width );
	}

	cw = gw->composite.children[2];
	if( cw->core.width != bw->core.width )
	{
	    /*
	    fputs( "LjpGameWindow configure lower\n", stdout );
	    */
	    XtConfigureWidget( cw, cw->core.x, cw->core.y,
		bw->core.width, cw->core.height, cw->core.border_width );
	}
    }

    layout( gw, True );
}

static void hook_insertChild( Widget w, ArgList args, Cardinal *num_args )
{
    LjpGameWindowWidget gw = (LjpGameWindowWidget)XtParent(w);
    CompositeWidgetClass cc;
    int count = gw->composite.num_children;

    /*
    fprintf( stdout, "LjpGameWindow hook insertChild\n" );
    */

    if( gw->composite.num_children > 3 )
    {
	XtAppError( XtWidgetToApplicationContext(w),
	    "LjpGameWindow: Can only manage three widgets." );
    }

    cc = (CompositeWidgetClass)compositeWidgetClass;
    /*
    (*cc->composite_class.insert_child)(w,args,num_args);
    */
    (*cc->composite_class.insert_child)(w);

    /*
    layout( gw, True );
    */
}

static void layout( LjpGameWindowWidget gw, Boolean doit )
{
    int i;
    Widget pw, cw;
    Position nx, ny;
    Dimension rw, rh;

    /*
    fprintf( stdout, "LjpGameWindow layout\n" );
    */

    if( !XtIsRealized((Widget)gw) )
	doit = False;

    pw = XtParent( (Widget)gw );

    /*
    ** Update GameWindow position.
    */

    placement( gw, doit );

    /*
    ** Update child positions. Parent window may be much larger than
    ** minimum. Act as if there were a minimum sized GameWindow in
    ** the center of the parent. GameWindows are not supposed to
    ** expand to fit the parent.
    */

    get_sizes( gw, &rw, &rh );
    rw -= 2*gw->gamewindow.margin_width;
    rh -= 2*gw->gamewindow.margin_height;

    nx = (gw->core.width-rw)/2;
    ny = (gw->core.height-rh)/2;

    if( gw->composite.num_children >= 1 )
    {
	cw = gw->composite.children[0];
	XtMoveWidget( cw, nx, ny );
	ny += cw->core.height;
    }
    ny += gw->gamewindow.margin_top;

    if( gw->composite.num_children >= 2 )
    {
	cw = gw->composite.children[1];
	XtMoveWidget( cw, nx, ny );
	ny += cw->core.height;
    }
    ny += gw->gamewindow.margin_bottom;

    if( gw->composite.num_children >= 3 )
    {
	cw = gw->composite.children[2];
	XtMoveWidget( cw, nx, ny );
    }
}

static void get_sizes( LjpGameWindowWidget gw, Dimension *w, Dimension *h )
{
    Dimension nh1, nh2, nh3, nw;

    nh1 = nh2 = nh3 = 0;
    nw = 0;

    if( gw->composite.num_children >= 1 )
    {
	nh1 = gw->composite.children[0]->core.height+
	    (2*gw->composite.children[0]->core.border_width);
	nw = gw->composite.children[0]->core.width;
	/*
	fprintf( stdout, "LjpGameWindow getSizes top row w%dh%d\n",
	    gw->composite.children[0]->core.width,
	    gw->composite.children[0]->core.height );
	*/
    }
    if( gw->composite.num_children >= 2 )
    {
	if( gw->composite.children[1]->core.width > nw )
	    nw = gw->composite.children[1]->core.width;
	nh2 = gw->composite.children[1]->core.height+
	    (2*gw->composite.children[1]->core.border_width);
    }
    if( gw->composite.num_children >= 2 )
    {
	if( gw->composite.children[2]->core.width > nw )
	    nw = gw->composite.children[2]->core.width;
	nh3 = gw->composite.children[2]->core.height+
	    (2*gw->composite.children[2]->core.border_width);
    }

    nw += (2*gw->gamewindow.margin_width);

    if( w != NULL )
	*w = nw;

    nh1 = nh1+nh2+nh3+(2*gw->gamewindow.margin_height)+
	gw->gamewindow.margin_top+gw->gamewindow.margin_bottom;
    if( h != NULL )
	*h = nh1;
}

static void get_sizes_replace( LjpGameWindowWidget gw, Widget tw,
    Dimension width, Dimension height, Dimension *nw, Dimension *nh )
{
    Dimension nh1, nh2, nh3;

    if( gw->composite.children[0] == tw )
	nh1 = height;
    else
	nh1 = gw->composite.children[0]->core.height;
    nh1 += gw->composite.children[0]->core.border_width;

    if( gw->composite.children[1] == tw )
    {
	*nw = width;
	nh2 = height;
    }
    else
    {
	*nw = gw->composite.children[1]->core.width;
	nh2 = gw->composite.children[1]->core.height;
    }
    nh2 += gw->composite.children[1]->core.border_width;

    if( gw->composite.children[1] == tw )
	nh3 = height;
    else
	nh3 = gw->composite.children[2]->core.height;
    nh3 += gw->composite.children[2]->core.border_width;

    *nw += (2*gw->gamewindow.margin_width);
    *nh = nh1+nh2+nh3+(2*gw->gamewindow.margin_height)+
	gw->gamewindow.margin_top+gw->gamewindow.margin_bottom;
}

static void placement( LjpGameWindowWidget gw, Boolean doit )
{
    Dimension nw, nh, rw, rh;
    Widget pw = XtParent(gw);

    /*
    fprintf( stdout, "LjpGameWindow before w%d h%d\n",
	gw->core.width, gw->core.height );
    */

    /*
    ** Children determine minimum. Parent determines size.
    */

    get_sizes( gw, &rw, &rh );

    nw = rw;
    nh = rh;

    if( pw->core.width > nw )
	nw = pw->core.width;
    if( pw->core.height > nh )
	nh = pw->core.height;

    if( doit == False )
    {
	gw->core.width = nw;
	gw->core.height = nh;
	return;
    }

    /*
    fprintf( stdout, "LjpGameWindow want w%d h%d\n",
	nw, nh );
    */

    /*
    ** Make width request.
    */
    if( nw != gw->core.width )
    {
	XtGeometryResult rc;
	XtWidgetGeometry request, reply;

	request.request_mode = CWWidth;
	request.width = nw;
	rc = XtMakeGeometryRequest( (Widget)gw, &request, &reply );
	/*
	fprintf( stdout, "-= geom width request %d\n", rc );
	*/
	if( rc != XtGeometryYes )
	{
	    fputs( "-= geom width request denied\n", stdout );
	}
    }

    /*
    ** Make height request.
    */
    if( nh != gw->core.height )
    {
	XtGeometryResult rc;
	XtWidgetGeometry request, reply;

	request.request_mode = CWHeight;
	request.height = nh;
	rc = XtMakeGeometryRequest( (Widget)gw, &request, &reply );
	/*
	fprintf( stdout, "-= geom height request %d\n", rc );
	*/
	if( rc != XtGeometryYes )
	{
	    fputs( "-= geom height request denied\n", stdout );
	}
    }

    /*
    fprintf( stdout, "LjpGameWindow w%d h%d\n",
	gw->core.width, gw->core.height );
    */
}

static void print_request( const char *msg, XtWidgetGeometry *rec )
{
    fprintf( stdout,
	"LjpGameWindow %s w%d%c h%d%c\n",
	msg,
	rec->width, (rec->request_mode&CWWidth)?'*':' ',
	rec->height, (rec->request_mode&CWHeight)?'*':' ' );
}
