/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

#include <stdio.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include "LabelP.h"

/*
** Widget hooks.
*/
static void hook_initialize( Widget, Widget, ArgList, Cardinal* );
static void hook_classPartInitialize( WidgetClass );
static void hook_destroy( Widget w );
static void hook_resize( Widget w );
static void hook_redisplay( Widget, XEvent*, Region );
static Boolean hook_setValues( Widget, Widget, Widget, ArgList, Cardinal* );
static XtGeometryResult hook_queryGeometry( Widget, XtWidgetGeometry*, 
    XtWidgetGeometry* );

static void hook_drawLabel( Widget w, Pixel );
static void mkGC( LjpLabelWidget );

static XtResource resources[] =
{
    {
	XljpNlabel,
	XljpCLabel,
	XtRString,
	sizeof(char*),
	XtOffsetOf(LjpLabelRec,label.label),
	XtRImmediate,
	NULL
    },
    {
	XljpNhalignment,
	XljpCHAlignment,
	XtRInt,
	sizeof(enum XljpAlignment),
	XtOffsetOf(LjpLabelRec,label.halignment),
	XtRImmediate,
	(XtPointer)XljpAlignCenter
    }
};
	  
LjpLabelClassRec ljpLabelClassRec =
{
   {
      (WidgetClass) &ljpPrimitiveClassRec,	/* superclass         */
      "LjpLabel",                            	/* class_name         */
      sizeof(LjpLabelRec),                   	/* widget_size        */
      NULL,                          		/* class_initialize   */
      hook_classPartInitialize,			/* class_part_initiali*/
      FALSE,                                    /* class_inited       */
      hook_initialize,                          /* initialize         */
      NULL,                                     /* initialize_hook    */
      XtInheritRealize,                         /* realize            */
      NULL,					/* actions            */
      0,					/* num_actions        */
      resources,                                /* resources          */
      XtNumber(resources),                      /* num_resources      */
      NULLQUARK,                                /* xrm_class          */
      TRUE,                                     /* compress_motion    */
      XtExposeCompressMaximal,                  /* compress_exposure  */
      TRUE,                                     /* compress_enterleave*/
      FALSE,                                    /* visible_interest   */
      hook_destroy,                             /* destroy            */
      hook_resize,                              /* resize             */
      hook_redisplay,                 		/* expose             */
      hook_setValues,                           /* set_values         */
      NULL,                                     /* set_values_hook    */
      NULL,                            		/* set_values_almost  */
      NULL,                                     /* get_values_hook    */
      NULL,                                     /* accept_focus       */
      XtVersion,                                /* version            */
      NULL,                                     /* callback_private   */
      NULL,					/* tm_table           */
      hook_queryGeometry,             		/* query_geometry     */
      XtInheritDisplayAccelerator,              /* display accel      */
      NULL,                                     /* extension          */
    },
    /* Primitve class. */
    {
	XljpInheritDrawFrame,			/* draw_frame.		*/
	XljpInheritCalcInterior,		/* calc_interior.	*/
	NULL                                    /* Extension.		*/
    },
    /* Label. */
    {
	hook_drawLabel,				/* Draw label.		*/
	NULL                                    /* Extension.		*/
    }
};

WidgetClass ljpLabelWidgetClass = (WidgetClass)&ljpLabelClassRec;

static void hook_initialize( Widget w, Widget neww, ArgList args,
    Cardinal *num_args )
{
    LjpLabelWidget lw = (LjpLabelWidget)w;
    LjpLabelWidget new_lw = (LjpLabelWidget)neww;
    LjpLabelWidgetClass bc = (LjpLabelWidgetClass)XtClass(neww);
    Dimension defaultWidth=0, defaultHeight=0;
    XGCValues values;
    XtGCMask mask;

    /*
    fprintf( stdout, "LjpLabel %x hook init\n", (unsigned int)neww );
    */

    if( new_lw->label.label != NULL )
    {
	int direction, ascent, descent;
	XCharStruct rec;

	new_lw->label.label = strdup(new_lw->label.label);

	/*
	fprintf( stdout, "LjpLabel %x hook init \"%s\" fid %d\n",
	    new_lw, new_lw->label.label, new_lw->primitive.font->fid );
	*/
	XQueryTextExtents( XtDisplay(new_lw), new_lw->primitive.font->fid,
	    new_lw->label.label, strlen(new_lw->label.label),
	    &direction, &ascent, &descent, &rec );

	defaultHeight = ascent+descent;
	defaultWidth = rec.width;
    }
    else
    {
	defaultHeight = 5;
	defaultWidth = 5;
    }

    {
	Position xl, yt, xr, yb;

	bc->primitive_class.calc_interior( neww, &xl, &yt, &xr, &yb,
	    NULL, NULL );
	defaultWidth += (xl+xr);
	defaultHeight += (yt+yb);

	/*
	** If the app did not set a width or height then ignore the
	** default set by the Primitive class and use one calculated
	** here.
	*/
	if( lw->core.width <= 0 )
	    new_lw->core.width = defaultWidth;
	if( lw->core.height <= 0 )
	    new_lw->core.height = defaultHeight;
	/*
	fprintf( stdout, "[LjpLabel] %x hook init w%d h%d, w%d h%d\n",
	    new_lw, lw->core.width, lw->core.height,
	    new_lw->core.width, new_lw->core.height );
	*/
    }

    {
	XGCValues values;
	XtGCMask mask;

	mask = GCFillStyle;
	values.fill_style = FillTiled;
	if( new_lw->core.background_pixmap != XtUnspecifiedPixmap )
	{
	    mask |= GCTile;
	    values.tile = new_lw->core.background_pixmap;
	}
	new_lw->label.gcTile = XCreateGC( XtDisplay(new_lw),
	    DefaultRootWindow(XtDisplay(new_lw)),
	    mask, &values );
    }
}

static void hook_classPartInitialize( WidgetClass wc )
{
    LjpLabelWidgetClass lc = (LjpLabelWidgetClass)wc;
    LjpLabelWidgetClass sc = (LjpLabelWidgetClass)wc->core_class.superclass;

    if( lc->label_class.draw_label == XljpInheritDrawLabel )
	lc->label_class.draw_label = sc->label_class.draw_label;
}

static void hook_destroy( Widget w )
{
    LjpLabelWidget lw = (LjpLabelWidget)w;

    if( lw->label.label != NULL )
	free( lw->label.label );
}

static void hook_resize( Widget w )
{
    LjpLabelWidget lw = (LjpLabelWidget)w;

    hook_drawLabel( w, lw->core.background_pixel );
}

static void hook_redisplay( Widget w, XEvent *event, Region region )
{
    LjpLabelWidget lw = (LjpLabelWidget)w;

    hook_drawLabel( w, lw->core.background_pixel );
}

static Boolean hook_setValues( Widget oldw, Widget w, Widget neww,
    ArgList args, Cardinal *num_args )
{
    LjpLabelWidget old_dw = (LjpLabelWidget)oldw;
    LjpLabelWidget new_dw = (LjpLabelWidget)neww;
    Boolean redisplay = False;

    /*
    fprintf( stdout, "LjpLabel %x hook setVal\n", (unsigned int)neww );
    */

    if( new_dw->label.label != old_dw->label.label )
    {
	if( old_dw->label.label != NULL )
	    free( old_dw->label.label );
	new_dw->label.label = strdup(new_dw->label.label);
	redisplay = True;
    }

    if( (new_dw->core.background_pixmap!=old_dw->core.background_pixmap) )
    {
	redisplay = True;
	mkGC( new_dw );
    }

    return redisplay;
}

static XtGeometryResult hook_queryGeometry( Widget w,
    XtWidgetGeometry *request, XtWidgetGeometry *reply )
{
    LjpLabelWidget dw = (LjpLabelWidget)w;

    reply->request_mode = CWWidth|CWHeight;

    if( ((request->request_mode&CWWidth) &&
	request->width==reply->width) &&
	((request->request_mode & CWHeight) &&
	request->height==reply->height) )
    {
	return XtGeometryYes;
    }

    if( (reply->width==dw->core.width) &&
	(reply->height==dw->core.height) )
    {
	return XtGeometryNo; 
    }

    return XtGeometryAlmost; 
}

static void hook_drawLabel( Widget aw, Pixel bg )
{
    LjpLabelWidget lw = (LjpLabelWidget)aw;
    LjpLabelWidgetClass bc = (LjpLabelWidgetClass)XtClass(aw);
    Dimension gutter;
    Position x, y;
    Dimension w, h;

    /*
    fprintf( stdout, "[LjpLabel] draw ml%d mr%d ix%d\n",
	lw->primitive.margin_left, lw->primitive.margin_right, x );
    */

    /*
    fprintf( stdout, "LjpLabel draw shadow %d\n", lw->primitive.shadow_type );
    */
    bc->primitive_class.draw_frame( aw );

    /*
    ** Find the area available for the label label (i.e. exclude the
    ** outer margin, frame and inner margin).
    */
    bc->primitive_class.calc_interior( aw, &x, &y, NULL, NULL, &w, &h );

    /*
    ** Clear area for text. Include the inner margins (which are not part of
    ** the interior area calculated above).
    */
    if( lw->core.background_pixmap != XtUnspecifiedPixmap )
    {
	XFillRectangle( XtDisplay(lw), XtWindow(aw), lw->label.gcTile,
	    x-lw->primitive.margin_left,
	    y-lw->primitive.margin_top,
	    w+lw->primitive.margin_left+lw->primitive.margin_right,
	    h+lw->primitive.margin_top+lw->primitive.margin_bottom );
    }
    else
    {
	if( bg != lw->core.background_pixel )
	    XSetForeground( XtDisplay(lw), lw->primitive.gcClear, bg );
	XFillRectangle( XtDisplay(lw), XtWindow(aw), lw->primitive.gcClear,
	    x-lw->primitive.margin_left,
	    y-lw->primitive.margin_top,
	    w+lw->primitive.margin_left+lw->primitive.margin_right,
	    h+lw->primitive.margin_top+lw->primitive.margin_bottom );
	if( bg != lw->core.background_pixel )
	{
	    XSetForeground( XtDisplay(lw), lw->primitive.gcClear,
		lw->core.background_pixel );
	}
    }

    if( lw->label.label != NULL )
    {
	int x_base, y_base;
	int direction, ascent, descent;
	XCharStruct rec;

	XQueryTextExtents( XtDisplay(aw), lw->primitive.font->fid,
	    lw->label.label, strlen(lw->label.label),
	    &direction, &ascent, &descent, &rec );
	/*
	fprintf( stdout, "LjpLabel draw fid %d %d\n",
	    lw->primitive.font->fid,
	    lw->primitive.gcTop->values.font );
	*/

	/*
	** Position the text.
	*/

	y_base = y+((h-(ascent+descent))/2)+ascent;

	if( lw->label.halignment == XljpAlignCenter )
	    x_base = x+(w-rec.width)/2;
	else if( lw->label.halignment == XljpAlignBeginning )
	    x_base = x;
	else /* XljpAlignEnd */
	    x_base = x+(w-rec.width);

	XSetForeground( XtDisplay(lw), lw->primitive.gcTop,
	    lw->primitive.pixelForeground );
	XDrawString( XtDisplay(aw), XtWindow(aw), lw->primitive.gcTop,
	    x_base, y_base, lw->label.label,
	    strlen(lw->label.label) );
	XSetForeground( XtDisplay(lw), lw->primitive.gcTop,
	    lw->primitive.pixelTopShadow );
    }
}

static void mkGC( LjpLabelWidget lw )
{
    XGCValues values;
    XtGCMask mask;

    mask = GCFillStyle;
    values.fill_style = FillTiled;
    if( lw->core.background_pixmap != XtUnspecifiedPixmap )
    {
	mask |= GCTile;
	values.tile = lw->core.background_pixmap;
    }

    XChangeGC( XtDisplay(lw), lw->label.gcTile, mask, &values );
}
