/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <stdio.h>
#include "LarryP.h"
#include "ButtonP.h"

/*
** FUTURE. Size of location buttons. Space between buttons is dynamic.
** Make this settable by the application.
*/
#define LARRY_LWIDTH  20
#define LARRY_LHEIGHT 20

#define offset(field) XtOffsetOf(LjpLarryRec, field)

static XtResource resources[] =
{
    {
	XljpNtopShadowColor,
	XljpCTopShadowColor,
	XtRPixel,
	sizeof(Pixel),
	offset(larry.topShadowColor),
	XtRString,
	XtDefaultBackground
    },
    {
	XljpNbottomShadowColor,
	XljpCBottomShadowColor,
	XtRPixel,
	sizeof(Pixel),
	offset(larry.bottomShadowColor),
	XtRString,
	XtDefaultForeground
    },
    {
	XljpNlocation, 
	XljpCLocation, 
	XtRInt, 
	sizeof(XljpLocation),
	offset(larry.location), 
	XtRImmediate, 
	(XtPointer)XljpLocationIRight
    },
    {
	XljpNbuttonWidget,
	XljpCButtonWidget,
	XtRWidget,
	sizeof(Widget),
	offset(larry.button_widget),
	XtRImmediate,
	NULL
    }
};

/* Methods. */

static void hook_initialize( Widget, Widget, ArgList, Cardinal* );
static Boolean hook_setValues();
static void hook_redisplay();
static void hook_resize();
static void hook_changeManaged( Widget );
static XtGeometryResult hook_geometryManager( Widget, XtWidgetGeometry*,
    XtWidgetGeometry* );
static XtGeometryResult hook_queryGeometry();
static void hook_destroy( Widget );
static void hook_insertChild( Widget, ArgList, Cardinal* );

static void action_leave( Widget, XEvent* );
static void action_press( Widget, XEvent* );
static void action_release( Widget, XEvent* );

static void mkClearGC( LjpLarryWidget w );
static void mkCopyGC( LjpLarryWidget w );
static void layout( Widget, Boolean );
static void placement( LjpLarryWidget w, Widget pw, Boolean );
static int button_release( LjpLarryWidget );
static int arrow_left( LjpLarryWidget, GC, GC, int, int );
static int arrow_ileft( LjpLarryWidget, GC, GC, int, int );
static int arrow_right( LjpLarryWidget, GC, GC, int, int );
static int arrow_iright( LjpLarryWidget, GC, GC, int, int );

static char hook_translations[] =
    "<Leave>:    leave() \n\
    <Btn1Down>: press() \n\
    <Btn1Up>:    release()";

static XtActionsRec hook_actions[] =
{
    { "leave", action_leave },
    { "press", action_press },
    { "release", action_release }
};

LjpLarryClassRec ljpLjpLarryClassRec =
{
    {
    /* core_class fields */
    /* superclass	  	 */ (WidgetClass) &overrideShellClassRec,
    /* class_name	  	 */ "LjpLarry",
    /* widget_size	  	 */ sizeof(LjpLarryRec),
    /* class_initialize   	 */ NULL,
    /* class_part_initialize	 */ NULL,
    /* class_inited       	 */ FALSE,
    /* initialize	  	 */ hook_initialize,
    /* initialize_hook		 */ NULL,
    /* realize		  	 */ XtInheritRealize,
    /* actions		  	 */ hook_actions,
    /* num_actions	  	 */ XtNumber(hook_actions),
    /* resources	  	 */ resources,
    /* num_resources	  	 */ XtNumber(resources),
    /* xrm_class	  	 */ NULLQUARK,
    /* compress_motion	  	 */ TRUE,
    /* compress_exposure  	 */ XtExposeCompressMultiple,
    /* compress_enterleave	 */ TRUE,
    /* visible_interest	  	 */ FALSE,
    /* destroy		  	 */ NULL,
    /* resize		  	 */ hook_resize,
    /* expose		  	 */ hook_redisplay,
    /* set_values	  	 */ hook_setValues,
    /* set_values_hook		 */ NULL,
    /* set_values_almost	 */ XtInheritSetValuesAlmost,
    /* get_values_hook		 */ NULL,
    /* accept_focus	 	 */ NULL,
    /* version			 */ XtVersion,
    /* callback_private   	 */ NULL,
    /* tm_table		   	 */ hook_translations,
    /* query_geometry		 */ hook_queryGeometry,
    /* display_accelerator       */ XtInheritDisplayAccelerator,
    /* extension                 */ NULL
    },
/* composite_class fields */
    {
    /* geometry_manager   */    hook_geometryManager,
    /* change_managed     */    hook_changeManaged,
    /* insert_child	  */	hook_insertChild,
    /* delete_child	  */	XtInheritDeleteChild,
    /* extension          */	NULL
    },
/* shell_class fields */
    {
	/* extension */		NULL
    },
#if 0
/* wm_class fields */
    {
	/* extension */		NULL
    },
/* vendor_class fields */
    {
	/* extension */		NULL
    },
#endif
/* override_class fields */
    {
	/* extension */		NULL
    },
/* larry class fields. */
    {
    /* dummy_field               */ 0,
    },
};

WidgetClass ljpLarryWidgetClass = (WidgetClass)&ljpLjpLarryClassRec;

static void hook_initialize( Widget treq, Widget tnew,
    ArgList args, Cardinal *num_args )
{
    LjpLarryWidget new = (LjpLarryWidget)tnew;
    LjpLarryWidget wreq = (LjpLarryWidget)treq;

/*
    fprintf( stdout, "LjpLarry hook init\n" );
*/

    /*
    ** Just big enough to fit the default arrows and a zero size
    ** child.
    */
    if( wreq->core.width == 0 )
	new->core.width = 80;
    else
	new->core.width = wreq->core.width;

    if( wreq->core.height == 0 )
	new->core.height = 40;
    else
	new->core.height = wreq->core.height;

    /*
    ** Leave these until layout() can compute them.
    */
    new->larry.x_space = 0;
    new->larry.y_popdown = 0;
    new->larry.y_arrows = 20;

    /*
    **
    */
    {
	Widget pw = XtParent(tnew);

	if( XtIsRealized(pw) )
	{
	    placement( new, pw, False );
	}
    }

    new->larry.buttonArmed = XljpLocationNull;

    mkCopyGC( new);
    mkClearGC( new);
}

static void hook_redisplay( Widget w, XExposeEvent *event )
{
    LjpLarryWidget lw = (LjpLarryWidget)w;
    register int x, y;
    unsigned int width, height;
    int x_tmp, y_tmp;

/*
    fprintf( stdout, "LjpLarry hook redisp\n" );
*/

    if( !XtIsRealized((Widget)lw) )
    {
	return;
    }

    /* called from btn-event or expose */
    if( event )
    {
	x = event->x;
	y = event->y; 
	width = event->width;
	height = event->height;
    } 
    else
    {
	/* called because complete redraw */
	x = 0;
	y = 0; 
	width = lw->core.width;
	height = lw->core.height;
    }

    /*fprintf( stdout, "hook -- clear and draw\n" );*/

    x_tmp = lw->larry.x_space;
    y_tmp = lw->larry.y_arrows;

    /* Use arrowColor as background for arrows (fg and bg are highlights). */
    XFillRectangle( XtDisplay(lw), XtWindow(lw), lw->larry.gcBg,
	0, y_tmp, lw->core.width, LARRY_LWIDTH );

    arrow_left( lw, lw->larry.gcTop, lw->larry.gcBottom, x_tmp, y_tmp );

    x_tmp += LARRY_LWIDTH+lw->larry.x_space;
    arrow_ileft( lw, lw->larry.gcTop, lw->larry.gcBottom, x_tmp, y_tmp );

    x_tmp += LARRY_LWIDTH+lw->larry.x_space;
    arrow_iright( lw, lw->larry.gcTop, lw->larry.gcBottom, x_tmp, y_tmp );

    x_tmp += LARRY_LWIDTH+lw->larry.x_space;
    arrow_right( lw, lw->larry.gcTop, lw->larry.gcBottom, x_tmp, y_tmp );
}

static Boolean hook_setValues( Widget current, Widget request, Widget new,
    ArgList args, Cardinal *num_args )
{
    LjpLarryWidget curw = (LjpLarryWidget)current;
    LjpLarryWidget neww = (LjpLarryWidget)new;
    Boolean do_redisplay = False;
    Boolean do_bg = False;
    Boolean do_sc = False;

/*
    fprintf( stdout, "LjpLarry hook setValues\n" );
*/

    if( curw->core.background_pixel != neww->core.background_pixel )
	do_bg = True;
    if( curw->larry.topShadowColor != neww->larry.topShadowColor )
	do_sc = True;
    if( curw->larry.bottomShadowColor != neww->larry.bottomShadowColor )
	do_sc = True;

    if( do_bg == True )
    {
	XtReleaseGC( current, curw->larry.gcBg );
	mkClearGC( neww );
	do_redisplay = True;
    }

    if( do_sc )
    {
	XtReleaseGC( current, curw->larry.gcTop );
	XtReleaseGC( current, curw->larry.gcBottom );
	mkCopyGC( neww );
    }

    return do_redisplay;
}

static void hook_destroy( Widget w )
{
    LjpLarryWidget lw = (LjpLarryWidget)w;

    XtReleaseGC( w, lw->larry.gcTop );
    XtReleaseGC( w, lw->larry.gcBottom );
    XtReleaseGC( w, lw->larry.gcBg );
}

static void hook_resize( Widget w )
{
    LjpLarryWidget lw = (LjpLarryWidget)w;

/*
    fprintf( stdout, "Ljplarry hook resize\n" );
*/
    layout( (Widget)lw, True );
}

static XtGeometryResult hook_queryGeometry( Widget w,
    XtWidgetGeometry *proposed,  XtWidgetGeometry *answer )
{
    LjpLarryWidget cw = (LjpLarryWidget)w;

/*
    fprintf( stdout, "hook queryGeometry\n" );
*/
    answer->request_mode = CWWidth|CWHeight;

    /* initial width and height */
    answer->width = cw->core.width;
    answer->height = cw->core.height;

    if( ((proposed->request_mode&(CWWidth|CWHeight))==(CWWidth|CWHeight))
	&& (proposed->width==answer->width)
	&& (proposed->height == answer->height) )
    {
        return XtGeometryYes;
    }
    else if( answer->width == cw->core.width &&
	answer->height == cw->core.height)
    {
	return XtGeometryNo;
    }
    else
        return XtGeometryAlmost;
}

static XtGeometryResult hook_geometryManager( Widget cw,
    XtWidgetGeometry *request, XtWidgetGeometry *reply )
{
    LjpLarryWidget lw = (LjpLarryWidget)XtParent(cw);
    XtWidgetGeometry allowed;

/*
    fprintf( stdout, "hook geometryManager\n" );
*/

    /* Only consider size requests. */
    if (request->request_mode & ~(XtCWQueryOnly|CWWidth|CWHeight))
    	return XtGeometryNo;

    /*
    ** Button child is handled separately.
    */
    if( cw == lw->larry.button_widget )
    {
	return XtGeometryNo;
    }

    /*
    ** FUTURE. Refuse to allow the child to grow its width beyond that
    ** of the parent of the Larry widget.
    */
    if( request->request_mode & CWWidth )
    	allowed.width = request->width;
    else
    	allowed.width = cw->core.width;

    /*
    ** FUTURE. Refuse to allow the child to grow its height beyond that
    ** of the parent of the Larry widget.
    */
    if( request->request_mode & CWWidth )
    if (request->request_mode & CWHeight)
    	allowed.height = request->height;
    else
    	allowed.height = cw->core.height;

    /* Nothing to do. */
    if( (allowed.width==cw->core.width) && (allowed.height==cw->core.height) )
    	return XtGeometryNo;

    if( !(request->request_mode&XtCWQueryOnly) )
    {
	layout( (Widget)lw, False );
    }

    return XtGeometryYes;
}

static void hook_changeManaged( Widget w )
{
    LjpLarryWidget lw = (LjpLarryWidget)w;

/*
    fprintf( stdout, "LjpLarry hook changeManaged\n" );
*/
    layout( (Widget)lw, True );
}

static void hook_insertChild( Widget w, ArgList args, Cardinal *num_args )
{
    Boolean flagButton = False;
    Boolean flagError = False;
    LjpLarryWidget lw = (LjpLarryWidget)XtParent(w);
    CompositeWidgetClass sc;
    int count = lw->composite.num_children;

/*
    fprintf( stdout, "LjpLarry hook insertChild\n" );
*/

    if( strcmp(w->core.widget_class->core_class.class_name,"LjpButton") == 0 )
    {
	/*
	fputs( "LjpLarry inserting an LjpButton\n", stdout );
	*/
	flagButton = True;
    }
    /*
    else
	fputs( "LjpLarry inserting a non LjpButton\n", stdout );
    */

    if( flagButton )
    {
	if( (lw->larry.button_widget!=NULL) && (count>0) )
	{
	    /*
	    fputs( "LjpLarry button and child exist.\n", stdout );
	    */
	    flagError = True;
	}
    }
    else if( (lw->larry.button_widget!=NULL) && (count>1) )
	flagError = True;
    else if( (lw->larry.button_widget==NULL) && (count>0) )
	flagError = True;

    if( flagError )
    {
	XtAppError( XtWidgetToApplicationContext((Widget)lw),
	    "LjpLarry: Can only manage one widget" );
    }

    if( flagButton && (lw->larry.button_widget==NULL) )
	lw->larry.button_widget = w;

    sc = (CompositeWidgetClass)compositeWidgetClass;
    /*
    (*sc->composite_class.insert_child)(w,args,num_args);
    */
    (*sc->composite_class.insert_child)(w);
}

static void action_leave( Widget w, XEvent *event )
{
    LjpLarryWidget lw = (LjpLarryWidget)w;

    if( lw->larry.buttonArmed == XljpLocationNull )
	return;

    button_release( lw );
}

static void action_press( Widget w, XEvent *event )
{
    int x_tmp;
    int bx;
    XButtonEvent *be = (XButtonEvent*)event;
    LjpLarryWidget lw = (LjpLarryWidget)w;

    /*
    ** Ignore presses outside of the arrow region.
    */
    if( be->y < lw->larry.y_arrows )
    {
	return;
    }

    /*fprintf( stdout, "action press %d, %d\n", be->x, be->y );*/

    bx = be->x;

    x_tmp = 0+lw->larry.x_space;
    if( (bx>x_tmp) && (bx<(x_tmp+LARRY_LWIDTH)) )
    {
	arrow_left( lw, lw->larry.gcBottom, lw->larry.gcTop,
	    x_tmp, lw->larry.y_arrows );
	lw->larry.buttonArmed = XljpLocationLeft;
    }

    x_tmp += LARRY_LWIDTH+lw->larry.x_space;
    if( (bx>x_tmp) && (bx<(x_tmp+LARRY_LWIDTH)) )
    {
	arrow_ileft( lw, lw->larry.gcBottom, lw->larry.gcTop,
	    x_tmp, lw->larry.y_arrows );
	lw->larry.buttonArmed = XljpLocationILeft;
    }

    x_tmp += LARRY_LWIDTH+lw->larry.x_space;
    if( (bx>x_tmp) && (bx<(x_tmp+LARRY_LWIDTH)) )
    {
	arrow_iright( lw, lw->larry.gcBottom, lw->larry.gcTop,
	    x_tmp, lw->larry.y_arrows );
	lw->larry.buttonArmed = XljpLocationIRight;
    }

    x_tmp += LARRY_LWIDTH+lw->larry.x_space;
    if( (bx>x_tmp) && (bx<(x_tmp+LARRY_LWIDTH)) )
    {
	arrow_right( lw, lw->larry.gcBottom, lw->larry.gcTop,
	    x_tmp, lw->larry.y_arrows );
	lw->larry.buttonArmed = XljpLocationRight;
    }
}

static void action_release( Widget w, XEvent *event )
{
    LjpLarryWidget lw = (LjpLarryWidget)w;

    if( lw->larry.buttonArmed == XljpLocationNull )
	return;

    /* Location arrow. Move window. */
    lw->larry.location = lw->larry.buttonArmed;
    layout( w, True );

    /*fprintf( stdout, "action release\n" );*/

    button_release( lw );
}

static void layout( Widget w, Boolean Doit )
{
    Boolean PartialDoit = Doit;
    LjpLarryWidget lw = (LjpLarryWidget)w;
    Widget pw;
    Widget ch;
    Position nx, ny;
    Dimension nw, nh;

    /*
    fprintf( stdout, "TMP layout %d\n", Doit );
    fprintf( stdout, "TMP   pre-before w%d h%d\n",
	lw->core.width, lw->core.height );
    */

    if( !XtIsRealized((Widget)w) )
    {
	/*
	fputs( "LjpLarry widget not realized\n", stdout );
	*/
	if( Doit )
	    PartialDoit = True;
	Doit = False;
    } 
    pw = XtParent(w);

    if( lw->larry.button_widget == NULL )
    {
	if( lw->composite.num_children < 1 )
	{
	    XtAppError( XtWidgetToApplicationContext((Widget)lw),
		"LjpLarry: Must manage at least one widget." );
	}
	ch = lw->composite.children[0];
    }
    else
    {
	if( lw->composite.num_children < 2 )
	{
	    XtAppError( XtWidgetToApplicationContext((Widget)lw),
		"LjpLarry: Must manage at least one widget." );
	}
	if( lw->larry.button_widget == lw->composite.children[0] )
	    ch = lw->composite.children[1];
	else
	    ch = lw->composite.children[0];
    }

    /*
    fprintf( stdout, "TMP   parent %d,%d w%d h%d\n",
	pw->core.x, pw->core.y, pw->core.width, pw->core.height );
    */

    /*
    ** Set position and size, including requests to parent.
    */
    placement( lw, pw, Doit );

    /*
    ** Update Button widget position.
    */
    if( (Doit||PartialDoit) && (lw->larry.button_widget!=NULL) )
    {
	Widget bw = lw->larry.button_widget;

	if( (bw->core.x!=lw->core.x) ||
	    (bw->core.y!=(lw->core.y+lw->larry.y_popdown)) )
	{
	    XtMoveWidget( bw, 0, lw->larry.y_popdown );
	}

	if( (bw->core.width+bw->core.border_width) != lw->core.width )
	{
	    if( XtIsRealized((Widget)bw) )
	    {
		XtResizeWidget( bw, lw->core.width, bw->core.height,
		    bw->core.border_width );
	    }
	    else
	    {
		bw->core.width = (lw->core.width-(2*bw->core.border_width));
	    }
	}
    }

    /*
    ** Update child size and position.
    */
    if( Doit || PartialDoit )
    {
	/*
	fputs( "LjpLarry handle child\n", stdout );
	*/
	if( (ch->core.width!=lw->core.width) ||
	    (ch->core.height!=(lw->core.height-40)) )
	{
	    /*
	    fprintf( stdout, "LjpLarry resize child to w%d h%d\n",
		lw->core.width, lw->larry.y_popdown );
	    */
	    XtResizeWidget( ch, lw->core.width, lw->larry.y_popdown, 0 );
	}
	if( (ch->core.x!=lw->core.x) ||
	    (ch->core.y!=lw->core.y) )
	{
	    /*
	    fprintf( stdout, "TMP   mv child to %d, %d\n",
		lw->core.x, lw->core.y );
	    */
	    XtMoveWidget( ch, 0, 0 );
	}
    }
}

static void placement( LjpLarryWidget lw, Widget pw, Boolean doit )
{
    Position nx, ny;
    Dimension nw, nh;
    Dimension heightButton = 0;

    if( lw->larry.button_widget != NULL )
    {
	heightButton = lw->larry.button_widget->core.height+
	    (2*lw->larry.button_widget->core.border_width);
    }

    /*
    fprintf( stdout, "LjpLarry layout parent x%d y%d w%d h%d\n",
	pw->core.x, pw->core.y, pw->core.width, pw->core.height );
    */

    nw = lw->core.width;
    nh = lw->core.height;

    /*
    ** Do not exceed parent width.
    */
    if( pw->core.width != 0 )
	nw = (nw<pw->core.width)?nw:pw->core.width;

    /*
    ** Height is always determined from the parent (unless the
    ** parent does not yet have a height).
    */
    if( pw->core.height != 0 )
	nh = pw->core.height-(2*lw->core.border_width);
    else
	nh = 80; /* Enough for buttons. */

    /*
    ** Parent is always the reference Y position with a horizontal
    ** Larry Window. The X position changes with the location.
    */
    ny = pw->core.y;
    nx = pw->core.x;

    /*
    ** Determine X position based on the parent width.
    */
    if( pw->core.height !=0 )
    {
	switch( lw->larry.location )
	{
	    case XljpLocationLeft:
		nx = pw->core.x-(nw+(2*lw->core.border_width));
		break;
	    case XljpLocationILeft:
		nx = pw->core.x;
		break;
	    case XljpLocationRight:
		nx = pw->core.x+pw->core.width+(2*pw->core.border_width);
		break;
	    case XljpLocationIRight:
		nx = (pw->core.x+pw->core.width+(2*pw->core.border_width))
		    -(nw+(2*lw->core.border_width));
		break;
	}
	/*
	fprintf( stdout, "LjpLarry layout new %d,%d w%d h%d\n",
	    nx, ny, nw, nh );
	*/
    }

    /*
    ** Do not go off screen.
    */
    {
	int tmp;

	if( nx < 0 )
	    nx = 0;
	if( ny < 0 )
	    ny = 0;

	if( XtIsRealized((Widget)lw) )
	{
	    tmp = XDisplayWidth( XtDisplay(lw),
		XScreenNumberOfScreen(XtScreen(lw)) );
	    if( (nx+nw) > tmp )
		nx = tmp-nw;
	    tmp = XDisplayHeight( XtDisplay(lw),
		XScreenNumberOfScreen(XtScreen(lw)) );
	    if( (ny+nh) > tmp )
		ny = tmp-nh;
	}
    }

    /*
    ** Only continue if we are going to make size and position requests
    ** to the parent.
    */
    if( doit == False )
    {
	lw->core.x = nx;
	lw->core.y = ny;
	lw->core.width = nw;
	lw->core.height = nh;
	lw->larry.x_space = (lw->core.width-(LARRY_LWIDTH*4))/5;
	lw->larry.y_arrows = lw->core.height-LARRY_LHEIGHT;
	lw->larry.y_popdown = lw->larry.y_arrows-heightButton;

	/*
	fprintf( stdout,
	    "TMP   w%d h%d y_arrows %d x_space %d\n",
	    lw->core.width, lw->core.height,
	    lw->larry.y_arrows,
	    lw->larry.x_space );
	*/
	return;
    }

    /*
    ** Make width request.
    */
    if( nw != lw->core.width )
    {
	XtGeometryResult rc;
	XtWidgetGeometry request, reply;

	request.request_mode = CWWidth;
	request.width = nw;
	rc = XtMakeGeometryRequest( (Widget)lw, &request, &reply );
	/*
	fprintf( stdout, "TMP -= geom width request %d\n", rc );
	if( rc != XtGeometryYes )
	{
	    fputs( "TMP -= geom width request denied\n", stdout );
	}
	*/
    }

    /*
    ** Make height request.
    */
    if( nh != lw->core.height )
    {
	XtGeometryResult rc;
	XtWidgetGeometry request, reply;

	request.request_mode = CWHeight;
	request.height = nh;
	rc = XtMakeGeometryRequest( (Widget)lw, &request, &reply );
	/*
	fprintf( stdout, "TMP -= geom height request %d\n", rc );
	if( rc != XtGeometryYes )
	{
	    fputs( "TMP -= geom height request denied\n", stdout );
	}
	*/
    }

    /*
    ** Make position request.
    */
    if( (nx!=lw->core.x) || (ny!=lw->core.y) )
    {
	XtGeometryResult rc;
	XtWidgetGeometry request, reply;

	request.request_mode = CWX|CWY;
	request.x = nx;
	request.y = ny;
	rc = XtMakeGeometryRequest( (Widget)lw, &request, &reply );
	/*
	fprintf( stdout, "TMP -= geom position request %d\n", rc );
	*/
	if( rc != XtGeometryYes )
	{
	    /*
	    ** This is potentially disasterous. One of the main benefits
	    ** of the Larry Window is that it can not roam from its parent.
	    */
	    ;
	    /*
	    fputs( "LjpLarry. geom position request denied\n", stdout );
	    */
	}
    }

    lw->larry.x_space = (lw->core.width-(LARRY_LWIDTH*4))/5;
    lw->larry.y_arrows = lw->core.height-LARRY_LHEIGHT;
    lw->larry.y_popdown = lw->larry.y_arrows-heightButton;

    /*
    fprintf( stdout,
	"LjpLarry x%d y%d w%d h%d y_arrows %d x_space %d\n",
	lw->core.x, lw->core.y,
	lw->core.width, lw->core.height,
	lw->larry.y_arrows, lw->larry.x_space );
    */
}

static void mkCopyGC( LjpLarryWidget lw )
{
    XGCValues values;
    XtGCMask mask;

    mask = GCForeground|GCBackground;
    values.foreground = lw->larry.topShadowColor;
    values.background = lw->larry.bottomShadowColor;

    lw->larry.gcTop = XCreateGC( XtDisplay(lw),
	RootWindow(XtDisplay(lw),DefaultScreen(XtDisplay(lw))),
	mask, &values );

    mask = GCForeground|GCBackground;
    values.foreground = lw->larry.bottomShadowColor;
    values.background = lw->larry.topShadowColor;

    lw->larry.gcBottom = XCreateGC( XtDisplay(lw),
	RootWindow(XtDisplay(lw),DefaultScreen(XtDisplay(lw))),
	mask, &values );
}

static void mkClearGC( LjpLarryWidget lw )
{
    XGCValues values;
    XtGCMask mask;

    mask = GCForeground|GCBackground|GCFillStyle|GCFillRule;
    values.foreground = lw->core.background_pixel;
    values.background = lw->larry.topShadowColor;
    values.fill_style = FillSolid;
    values.fill_rule = EvenOddRule;

    lw->larry.gcBg = XCreateGC( XtDisplay(lw),
	RootWindow(XtDisplay(lw),DefaultScreen(XtDisplay(lw))),
	mask, &values );
}

static int arrow_left( LjpLarryWidget lw, GC top_gc, GC bottom_gc,
    int x_dest, int y_dest )
{
    const int width = LARRY_LWIDTH;
    const int height = 20;
    XSegment segments[4];
    int y_mid;

    XFillRectangle( XtDisplay(lw), XtWindow(lw), lw->larry.gcBg,
	x_dest, y_dest, width, height );

    y_mid = width/2;

    segments[0].x1 = x_dest+(width-1);
    segments[0].y1 = y_dest;
    segments[0].x2 = x_dest+(width-1);
    segments[0].y2 = y_dest+(height-1);
    segments[1].x1 = x_dest+(width-1);
    segments[1].y1 = y_dest+(height-1);
    segments[1].x2 = x_dest;
    segments[1].y2 = y_dest+(y_mid-1);
    XDrawSegments( XtDisplay(lw), XtWindow(lw), bottom_gc,
	segments, 2 );

    segments[0].x1 = x_dest;
    segments[0].y1 = y_dest+(y_mid-1);
    segments[0].x2 = x_dest+(width-1);
    segments[0].y2 = y_dest;
    XDrawSegments( XtDisplay(lw), XtWindow(lw), top_gc,
	segments, 1 );

    return 1;
}

static int arrow_ileft( LjpLarryWidget lw, GC top_gc, GC bottom_gc,
    int x_dest, int y_dest )
{
    const int width = LARRY_LWIDTH;
    const int height = 20;
    XSegment segments[4];
    int y_mid;

    XFillRectangle( XtDisplay(lw), XtWindow(lw), lw->larry.gcBg,
	x_dest, y_dest, width, height );

    y_mid = width/2;

    segments[0].x1 = x_dest+(width-1);
    segments[0].y1 = y_dest;
    segments[0].x2 = x_dest+(width-1);
    segments[0].y2 = y_dest+(height-1);
    segments[1].x1 = x_dest+(width-1);
    segments[1].y1 = y_dest+(height-1);
    segments[1].x2 = x_dest;
    segments[1].y2 = y_dest+(y_mid-1);
    XDrawSegments( XtDisplay(lw), XtWindow(lw), bottom_gc,
	segments, 2 );

    segments[0].x1 = x_dest;
    segments[0].y1 = y_dest+(y_mid-1);
    segments[0].x2 = x_dest+(width-1);
    segments[0].y2 = y_dest;
    XDrawSegments( XtDisplay(lw), XtWindow(lw), top_gc,
	segments, 1 );

    return 1;
}

static int arrow_iright( LjpLarryWidget lw, GC top_gc, GC bottom_gc,
    int x_dest, int y_dest )
{
    const int width = LARRY_LWIDTH;
    const int height = 20;
    XSegment segments[4];
    int y_mid;

    XFillRectangle( XtDisplay(lw), XtWindow(lw), lw->larry.gcBg,
	x_dest, y_dest, width, height );

    y_mid = height/2;

    segments[0].x1 = x_dest;
    segments[0].y1 = y_dest;
    segments[0].x2 = x_dest+(width-1);
    segments[0].y2 = y_dest+(y_mid-1);
    segments[1].x1 = x_dest;
    segments[1].y1 = y_dest+(height-1);
    segments[1].x2 = x_dest;
    segments[1].y2 = y_dest;
    XDrawSegments( XtDisplay(lw), XtWindow(lw), top_gc,
	segments, 2 );

    segments[0].x1 = x_dest+(width-1);
    segments[0].y1 = y_dest+(y_mid-1);
    segments[0].x2 = x_dest;
    segments[0].y2 = y_dest+(height-1);
    XDrawSegments( XtDisplay(lw), XtWindow(lw), bottom_gc,
	segments, 1 );

    return 1;
}

static int arrow_right( LjpLarryWidget lw, GC top_gc, GC bottom_gc,
    int x_dest, int y_dest )
{
    const int width = LARRY_LWIDTH;
    const int height = 20;
    XSegment segments[4];
    int y_mid;

    XFillRectangle( XtDisplay(lw), XtWindow(lw), lw->larry.gcBg,
	x_dest, y_dest, width, height );

    y_mid = height/2;

    segments[0].x1 = x_dest;
    segments[0].y1 = y_dest;
    segments[0].x2 = x_dest+(width-1);
    segments[0].y2 = y_dest+(y_mid-1);
    segments[1].x1 = x_dest;
    segments[1].y1 = y_dest+(height-1);
    segments[1].x2 = x_dest;
    segments[1].y2 = y_dest;
    XDrawSegments( XtDisplay(lw), XtWindow(lw), top_gc,
	segments, 2 );

    segments[0].x1 = x_dest+(width-1);
    segments[0].y1 = y_dest+(y_mid-1);
    segments[0].x2 = x_dest;
    segments[0].y2 = y_dest+(height-1);
    XDrawSegments( XtDisplay(lw), XtWindow(lw), bottom_gc,
	segments, 1 );

    return 1;
}

static int button_release( LjpLarryWidget lw )
{
    Position x_tmp;

    /*
    fprintf( stdout, "TMP arrow_release %d\n", lw->larry.y_arrows );
    */

    switch( lw->larry.buttonArmed )
    {
	case XljpLocationLeft:
	    x_tmp = (1*lw->larry.x_space)+0;
	    arrow_left( lw, lw->larry.gcTop, lw->larry.gcBottom,
		x_tmp, lw->larry.y_arrows );
	    break;
	case XljpLocationILeft:
	    x_tmp = (2*lw->larry.x_space)+(1*LARRY_LWIDTH);
	    arrow_ileft( lw, lw->larry.gcTop, lw->larry.gcBottom,
		x_tmp, lw->larry.y_arrows );
	    break;
	case XljpLocationIRight:
	    x_tmp = (3*lw->larry.x_space)+(2*LARRY_LWIDTH);
	    arrow_iright( lw, lw->larry.gcTop, lw->larry.gcBottom,
		x_tmp, lw->larry.y_arrows );
	    break;
	case XljpLocationRight:
	    x_tmp = (4*lw->larry.x_space)+(3*LARRY_LWIDTH);
	    arrow_right( lw, lw->larry.gcTop, lw->larry.gcBottom,
		x_tmp, lw->larry.y_arrows );
	    break;
    }

    lw->larry.buttonArmed = XljpLocationNull;
}
