/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

#ifndef LARRY_H
#define LARRY_H

/*
** Xljp widget for a Larry Window.
**
** PURPOSE
**
** A Larry Window is a transient popup with built-in location buttons and
** a popdown button. The window is always located in one of 4 possible
** positions in and next to the parent window. The location buttons may
** be used to select the location. A Larry widget can take a single child
** widget, typically a container.
**
** This widget is meant for games and other applications which use popups
** but do not wish for popups to stray to far from the parent window. The
** location buttons assure that the main window is not needlessly obscured.
**
** USE
**
** If a button widget is created as a child it will be placed below the
** regular child (just above the location arrows). This button will be
** forced to be as wide as the window. This special button is meant for
** use as a popdown button (but this action must be set up by the
** application). The top and bottom shadow colors are used by the built-in
** arrow buttons.
**
** TODO
**
** Locations other than the four horizontal positions should be available
** as an option. In particular, there should be modes: horizontal,
** vertical, interior corners.
**
** RESOURCES
**
**  Name		RepType		Default Value
**  ----		-------		-------------
**  bottomShadowColor	Pixel		XtDefaultBackground
**  buttonWidget	Widget		NULL
**  location		XljpLocation	XljpLocationIRight
**  topShadowColor	Pixel		XtDefaultForeground
**
** Other resources of interest.
**
**  background		Pixel		XtDefaultBackground
**  backgroundPixmap	Pixmap		XtUnspecifiedPixmap
**  border		Pixel		XtDefaultForeground
**  borderWidth		Dimension	0
**  foreground		Pixel		XtDefaultForeground
*/

#include <LjpStringDefs.h>

/*
** Horizontal (the only mode currently supported). Height is same as parent.
**     Outside left, Interior left, Interior right, Outside right
**
** Vertical. Width is same as parent.
**     Outside above, Interior top, Interior bottom, Outside below
**
** Interior corners
**     Center, Upper left, Upper right, Lower left, Lower right
*/
typedef enum
{
    XljpLocationNull = 0,       /* Can not be set by application. */
    XljpLocationLeft,
    XljpLocationRight,
    XljpLocationTop,            /* Reserved for future use. */
    XljpLocationBottom,         /* Reserved for future use. */
    XljpLocationILeft,
    XljpLocationIRight,
    XljpLocationITop,           /* Reserved for future use. */
    XljpLocationIBottom,        /* Reserved for future use. */
    XljpLocationICenter,        /* Reserved for future use. */
    XljpLocationIUL,            /* Reserved for future use. */
    XljpLocationIUR,            /* Reserved for future use. */
    XljpLocationILL,            /* Reserved for future use. */
    XljpLocationILR             /* Reserved for future use. */
} XljpLocation;

extern WidgetClass ljpLarryWidgetClass;

typedef struct _LjpLarryClassRec *LjpLarryWidgetClass;
typedef struct _LjpLarryRec      *LjpLarryWidget;

#endif
