/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

#include <stdio.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include "PrimitiveP.h"

/*
** Widget hooks.
*/
static void hook_initialize( Widget, Widget, ArgList, Cardinal* );
static void hook_classPartInitialize( WidgetClass );
static Boolean hook_setValues( Widget, Widget, Widget, ArgList, Cardinal* );
static void hook_resize( Widget );
static XtGeometryResult hook_queryGeometry( Widget, XtWidgetGeometry*, 
    XtWidgetGeometry* );
static void hook_destroy( Widget );
static void hook_drawFrame( Widget );
static void hook_calcInterior( Widget w, Position*, Position*,
    Position*, Position*, Dimension*, Dimension* );

static void mkGC( LjpPrimitiveWidget );

static XtResource resources[] =
{
    {
	XtNforeground,
	XtCForeground,
	XtRPixel,
	sizeof(Pixel),
	XtOffsetOf(LjpPrimitiveRec,primitive.pixelForeground),
	XtRString,
	XtDefaultForeground
    },
    {
	XljpNfont,
	XljpCFont,
	XtRFontStruct,
	sizeof(XFontStruct*),
	XtOffsetOf(LjpPrimitiveRec,primitive.font),
	XtRString,
	XtDefaultFont
    },
    {
	XljpNmarginWidth,
	XljpCMarginWidth,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpPrimitiveRec,primitive.margin_width),
	XtRImmediate,
	(XtPointer)0
    },
    {
	XljpNmarginHeight,
	XljpCMarginHeight,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpPrimitiveRec,primitive.margin_height),
	XtRImmediate,
	(XtPointer)0
    },
    {
	XljpNmarginTop,
	XljpCMarginTop,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpPrimitiveRec,primitive.margin_top),
	XtRImmediate,
	(XtPointer)2
    },
    {
	XljpNmarginBottom,
	XljpCMarginBottom,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpPrimitiveRec,primitive.margin_bottom),
	XtRImmediate,
	(XtPointer)2
    },
    {
	XljpNmarginLeft,
	XljpCMarginLeft,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpPrimitiveRec,primitive.margin_left),
	XtRImmediate,
	(XtPointer)2
    },
    {
	XljpNmarginRight,
	XljpCMarginRight,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpPrimitiveRec,primitive.margin_right),
	XtRImmediate,
	(XtPointer)2
    },
    {
	XljpNshadowThickness,
	XljpCShadowThickness,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpPrimitiveRec,primitive.shadow_thickness),
	XtRImmediate,
	(XtPointer)0
    },
    {
	XljpNshadowType,
	XljpCShadowType,
	XtRInt,
	sizeof(enum XljpShadowType),
	XtOffsetOf(LjpPrimitiveRec,primitive.shadow_type),
	XtRImmediate,
	(XtPointer)XljpShadowNone
    },
    {
	XljpNtopShadowColor,
	XljpCTopShadowColor,
	XtRPixel,
	sizeof(Pixel),
	XtOffsetOf(LjpPrimitiveRec,primitive.pixelTopShadow),
	XtRString,
	XtDefaultBackground
    },
    {
	XljpNbottomShadowColor,
	XljpCBottomShadowColor,
	XtRPixel,
	sizeof(Pixel),
	XtOffsetOf(LjpPrimitiveRec,primitive.pixelBottomShadow),
	XtRString,
	XtDefaultForeground
    }
};

LjpPrimitiveClassRec ljpPrimitiveClassRec =
{
   {
      (WidgetClass)&coreClassRec,	/* superclass         */
      "LjpPrimitive",			/* class_name         */
      sizeof(LjpPrimitiveRec),		/* widget_size        */
      NULL,    				/* class_initialize   */
      hook_classPartInitialize,        		/* class_part_initiali*/
      FALSE,                                    /* class_inited       */
      hook_initialize,                          /* initialize         */
      NULL,                                     /* initialize_hook    */
      XtInheritRealize,                         /* realize            */
      NULL, 	                                /* actions            */
      0,                                        /* num_actions        */
      resources,                                /* resources          */
      XtNumber(resources),                      /* num_resources      */
      NULLQUARK,                                /* xrm_class          */
      TRUE,                                     /* compress_motion    */
      XtExposeCompressMaximal,                  /* compress_exposure  */
      TRUE,                                     /* compress_enterleave*/
      FALSE,                                    /* visible_interest   */
      hook_destroy,                             /* destroy            */
      hook_resize,                              /* resize             */
      NULL,	                 		/* expose             */
      hook_setValues,                           /* set_values         */
      NULL,                                     /* set_values_hook    */
      NULL,                            		/* set_values_almost  */
      NULL,                                     /* get_values_hook    */
      NULL,                                     /* accept_focus       */
      XtVersion,                                /* version            */
      NULL,                                     /* callback_private   */
      NULL, 	    	                        /* tm_table           */
      hook_queryGeometry,             		/* query_geometry     */
      XtInheritDisplayAccelerator,              /* display accel      */
      NULL,                                     /* extension          */
    },
    /* Primitive. */
    {
	hook_drawFrame,				/* draw_frame		*/
	hook_calcInterior,			/* calc_interior	*/
	NULL					/* extension		*/
    }
};

WidgetClass ljpPrimitiveWidgetClass = (WidgetClass)&ljpPrimitiveClassRec;

static void hook_initialize( Widget w, Widget neww, ArgList args,
    Cardinal *num_args )
{
    LjpPrimitiveWidget fw = (LjpPrimitiveWidget)w;
    LjpPrimitiveWidget new_fw = (LjpPrimitiveWidget)neww;

    /*
    fprintf( stdout, "LjpPrimitive 0x%x hook init\n", neww );
    */

    /*
    ** Override Core class default. App will have to change
    ** it after widget creation if it desires a different
    ** value.
    */
    new_fw->core.border_width = 0;

    {
	XrmValue xrmFrom, xrmTo;
	XFontStruct *rec;
	XGCValues values;
	XtGCMask mask;

#if 0
	xrmFrom.size = strlen(XtDefaultFont);
	xrmFrom.addr = XtDefaultFont;
	xrmTo.addr = NULL;
	XtConvert( w, XtRString, &xrmFrom, XtRFontStruct, &xrmTo );
	new_fw->primitive.font = *((XFontStruct**)xrmTo.addr);
#endif

	mask = GCFont;
	values.font = new_fw->primitive.font->fid;
	new_fw->primitive.gcTop = XCreateGC( XtDisplay(new_fw),
	    DefaultRootWindow(XtDisplay(new_fw)),
	    mask, &values );
	mask = GCFont;
	values.font = new_fw->primitive.font->fid;
	new_fw->primitive.gcBottom = XCreateGC( XtDisplay(new_fw),
	    DefaultRootWindow(XtDisplay(new_fw)),
	    mask, &values );
	mask = GCFillStyle|GCFillRule|GCFont;
	values.fill_style = FillSolid;
	values.fill_rule = EvenOddRule;
	new_fw->primitive.gcClear = XCreateGC( XtDisplay(new_fw),
	    DefaultRootWindow(XtDisplay(fw)),
	    mask, &values );
    }

    {
	Position xl, yt, xr, yb;

	hook_calcInterior( neww, &xl, &yt, &xr, &yb, NULL, NULL );

	if( fw->core.width <= 0 )
	    new_fw->core.width = xl + xr;
	if( fw->core.height <= 0 )
	    new_fw->core.height = yt + yb;
	/*
	fprintf( stdout, "[LjpPrimitive] %x hook init w%d h%d, w%d h%d\n",
	    new_fw, fw->core.width, fw->core.height,
	    new_fw->core.width, new_fw->core.height );
	*/
    }

    mkGC( new_fw );
}

static void hook_classPartInitialize( WidgetClass wc )
{
    LjpPrimitiveWidgetClass bc = (LjpPrimitiveWidgetClass)wc;
    LjpPrimitiveWidgetClass sc =
	(LjpPrimitiveWidgetClass)wc->core_class.superclass;

    if( bc->primitive_class.draw_frame == XljpInheritDrawFrame )
	bc->primitive_class.draw_frame = sc->primitive_class.draw_frame;

    if( bc->primitive_class.calc_interior == XljpInheritCalcInterior )
	bc->primitive_class.calc_interior = sc->primitive_class.calc_interior;
}

static Boolean hook_setValues( Widget oldw, Widget w, Widget neww,
    ArgList args, Cardinal *num_args )
{
    LjpPrimitiveWidget old_fw = (LjpPrimitiveWidget)oldw;
    LjpPrimitiveWidget new_fw = (LjpPrimitiveWidget)neww;
    Boolean do_redisplay = False;
    Boolean do_gc = False;

    /*
    fprintf( stdout, "LjpPrimitive %x hook setVal\n",
       (unsigned int)neww );
    */

    if( (new_fw->core.background_pixel!=old_fw->core.background_pixel) ||
        (new_fw->primitive.pixelTopShadow!=old_fw->primitive.pixelTopShadow) ||
        (new_fw->primitive.pixelBottomShadow!=
	    old_fw->primitive.pixelBottomShadow) ||
        (new_fw->primitive.font!=old_fw->primitive.font) )
    {
	do_redisplay = True;
	mkGC( new_fw );
    }

    if( new_fw->primitive.margin_height != old_fw->primitive.margin_height ||
        new_fw->primitive.margin_width != old_fw->primitive.margin_width ||
        new_fw->primitive.margin_top!=old_fw->primitive.margin_top ||
        new_fw->primitive.margin_bottom!=old_fw->primitive.margin_bottom ||
        new_fw->primitive.margin_left!=old_fw->primitive.margin_left ||
        new_fw->primitive.margin_right!=old_fw->primitive.margin_right ||
        new_fw->core.height != old_fw->core.height ||
        new_fw->core.width != old_fw->core.width ||
        new_fw->primitive.shadow_thickness !=
	    old_fw->primitive.shadow_thickness )
    {
	do_redisplay = True;
    }

    if( new_fw->primitive.shadow_type != old_fw->primitive.shadow_type )
	do_redisplay = True;
    if( new_fw->primitive.shadow_type == XljpShadowNone )
	new_fw->primitive.shadow_thickness = 0;
    if( new_fw->primitive.shadow_type != XljpShadowNone )
    {
	if( new_fw->primitive.shadow_thickness == 0 )
	    new_fw->primitive.shadow_thickness = 1;
	do_redisplay = True;
    }

    return do_redisplay;
}

static void hook_destroy( Widget w )
{
    LjpPrimitiveWidget fw = (LjpPrimitiveWidget)w;
   
    XtReleaseGC( w, fw->primitive.gcTop );
    XtReleaseGC( w, fw->primitive.gcBottom );
    XtReleaseGC( w, fw->primitive.gcClear );
}

static void hook_resize( Widget w )
{
    LjpPrimitiveWidget fw = (LjpPrimitiveWidget)w;

    /*
    fprintf( stdout, "TMP LjpPrimitive hook resize\n" );
    */
    /*layout( (Widget)fw, True );*/
}

static XtGeometryResult hook_queryGeometry( Widget w,
    XtWidgetGeometry *proposed,  XtWidgetGeometry *answer )
{
    LjpPrimitiveWidget fw = (LjpPrimitiveWidget)w;

    /*
    fprintf( stdout, "TMP LjpPrimitive hook queryGeometry\n" );
    */
    answer->request_mode = CWWidth|CWHeight;

    /* initial width and height */
    answer->width = fw->core.width;
    answer->height = fw->core.height;

    if( ((proposed->request_mode&(CWWidth|CWHeight))==(CWWidth|CWHeight))
	&& (proposed->width==answer->width)
	&& (proposed->height == answer->height) )
    {
        return XtGeometryYes;
    }
    else if( answer->width == fw->core.width &&
	answer->height == fw->core.height)
    {
	return XtGeometryNo;
    }
    else
        return XtGeometryAlmost;
}

static void hook_drawFrame( Widget w )
{
    int width, h, thick;
    Position x, y;
    XPoint tPoints[7], bPoints[7];
    GC gcTop, gcBottom;
    LjpPrimitiveWidget fw = (LjpPrimitiveWidget)w;

    switch( fw->primitive.shadow_type )
    {
	case XljpShadowNone:
	    gcTop = gcBottom = fw->primitive.gcClear;
	    break;
	case XljpShadowLine:
	    gcTop = gcBottom = fw->primitive.gcTop;
	    break;
	case XljpShadowIn:
	case XljpShadowEngrave:
	    gcTop = fw->primitive.gcBottom;
	    gcBottom = fw->primitive.gcTop;
	    break;
	case XljpShadowOut:
	case XljpShadowRelief:
	    gcTop = fw->primitive.gcTop;
	    gcBottom = fw->primitive.gcBottom;
	    break;
    }

    width = fw->core.width-(2*fw->primitive.margin_width);
    h = fw->core.height-(2*fw->primitive.margin_height);
    thick = fw->primitive.shadow_thickness;

    /* Upper left corner. */
    tPoints[0].x = fw->primitive.margin_width;
    tPoints[0].y = fw->primitive.margin_height;
    /* Right. */
    tPoints[1].x = width;
    tPoints[1].y = fw->primitive.margin_height;
    /* Down-left. */
    tPoints[2].x = width-thick;
    tPoints[2].y = thick;
    /* Left. */
    tPoints[3].x = thick;
    tPoints[3].y = thick;
    /* Down. */
    tPoints[4].x = thick;
    tPoints[4].y = h-thick;
    /* Left-down. */
    tPoints[5].x = fw->primitive.margin_width;
    tPoints[5].y = h;
    /* Up. */
    tPoints[6].x = fw->primitive.margin_width;
    tPoints[6].y = fw->primitive.margin_height;

    /* Lower right corner. */
    bPoints[0].x = width;
    bPoints[0].y = h;
    /* Left. */
    bPoints[1].x = fw->primitive.margin_width;
    bPoints[1].y = h;
    /* Upper-right. */
    bPoints[2].x = thick;
    bPoints[2].y = h-thick;
    /* Right. */
    bPoints[3].x = width-thick;
    bPoints[3].y = h-thick;
    /* Up. */
    bPoints[4].x = width-thick;
    bPoints[4].y = thick;
    /* Up-right */
    bPoints[5].x = width;
    bPoints[5].y = fw->primitive.margin_height;
    /* Down. */
    bPoints[6].x = width;
    bPoints[6].y = h;

    XFillPolygon( XtDisplay(fw), XtWindow(fw), gcTop, tPoints, 7,
	Nonconvex, CoordModeOrigin );
    XFillPolygon( XtDisplay(fw), XtWindow(fw), gcBottom, bPoints, 7,
	Nonconvex, CoordModeOrigin );

    switch( fw->primitive.shadow_type )
    {
	case XljpShadowRelief:
	    gcTop = fw->primitive.gcBottom;
	    gcBottom = fw->primitive.gcTop;
	    break;
	case XljpShadowEngrave:
	    gcTop = fw->primitive.gcTop;
	    gcBottom = fw->primitive.gcBottom;
	    break;
	default:
	    return;
    }

    width -= (4*thick);
    h -= (4*thick);

    /* Upper left corner. */
    tPoints[0].x = 0+(2*thick)+1;
    tPoints[0].y = 0+(2*thick);
    /* Right. */
    tPoints[1].x = width+(2*thick);
    tPoints[1].y = 0+(2*thick);
    /* Left-down. */
    tPoints[2].x = (width-thick)+(2*thick);
    tPoints[2].y = thick+(2*thick);
    /* Left. */
    tPoints[3].x = thick+(2*thick);
    tPoints[3].y = thick+(2*thick);
    /* Down. */
    tPoints[4].x = thick+(2*thick);
    tPoints[4].y = (h-thick)+(2*thick);
    tPoints[5].x = 0+(2*thick);
    tPoints[5].y = h+(2*thick);
    tPoints[6].x = 0+(2*thick);
    tPoints[6].y = 0+(2*thick);

    /* Lower right corner. */
    bPoints[0].x = width+(2*thick);
    bPoints[0].y = h+(2*thick);
    /* Left. */
    bPoints[1].x = 0+(2*thick);
    bPoints[1].y = h+(2*thick);
    /* Up-right. */
    bPoints[2].x = thick+(2*thick);
    bPoints[2].y = (h-thick)+(2*thick);
    /* Right. */
    bPoints[3].x = width-thick+(2*thick);
    bPoints[3].y = (h-thick)+(2*thick);
    bPoints[4].x = width-thick+(2*thick);
    bPoints[4].y = thick+(2*thick);
    bPoints[5].x = width+(2*thick);
    bPoints[5].y = 0+(2*thick);
    bPoints[6].x = width+(2*thick);
    bPoints[6].y = h+(2*thick);

    XFillPolygon( XtDisplay(fw), XtWindow(fw), gcTop, tPoints, 7,
	Nonconvex, CoordModeOrigin );
    XFillPolygon( XtDisplay(fw), XtWindow(fw), gcBottom, bPoints, 7,
	Nonconvex, CoordModeOrigin );

#if 0
{
    tPoints[0].x = 12;
    tPoints[0].y = 0;
    tPoints[1].x = 11;
    tPoints[1].y = 1;
    tPoints[2].x = 10;
    tPoints[2].y = 2;
    XDrawPoints( XtDisplay(fw), XtWindow(fw), gcTop, tPoints, 3,
       CoordModeOrigin );
    tPoints[0].x = 12;
    tPoints[0].y = 3;
    tPoints[1].x = 11;
    tPoints[1].y = 4;
    tPoints[2].x = 10;
    tPoints[2].y = 5;
    XDrawPoints( XtDisplay(fw), XtWindow(fw), gcTop, tPoints, 3,
       CoordModeOrigin );
    tPoints[0].x = 15;
    tPoints[0].y = 6;
    tPoints[1].x = 15;
    tPoints[1].y = 7;
    tPoints[2].x = 15;
    tPoints[2].y = 8;
    XDrawPoints( XtDisplay(fw), XtWindow(fw), gcBottom, tPoints, 3,
       CoordModeOrigin );
    tPoints[0].x = 18;
    tPoints[0].y = 6;
    tPoints[1].x = 18;
    tPoints[1].y = 7;
    tPoints[2].x = 18;
    tPoints[2].y = 8;
    XDrawPoints( XtDisplay(fw), XtWindow(fw), gcTop, tPoints, 3,
       CoordModeOrigin );
}
#endif
}

static void hook_calcInterior( Widget aw, Position *nx, Position *ny,
    Position *nx2, Position *ny2, Dimension *nw, Dimension *nh )
{
    Dimension w, h, th=0;
    LjpPrimitiveWidget fw = (LjpPrimitiveWidget)aw;

    w = fw->core.width;
    w -= 2*fw->primitive.margin_width;
    w -= fw->primitive.margin_left+fw->primitive.margin_right;

    h = fw->core.height;
    h -= 2*fw->primitive.margin_height;
    h -= fw->primitive.margin_top+fw->primitive.margin_bottom;

    switch( fw->primitive.shadow_type )
    {
	case XljpShadowLine:
	case XljpShadowIn:
	case XljpShadowOut:
	    w -= 2*fw->primitive.shadow_thickness;
	    h -= 2*fw->primitive.shadow_thickness;
	    th = fw->primitive.shadow_thickness;
	    break;
	case XljpShadowEngrave:
	case XljpShadowRelief:
	    w -= 6*fw->primitive.shadow_thickness;
	    h -= 6*fw->primitive.shadow_thickness;
	    th = 3*fw->primitive.shadow_thickness;
	    break;
	case XljpShadowNone:
	default:
	    break;
    }

    /*
    fprintf( stdout, "[LjpPrimitive] %x hook calcInt x1%d y1%d, x2%d y2%d\n",
	fw,
	fw->primitive.margin_width+th+fw->primitive.margin_left,
	fw->primitive.margin_height+th+fw->primitive.margin_top,
	fw->primitive.margin_width+th+fw->primitive.margin_right,
	fw->primitive.margin_height+th+fw->primitive.margin_bottom );
    */
    if( nx != NULL )
	*nx = fw->primitive.margin_width+th+fw->primitive.margin_left;
    if( ny != NULL )
	*ny = fw->primitive.margin_height+th+fw->primitive.margin_top;
    if( nx2 != NULL )
	*nx2 = fw->primitive.margin_width+th+fw->primitive.margin_right;
    if( ny2 != NULL )
	*ny2 = fw->primitive.margin_height+th+fw->primitive.margin_bottom;
    if( nw != NULL )
	*nw = w;
    if( nh != NULL )
	*nh = h;
}

static void mkGC( LjpPrimitiveWidget fw )
{
    XGCValues values;
    XtGCMask mask;

    mask = GCForeground|GCBackground;
    values.foreground = fw->primitive.pixelTopShadow;
    values.background = fw->core.background_pixel;

    XChangeGC( XtDisplay(fw), fw->primitive.gcTop, mask, &values );

    mask = GCForeground|GCBackground;
    values.foreground = fw->primitive.pixelBottomShadow;
    values.background = fw->core.background_pixel;

    XChangeGC( XtDisplay(fw), fw->primitive.gcBottom, mask, &values );

    mask = GCForeground|GCBackground|GCFillStyle|GCFillRule;
    values.foreground = fw->core.background_pixel;
    values.background = fw->primitive.pixelTopShadow;
    values.fill_style = FillSolid;
    values.fill_rule = EvenOddRule;

    XChangeGC( XtDisplay(fw), fw->primitive.gcClear, mask, &values );
}
