/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

#include <stdio.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include "RowP.h"

/*
** Widget hooks.
*/
static void hook_initialize( Widget, Widget, ArgList, Cardinal* );
static Boolean hook_setValues( Widget, Widget, Widget, ArgList, Cardinal* );
static void hook_resize( Widget );
static void hook_changeManaged( Widget );
static XtGeometryResult hook_geometryManager( Widget, XtWidgetGeometry*,
    XtWidgetGeometry* );
static XtGeometryResult hook_queryGeometry( Widget, XtWidgetGeometry*, 
    XtWidgetGeometry* );

static void mkGC( LjpRowWidget );
static void placement( LjpRowWidget, Widget, Boolean );
static void layout( Widget );
static void draw_menubar( Widget );
static print_request( const char*, XtWidgetGeometry* );

static XtResource resources[] =
{
    {
	XljpNorientation,
	XljpCOrientation,
	XtRInt,
	sizeof(enum XljpOrientation),
	XtOffsetOf(LjpRowRec,row.orientation),
	XtRImmediate,
	(XtPointer)XljpOrientVertical
    },
    {
	XljpNhfit,
	XljpCHFit,
	XtRInt,
	sizeof(enum XljpFit),
	XtOffsetOf(LjpRowRec,row.hfit),
	XtRImmediate,
	(XtPointer)XljpFitLoose
    },
    {
	XljpNvfit,
	XljpCVFit,
	XtRInt,
	sizeof(enum XljpFit),
	XtOffsetOf(LjpRowRec,row.vfit),
	XtRImmediate,
	(XtPointer)XljpFitLoose
    },
    {
	XljpNhalignment,
	XljpCHAlignment,
	XtRInt,
	sizeof(enum XljpAlignment),
	XtOffsetOf(LjpRowRec,row.halignment),
	XtRImmediate,
	(XtPointer)XljpAlignCenter
    },
    {
	XljpNvalignment,
	XljpCVAlignment,
	XtRInt,
	sizeof(enum XljpAlignment),
	XtOffsetOf(LjpRowRec,row.valignment),
	XtRImmediate,
	(XtPointer)XljpAlignCenter
    },
    {
	XljpNmarginWidth,
	XljpCMarginWidth,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpRowRec,row.margin_width),
	XtRDimension,
	(XtPointer)0
    },
    {
	XljpNmarginHeight,
	XljpCMarginHeight,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpRowRec,row.margin_height),
	XtRDimension,
	(XtPointer)0
    },
    {
	XljpNspacing,
	XljpCSpacing,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpRowRec,row.spacing),
	XtRDimension,
	(XtPointer)0
    }
};
	  
LjpRowClassRec ljpRowClassRec =
{
   {
      (WidgetClass) &compositeClassRec,         /* superclass         */
      "LjpRow",                                	/* class_name         */
      sizeof(LjpRowRec),                       	/* widget_size        */
      NULL,                          		/* class_initialize   */
      NULL,                            		/* class_part_initiali*/
      FALSE,                                    /* class_inited       */
      hook_initialize,                          /* initialize         */
      NULL,                                     /* initialize_hook    */
      XtInheritRealize,                         /* realize            */
      NULL, 	                                /* actions            */
      0,                                        /* num_actions        */
      resources,                                /* resources          */
      XtNumber(resources),                      /* num_resources      */
      NULLQUARK,                                /* xrm_class          */
      TRUE,                                     /* compress_motion    */
      XtExposeCompressMaximal,                  /* compress_exposure  */
      TRUE,                                     /* compress_enterleave*/
      FALSE,                                    /* visible_interest   */
      NULL,					/* destroy            */
      hook_resize,                              /* resize             */
      NULL,	                 		/* expose             */
      hook_setValues,                           /* set_values         */
      NULL,                                     /* set_values_hook    */
      NULL,                            		/* set_values_almost  */
      NULL,                                     /* get_values_hook    */
      NULL,                                     /* accept_focus       */
      XtVersion,                                /* version            */
      NULL,                                     /* callback_private   */
      NULL, 	    	                        /* tm_table           */
      hook_queryGeometry,             		/* query_geometry     */
      XtInheritDisplayAccelerator,              /* display accel      */
      NULL,                                     /* extension          */
    },
    /* Composite class. */
    {
    /* geometry_manager   */    hook_geometryManager,
    /* change_managed     */    hook_changeManaged,
    /* insert_child	  */	XtInheritInsertChild,
    /* delete_child	  */	XtInheritDeleteChild,
    /* extension          */	NULL
    },
    /* Row. */
    {
	NULL                                    /* extension          */
    }
};

WidgetClass ljpRowWidgetClass = (WidgetClass)&ljpRowClassRec;

static void hook_initialize( Widget w, Widget neww, ArgList args,
    Cardinal *num_args )
{
    LjpRowWidget rw = (LjpRowWidget)w;
    LjpRowWidget new_rw = (LjpRowWidget)neww;
    Dimension defaultWidth=0, defaultHeight=0;

    /*
    fprintf( stdout, "LjpRow 0x%x hook initialize\n", neww );
    */

    /*
    ** Override Core class default. App will have to change
    ** it after widget creation if it desires a different
    ** value.
    */
    new_rw->core.border_width = 0;

    /* Arbitrary defaults. */
    if( neww->core.width == 0 )
	neww->core.width = 60;
    if( neww->core.height == 0 )
	neww->core.height = 40;
}

static Boolean hook_setValues( Widget oldw, Widget w, Widget neww,
    ArgList args, Cardinal *num_args )
{
    LjpRowWidget old_mw = (LjpRowWidget)oldw;
    LjpRowWidget new_mw = (LjpRowWidget)neww;

    /*
    fprintf( stdout, "LjpRow %x hook setValues\n", neww );
    */

    if( new_mw->row.margin_height != old_mw->row.margin_height ||
        new_mw->row.margin_width != old_mw->row.margin_width ||
        new_mw->row.halignment != old_mw->row.halignment ||
        new_mw->row.valignment != old_mw->row.valignment ||
        new_mw->row.orientation != old_mw->row.orientation )
    {
	layout( neww );
    }

    return True;
}

static void hook_resize( Widget w )
{
    LjpRowWidget rw = (LjpRowWidget)w;

    /*
    fprintf( stdout, "LjpRow %x hook resize w%d h%d\n",
	w, w->core.width, w->core.height );
    */

    layout( w );
}

/*
** Parent wants to know what geometry we want. Accept
** whatever they propose. We want to expand to encompass
** the area the parent has for us.
*/
static XtGeometryResult hook_queryGeometry( Widget w,
    XtWidgetGeometry *request, XtWidgetGeometry *reply )
{
    Boolean flag;
    LjpRowWidget rw = (LjpRowWidget)w;

    /*
    fprintf( stdout, "LjpRow %x hook queryGeometry\n", w );
    */

    flag = True;
    if( (request->request_mode&CWWidth) && (rw->core.width!=request->width) )
	flag = False;
    if( (request->request_mode&CWHeight) &&
	(rw->core.height!=request->height) )
    {
	flag = False;
    }
    if( flag == True )
	return XtGeometryNo;

    return XtGeometryYes;
}

/*
** Child wants to change its size. We allow the child to grow only within
** the existing borders.
*/
static XtGeometryResult hook_geometryManager( Widget cw,
    XtWidgetGeometry *request, XtWidgetGeometry *reply )
{
    Boolean flagAlmost = False;
    LjpRowWidget rw = (LjpRowWidget)XtParent(cw);
    XtGeometryResult result = XtGeometryYes;
    XtWidgetGeometry geomRequest, geomReply;

    /*
    fprintf( stdout, "LjpRow %x(%x) hook geometryManager\n",
	rw, cw );
    */

    /*
    print_request( "child wants", request );
    */

    geomRequest.request_mode = 0;
    geomReply.request_mode = 0;
    reply->request_mode = 0;

    /* Only consider size requests. */
    if( !(request->request_mode&(CWWidth|CWHeight)) )
    {
	/*
	fputs( "TMP LjpRow reject non size request\n", stdout );
	*/
    	return XtGeometryNo;
    }

    /*
    ** If child already has requested height and width no work is needed.
    */
    flagAlmost = True;
    if( (request->request_mode&CWWidth) &&
	(request->width!=rw->core.width) )
    {
	flagAlmost = False;
    }
    if( (request->request_mode&CWHeight) &&
	(request->height!=rw->core.height) )
    {
	flagAlmost = False;
    }
    if( flagAlmost == True )
	return XtGeometryNo;

    flagAlmost = False;

    if( request->request_mode&CWWidth )
    {
	if( request->width > rw->core.width )
	{
	    reply->width = rw->core.width;
	    reply->request_mode |= CWWidth;
	    flagAlmost = True;
	}
    }
    if( request->request_mode&CWHeight )
    {
	if( request->height > rw->core.height )
	{
	    reply->height = rw->core.height;
	    reply->request_mode |= CWHeight;
	    flagAlmost = True;
	}
    }

    if( flagAlmost )
	result = XtGeometryAlmost;

    if( request->request_mode & XtCWQueryOnly )
	return result;

    /*
    layout( (Widget)mw );
    */

    return result;
}

static void hook_changeManaged( Widget w )
{
    int i;
    Dimension nw, nh;
    LjpRowWidget rw = (LjpRowWidget)w;
    Widget cw;

    /*
    fprintf( stdout, "LjpRow %x hook changeManaged\n", w );
    */

    if( rw->composite.num_children == 0 )
	return;

    /* Resize and place row widget. */
    placement( rw, XtParent(rw), True );

    /* Resize and place children. */
    layout( w );
}

/*
** Size of widget is assumed to be correct for the current orientation
** and fit. This function positions and sizes the children.
*/
static void layout( Widget w )
{
    int i;
    LjpRowWidget rw = (LjpRowWidget)w;
    Widget cw;
    Position nx, ny;
    Dimension width, height, cwidth, cheight;
    Dimension nw, nh;

    /*
    fprintf( stdout, "LjpRow %x layout\n", w );
    */

    if( rw->composite.num_children < 1 )
	return;

    width = rw->core.width - (2*rw->row.margin_width);
    height = rw->core.height - (2*rw->row.margin_height);

    /*
    ** Horizontal and vertical space used by children.
    */
    nw = nh = 0;
    for( i=0; i<rw->composite.num_children; ++i )
    {
	cw = rw->composite.children[i];
	nw += cw->core.width+(2*cw->core.border_width);
	nh += cw->core.height+(2*cw->core.border_width);
    }
    nw += rw->row.spacing*(rw->composite.num_children-1);
    nh += rw->row.spacing*(rw->composite.num_children-1);

    /*
    ** Perform horizontal expansion if needed. Note that horizontal
    ** expansion is different for horizontal and vertical orientation.
    */
    if( (rw->row.hfit==XljpFitExpand) && (nw<width) )
    {
	int difference = width-nw;
	int count = 0;
	int expansion;

	/*
	** FUTURE. Count number of row widget children. This really
	** ought to be counting the number of composite children.
	*/
	/* This is only needed when the orientation is horizontal. */
	for( i=0; i<rw->composite.num_children; ++i )
	{
	    cw = rw->composite.children[i];
	    if( strcmp(cw->core.widget_class->core_class.class_name,
		"LjpRow") == 0 )
	    {
		++count;
	    }
	}

	if( rw->row.orientation == XljpOrientHorizontal )
	{
	    expansion = difference/count;
	    /*
	    fprintf( stdout, "LjpRow h expand %d rows by %d.\n",
		count, difference );
	    */
	}

	/* Expand the children. */
	for( i=0; i<rw->composite.num_children; ++i )
	{
	    cw = rw->composite.children[i];
	    if( strcmp(cw->core.widget_class->core_class.class_name,
		"LjpRow") != 0 )
	    {
		continue;
	    }
	    if( rw->row.orientation == XljpOrientHorizontal )
		nw = cw->core.width + expansion;
	    else
		nw = width;
	    /*
	    fprintf( stdout, "LjpRow h expand resize 0x%x to %d.\n",
		cw, nw );
	    */
	    XtResizeWidget( cw, nw, cw->core.height,
		cw->core.border_width );
	}
    }

    /* Determine horizontal starting position. */
    if( rw->row.halignment == XljpAlignBeginning )
	nx = rw->row.margin_width;
    else if( rw->row.halignment == XljpAlignEnd )
	nx = rw->row.margin_width+(width-nw);
    else
	nx = rw->row.margin_width+((width-nw)/2);

    /* Determine vertical starting position. */
    if( rw->row.valignment == XljpAlignBeginning )
	ny = rw->row.margin_height;
    else if( rw->row.valignment == XljpAlignEnd )
	ny = rw->row.margin_height+(height-nh);
    else
	ny = rw->row.margin_height+((height-nh)/2);

    /*
    ** Move the children into position.
    */
    for( i=0; i<rw->composite.num_children; ++i )
    {
	if( (cw=rw->composite.children[i]) == NULL )
	    continue;

	cwidth = cw->core.width+(2*cw->core.border_width);
	cheight = cw->core.height+(2*cw->core.border_width);

	if( rw->row.orientation == XljpOrientHorizontal )
	{
	    if( rw->row.valignment == XljpAlignBeginning )
		ny = rw->row.margin_height;
	    else if( rw->row.valignment == XljpAlignEnd )
		ny = rw->core.height-(cheight+rw->row.margin_height);
	    else
		ny = rw->row.margin_height+((height-cheight)/2);
	    XtMoveWidget( cw, nx, ny );
	    /*
	    fprintf( stdout, "LjpRow layout ch%x x%d y%d\n", cw, nx, ny );
	    */
	    nx += cwidth+rw->row.spacing;
	}
	else
	{
	    if( rw->row.halignment == XljpAlignBeginning )
		nx = rw->row.margin_width;
	    else if( rw->row.halignment == XljpAlignEnd )
		nx = rw->core.width-(cwidth+rw->row.margin_width);
	    else
		nx = rw->row.margin_width+((width-cwidth)/2);
	    XtMoveWidget( cw, nx, ny );
	    /*
	    fprintf( stdout, "LjpRow layout ch%x x%d y%d\n", cw, nx, ny );
	    */
	    ny += cheight+rw->row.spacing;
	}
    }
}

static void placement( LjpRowWidget rw, Widget pw, Boolean doit )
{
    int i;
    Dimension nw, nh;
    XtGeometryResult result;
    XtWidgetGeometry request, reply;
    Widget cw;

    /* Sizes required to fit children. */
    nh = 0;
    nw = 0;

    if( rw->row.orientation == XljpOrientHorizontal )
    {
	for( i=0; i<rw->composite.num_children; ++i )
	{
	    if( (cw=rw->composite.children[i]) == NULL )
		continue;
	    nw += cw->core.width+(2*cw->core.border_width);
	    if( (cw->core.height+(2*cw->core.border_width)) > nh )
		nh = cw->core.height+(2*cw->core.border_width);
	}
	nw += rw->row.spacing*(rw->composite.num_children-1);
	/*
	fprintf( stdout, "LjpRow 0x%x add in %d for w spacing\n",
	    rw, rw->row.spacing*(rw->composite.num_children-1) );
	*/
    }
    else
    {
	for( i=0; i<rw->composite.num_children; ++i )
	{
	    if( (cw=rw->composite.children[i]) == NULL )
		continue;
	    nh += cw->core.height+(2*cw->core.border_width);
	    if( (cw->core.width+(2*cw->core.border_width)) > nw )
		nw = cw->core.width+(2*cw->core.border_width);
	}
	nh += rw->row.spacing*(rw->composite.num_children-1);
	/*
	fprintf( stdout, "LjpRow 0x%x add in %d for h spacing\n",
	    rw, rw->row.spacing*(rw->composite.num_children-1) );
	*/
    }

    /* Add in margins. */
    nw += (2*rw->row.margin_width);
    nh += (2*rw->row.margin_height);

    /*
    ** FUTURE. Below is only true for loose or expanded fit. For
    ** others the widget needs to shrink.
    */
    if( (nw<=rw->core.width) && (nh<=rw->core.height) )
    {
	/*
	fprintf( stdout, "LjpRow 0x%x placement ok with w%dh%d\n",
	    rw, nw, nh );
	*/
	return;
    }

    request.request_mode = CWWidth | CWHeight;
    request.width = nw;
    request.height = nh;
    /*
    print_request( "placement request", &request );
    */
    result = XtMakeGeometryRequest( (Widget)rw, &request, &reply );
}

static print_request( const char *msg, XtWidgetGeometry *rec )
{
    fprintf( stdout, "LjpRow %s w%d%c h%d%c\n",
	msg,
	rec->width, (rec->request_mode&CWWidth)?'*':' ',
	rec->height, (rec->request_mode&CWHeight)?'*':' ' );
}
