/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

#include <stdio.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include "ScrollWinP.h"
#include "Scrollbar.h"

/*
** Widget hooks.
*/
static void hook_initialize( Widget, Widget );
static Boolean hook_setValues( Widget, Widget, Widget, ArgList, Cardinal* );
static void hook_resize( Widget );
static void hook_changeManaged( Widget );
static XtGeometryResult hook_geometryManager( Widget, XtWidgetGeometry*,
    XtWidgetGeometry* );
static XtGeometryResult hook_queryGeometry( Widget, XtWidgetGeometry*, 
    XtWidgetGeometry* );
static void hook_destroy( Widget );
static void hook_insertChild( Widget, ArgList, Cardinal* );
static void hook_realize( Widget, Mask*, XSetWindowAttributes* );

static void layout( LjpScrollWinWidget, Boolean );
static print_request( const char*, XtWidgetGeometry* );
static void hscroll_cb( Widget, XtPointer, struct ljpGenericCallbackStruct* );
static void vscroll_cb( Widget, XtPointer, struct ljpGenericCallbackStruct* );

static XtResource resources[] =
{
    {
	XljpNhorizontalScrollBar,
	XljpCHorizontalScrollBar,
	XtRWidget,
	sizeof(Widget),
	XtOffsetOf(LjpScrollWinRec,scrollwin.whs),
	XtRImmediate,
	(XtPointer)NULL
    },
    {
	XljpNverticalScrollBar,
	XljpCVerticalScrollBar,
	XtRWidget,
	sizeof(Widget),
	XtOffsetOf(LjpScrollWinRec,scrollwin.wvs),
	XtRImmediate,
	(XtPointer)NULL
    },
    {
	XljpNmarginWidth,
	XljpCMarginWidth,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpScrollWinRec,scrollwin.margin_width),
	XtRDimension,
	(XtPointer)0
    },
    {
	XljpNmarginHeight,
	XljpCMarginHeight,
	XtRDimension,
	sizeof(Dimension),
	XtOffsetOf(LjpScrollWinRec,scrollwin.margin_height),
	XtRDimension,
	(XtPointer)0
    },
};
	  
LjpScrollWinClassRec ljpScrollWinClassRec =
{
   {
      (WidgetClass)&compositeClassRec,	/* superclass         */
      "LjpScrollWin",			/* class_name         */
      sizeof(LjpScrollWinRec),		/* widget_size        */
      NULL,                          		/* class_initialize   */
      NULL,                            		/* class_part_initiali*/
      FALSE,                                    /* class_inited       */
      hook_initialize,			/* initialize         */
      NULL,				/* initialize_hook    */
      XtInheritRealize,			/* realize            */
      NULL, 	                                /* actions            */
      0,                                        /* num_actions        */
      resources,                                /* resources          */
      XtNumber(resources),                      /* num_resources      */
      NULLQUARK,                                /* xrm_class          */
      TRUE,                                     /* compress_motion    */
      XtExposeCompressMaximal,                  /* compress_exposure  */
      TRUE,                                     /* compress_enterleave*/
      FALSE,                                    /* visible_interest   */
      hook_destroy,                             /* destroy            */
      hook_resize,                              /* resize             */
      NULL,	                 		/* expose             */
      hook_setValues,                           /* set_values         */
      NULL,                                     /* set_values_hook    */
      NULL,                            		/* set_values_almost  */
      NULL,                                     /* get_values_hook    */
      NULL,                                     /* accept_focus       */
      XtVersion,                                /* version            */
      NULL,                                     /* callback_private   */
      NULL, 	    	                        /* tm_table           */
      hook_queryGeometry,             		/* query_geometry     */
      XtInheritDisplayAccelerator,              /* display accel      */
      NULL,                                     /* extension          */
    },
    /* Composite class. */
    {
    /* geometry_manager   */    hook_geometryManager,
    /* change_managed     */    hook_changeManaged,
    /* insert_child	  */	hook_insertChild,
    /* delete_child	  */	XtInheritDeleteChild,
    /* extension          */	NULL
    },
    /* Frame. */
    {
	NULL                                    /* extension          */
    }
};

WidgetClass ljpScrollWinWidgetClass = (WidgetClass)&ljpScrollWinClassRec;

static void hook_initialize( Widget request, Widget new )
{
    int i;
    Arg args[16];
    LjpScrollWinWidget sbwnew = (LjpScrollWinWidget)new;
    LjpScrollWinWidget sbwreq = (LjpScrollWinWidget)request;

    /*
    fprintf( stdout, "LjpScrollWin 0x%x hook initialize.\n", new );
    */

    if( sbwnew->core.width == 0 )
        sbwnew->core.width = 100;

    if( sbwnew->core.height == 0 )
        sbwnew->core.height = 100;

    i = 0;
    XtSetArg( args[i], XljpNorientation, XljpOrientHorizontal ); ++i;
    XtSetArg( args[i], XtNheight, 10 ); ++i;
    XtSetArg( args[i], XtNborderWidth, 0 ); ++i;
    sbwnew->scrollwin.whs = XtCreateManagedWidget( "whs",
	ljpScrollbarWidgetClass, new, args, i );

    i = 0;
    XtSetArg( args[i], XljpNorientation, XljpOrientVertical ); ++i;
    XtSetArg( args[i], XtNwidth, 10 ); ++i;
    XtSetArg( args[i], XtNborderWidth, 0 ); ++i;
    sbwnew->scrollwin.wvs = XtCreateManagedWidget( "wvs",
	ljpScrollbarWidgetClass, new, args, i );

    i = 0;
    XtSetArg( args[i], XtNborderWidth, 0 ); ++i;
    sbwnew->scrollwin.wclip = XtCreateManagedWidget( "wclip",
	ljpBoardWidgetClass, new, args, i );

    XtAddCallback( sbwnew->scrollwin.whs, XljpNactivateCallback,
	(XtCallbackProc)hscroll_cb, NULL );

    XtAddCallback( sbwnew->scrollwin.wvs, XljpNactivateCallback,
	(XtCallbackProc)vscroll_cb, NULL );

    sbwnew->scrollwin.wchild = NULL;
    sbwnew->scrollwin.xoff = 0;
    sbwnew->scrollwin.yoff = 0;

    layout( sbwnew, False );
}

#if 0
static void hook_realize( Widget w, Mask *mask, XSetWindowAttributes *attr )
{
    LjpScrollWinWidget sbw = (LjpScrollWinWidget)w;
    CoreWidgetClass sc;

    /*
    fputs( "LjpScrollWin hook realize\n", stdout );
    */

    coreWidgetClass->core_class.realize(w,mask,attr);

    /*
    sbw->scrollwin.winclip = XCreateSimpleWindow(
	XtDisplay(sbw), XtWindow(sbw), 0, 0, 10, 10, 0, 0,
	    sbw->core.background_pixel );
    */
    
    sbw->scrollwin.xoff = 0;
    sbw->scrollwin.yoff = 0;
}
#endif

static Boolean hook_setValues( Widget current,  Widget request,  Widget new,
    ArgList args, Cardinal *num_args )
{
    LjpScrollWinWidget sbwcurrent = (LjpScrollWinWidget) current;
    LjpScrollWinWidget sbwnew = (LjpScrollWinWidget) new;

    if( (sbwnew->scrollwin.margin_width!=sbwcurrent->scrollwin.margin_width)
	|| (sbwnew->scrollwin.margin_height!=
	sbwcurrent->scrollwin.margin_height) )
    {
        layout( sbwnew, True );
    }

    return True;
}

static void hook_destroy( Widget w )
{
    LjpScrollWinWidget lw = (LjpScrollWinWidget)w;
}

static void hook_changeManaged( Widget w )
{
    LjpScrollWinWidget sbw = (LjpScrollWinWidget)w;

    /*
    fputs( "LjpScrollWin hook changeManaged\n", stdout );
    */
    layout( sbw, True );
}

static void hook_insertChild( Widget w, ArgList args, Cardinal *num_args )
{
    LjpScrollWinWidget sbw = (LjpScrollWinWidget)XtParent(w);
    CompositeWidgetClass sc;
    int count = sbw->composite.num_children;

    /*
    fprintf( stdout, "LjpScrollWin hook insertChild\n" );
    */

    if( sbw->composite.num_children > 3 )
    {
	XtAppError( XtWidgetToApplicationContext((Widget)sbw),
	    "LjpScrollWin: Can only manage one widget." );
    }

    if( sbw->composite.num_children == 3 )
    {
	sbw->scrollwin.wchild = w;
	sbw->scrollwin.wchild->core.parent = sbw->scrollwin.wclip;
    }

    sc = (CompositeWidgetClass)compositeWidgetClass;
    /*
    (*sc->composite_class.insert_child)(w,args,num_args);
    */
    (*sc->composite_class.insert_child)(w);
}

static void hook_resize( Widget w )
{
    LjpScrollWinWidget sbw = (LjpScrollWinWidget)w;

    /*
    fputs( "LjpScrollWin hook resize\n", stdout );
    */
    layout( sbw, True );
}

static XtGeometryResult hook_queryGeometry( Widget w,
    XtWidgetGeometry *request, XtWidgetGeometry *reply_return )
{
    XtGeometryResult result;

    /*
    fputs( "LjpScrollWin hook queryGeometry\n", stdout );
    */

    request->request_mode &= CWWidth | CWHeight;

    if( request->request_mode == 0 )
	return XtGeometryYes;

    if( request->request_mode & CWHeight )
    {
	if( request->height < 100 )
	{
            result = XtGeometryAlmost;
            reply_return->height = 100;
            reply_return->request_mode &= CWHeight;
        }
        else
            result = XtGeometryYes;
    }

    if( request->request_mode & CWWidth )
    {
	if( request->width < 100 )
	{
            result = XtGeometryAlmost;
            reply_return->width = 100;
            reply_return->request_mode &= CWWidth;
        }
        else
            result = XtGeometryYes;
    }

    return result;
}

static XtGeometryResult hook_geometryManager( Widget cw,
    XtWidgetGeometry *request, XtWidgetGeometry *reply )
{
    LjpScrollWinWidget sbw = (LjpScrollWinWidget)XtParent(cw);

    /*
    fputs( "LjpScrollWin hook geoMgr\n", stdout );
    */

    if( request->request_mode & ~(XtCWQueryOnly|CWWidth|CWHeight) )
	return XtGeometryNo;

    /*
    ** Scrolled child can do whatever it wants.
    */
    if( cw == sbw->scrollwin.wchild )
    {
	/*
	fputs( "LjpScrollWin hook geoMgr scrolled child given ok\n",
	   stdout );
	*/
	if( (cw->core.width==request->width) &&
	    (cw->core.height==request->height) )
	{
	    return XtGeometryNo;
	}
	return XtGeometryYes;
    }

    /*
    ** Clip child can not change size on its own.
    */
    if( cw == sbw->scrollwin.wclip )
    {
	/*
	fputs( "LjpScrollWin hook geoMgr clip child given nak\n",
	   stdout );
	*/
	return XtGeometryNo;
    }

    /*
    ** Scrollbars can do anything but exceed parent size.
    ** FUTURE. Allow scrollbars to exceed parent size (grow to accomodate).
    */
    reply->request_mode = 0;
    if( request->request_mode & CWWidth )
    {
        reply->width = request->width;
	reply->request_mode |= CWWidth;
    }

    if( request->request_mode & CWHeight )
    {
        reply->height = request->height;
	reply->request_mode |= CWHeight;
    }

    if( (reply->width==cw->core.width) && (reply->height==cw->core.height) )
    {
	/*
	fputs( "LjpScrollWin hook geoMgr sbar child already sized\n",
	   stdout );
	*/
        return XtGeometryNo;
    }

    if( !(request->request_mode & XtCWQueryOnly) )
        layout( sbw, False );

    /*
    fprintf( stdout,
	"LjpScrollWin hook geoMgr sbar child 0x%x ok for w%d h%d\n",
	reply->width, reply->height );
    */

    return XtGeometryYes;
}

static void layout( LjpScrollWinWidget w, Boolean doit )
{
    LjpScrollWinWidget sbw = (LjpScrollWinWidget) w;
    Widget wc, wvs, whs;
    Dimension wPort, hPort;
    Dimension hslider_width, vslider_width;
    Dimension wChild, hChild;
    float wPer, hPer;
    int i;

    /*
    fputs( "LjpScrollWin layout\n", stdout );
    */

    if( sbw->composite.num_children != 3 )
    {
	XtAppError( XtWidgetToApplicationContext((Widget)sbw),
	    "ScrollWin: must manage exactly three widgets." );
    }

    for( i=0; i<sbw->composite.num_children; ++i )
    {
	wc = sbw->composite.children[i];
	if( !XtIsManaged(wc) )
	{
	    XtAppError(XtWidgetToApplicationContext((Widget)sbw),
		"ScrollWin: all three widgets must be managed.");
	}
    }

    wc = sbw->scrollwin.wchild;
    whs = sbw->scrollwin.whs;
    wvs = sbw->scrollwin.wvs;
	
    wChild = (wc->core.width+(2*wc->core.border_width));
    hChild = (wc->core.height+(2*wc->core.border_width));

    wPort = sbw->core.width - (wvs->core.width-(2*wvs->core.border_width));
    hPort = sbw->core.height - (whs->core.height-(2*whs->core.border_width));

    if( wChild <= wPort )
    {
	hslider_width = wPort;
	sbw->scrollwin.xInc = 0;
    }
    else
    {
	wPer = (float)wChild/(float)wPort;
	/*
	fprintf( stdout, "LjpScrollWin wPer %d wPort %d\n", wPer, wPort );
	*/
	sbw->scrollwin.xInc = wPer;
	if( sbw->scrollwin.xInc < 1 )
	    sbw->scrollwin.xInc = 1;
	hslider_width = (int)((float)wPort/wPer);
	if( hslider_width < 1 )
	    hslider_width = 1;
    }
    if( (wc->core.height+(2*wc->core.border_width)) <= hPort )
    {
	vslider_width = hPort;
	sbw->scrollwin.yInc = 0;
    }
    else
    {
	hPer = (wc->core.height+(2*wc->core.border_width))/hPort;
	sbw->scrollwin.yInc = hPer;
	if( sbw->scrollwin.yInc < 1 )
	    sbw->scrollwin.yInc = 1;
	if( (vslider_width=hPort/hPer) < 1 )
	    vslider_width = 1;
    }

    XtVaSetValues( wvs, XljpNsliderSize, vslider_width, NULL );
    XtVaSetValues( whs, XljpNsliderSize, hslider_width, NULL );

    if( doit )
    {
	XtResizeWidget( sbw->scrollwin.wclip, wPort, hPort, 0 );

	XtResizeWidget( wvs, wvs->core.width, hPort, 0 );
	XtMoveWidget( wvs, wPort, 0 );

	XtResizeWidget( whs, wPort, whs->core.height, 0 );
	XtMoveWidget( whs, 0, hPort );
    }

    /*
    fprintf( stdout, "LjpScrollWin layout win w%d, h%d.\n",
	sbw->core.width, sbw->core.height );
    fprintf( stdout, "LjpScrollWin layout clip w%d, h%d.\n",
	sbw->scrollwin.wclip->core.width,
	sbw->scrollwin.wclip->core.height );
    fprintf( stdout, "LjpScrollWin layout child w%d, h%d.\n",
	wc->core.width, wc->core.height );
    fprintf( stdout,
	"LjpScrollWin layout sliderw%d, sliderh%d, xinc %d, yinc %d.\n",
	hslider_width, vslider_width,
	sbw->scrollwin.xInc, sbw->scrollwin.yInc );
    */
}

static void hscroll_cb( Widget w, XtPointer closure,
    struct ljpGenericCallbackStruct *rec )
{
    LjpScrollWinWidget sbw = (LjpScrollWinWidget)XtParent(w);

    if( sbw->composite.num_children < 1 )
	return;

    sbw->scrollwin.xoff = rec->value*sbw->scrollwin.xInc;
    XtMoveWidget( sbw->scrollwin.wchild, -sbw->scrollwin.xoff,
	-sbw->scrollwin.yoff );
}

static void vscroll_cb( Widget w, XtPointer closure,
    struct ljpGenericCallbackStruct *rec )
{
    LjpScrollWinWidget sbw = (LjpScrollWinWidget)XtParent(w);

    if( sbw->composite.num_children < 1 )
	return;

    sbw->scrollwin.yoff = rec->value*sbw->scrollwin.yInc;
    XtMoveWidget( sbw->scrollwin.wchild, -sbw->scrollwin.xoff,
	-sbw->scrollwin.yoff );
}
