/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include "TitulusP.h"

/*
** Widget hooks.
*/
static void hook_initialize( Widget, Widget, ArgList, Cardinal* );
static void hook_destroy( Widget w );
static void hook_resize( Widget w );
static void hook_redisplay( Widget, XEvent*, Region );
static Boolean hook_setValues( Widget, Widget, Widget, ArgList, Cardinal* );
static void hook_setValuesAlmost( Widget, Widget, XtWidgetGeometry*,
    XtWidgetGeometry* );
static XtGeometryResult hook_queryGeometry( Widget, XtWidgetGeometry*, 
    XtWidgetGeometry* );

static void draw_titulus( LjpTitulusWidget );
static void find_max( LjpTitulusWidget, Dimension*, Dimension* );
static void layout( LjpTitulusWidget, Boolean );

static XtResource resources[] =
{
    {
	XljpNchunks,
	XljpCChunks,
	XtRPointer,
	sizeof(XtPointer),
	XtOffsetOf(LjpTitulusRec,titulus.chunks),
	XtRImmediate,
	NULL
    }
};
	  
LjpTitulusClassRec ljpTitulusClassRec =
{
   {
      (WidgetClass) &ljpPrimitiveClassRec,      /* superclass         */
      "LjpTitulus",                            	/* class_name         */
      sizeof(LjpTitulusRec),                   	/* widget_size        */
      NULL,                          		/* class_initialize   */
      NULL,                            		/* class_part_initiali*/
      FALSE,                                    /* class_inited       */
      hook_initialize,                          /* initialize         */
      NULL,                                     /* initialize_hook    */
      XtInheritRealize,                         /* realize            */
      NULL,                             /* actions            */
      0,                   /* num_actions        */
      resources,                                /* resources          */
      XtNumber(resources),                      /* num_resources      */
      NULLQUARK,                                /* xrm_class          */
      TRUE,                                     /* compress_motion    */
      XtExposeCompressMaximal,                  /* compress_exposure  */
      TRUE,                                     /* compress_enterleave*/
      FALSE,                                    /* visible_interest   */
      hook_destroy,                             /* destroy            */
      hook_resize,                              /* resize             */
      hook_redisplay,                 		/* expose             */
      hook_setValues,                           /* set_values         */
      NULL,                                     /* set_values_hook    */
      NULL,                            		/* set_values_almost  */
      NULL,                                     /* get_values_hook    */
      NULL,                                     /* accept_focus       */
      XtVersion,                                /* version            */
      NULL,                                     /* callback_private   */
      NULL,					/* tm_table           */
      hook_queryGeometry,			/* query_geometry     */
      XtInheritDisplayAccelerator,              /* display accel      */
      NULL,                                     /* extension          */
    },
    /* LjpPrimitive class. */
    {
	XljpInheritDrawFrame,			/* draw_frame.		*/
	XljpInheritCalcInterior,		/* calc_interior.	*/
	NULL                                    /* extension		*/
    },
    /* LjpTitulus. */
    {
	NULL,                                   /* extension          */
    }
};

WidgetClass ljpTitulusWidgetClass = (WidgetClass)&ljpTitulusClassRec;

static void hook_initialize( Widget w, Widget neww, ArgList args,
    Cardinal *num_args )
{
    LjpTitulusWidget tw = (LjpTitulusWidget)w;
    LjpTitulusWidget new_tw = (LjpTitulusWidget)neww;

    /*
    fprintf( stdout, "LjpTitulus 0x%x hook initialize\n", neww );
    */

    /* Set default size (possibly based on initial chunks). */
    layout( tw, False );
}

static void hook_destroy( Widget w )
{
    LjpTitulusWidget dw = (LjpTitulusWidget)w;
}

static void hook_resize( Widget w )
{
}

static void hook_redisplay( Widget w, XEvent *event, Region region )
{
    LjpTitulusWidget tw = (LjpTitulusWidget)w;

    draw_titulus( tw );
}

static Boolean hook_setValues( Widget oldw, Widget w, Widget neww,
    ArgList args, Cardinal *num_args )
{
    LjpTitulusWidget old_dw = (LjpTitulusWidget) oldw;
    LjpTitulusWidget new_dw = (LjpTitulusWidget) neww;
    Boolean redisplay = False;

    if( new_dw->titulus.chunks != old_dw->titulus.chunks )
    {
	redisplay = True;
    }

    return redisplay;
}

static XtGeometryResult hook_queryGeometry( Widget w,
    XtWidgetGeometry *request, XtWidgetGeometry *reply )
{
    LjpTitulusWidget tw = (LjpTitulusWidget)w;

    if( ((request->request_mode&CWWidth) &&
	request->width==tw->core.width) &&
	((request->request_mode & CWHeight) &&
	request->height==tw->core.height) )
    {
	return XtGeometryNo;
    }

    *reply = *request;
    return XtGeometryYes; 
}

static void layout( LjpTitulusWidget tw, Boolean doit )
{
    Position xl, xr, yt, yb;
    Dimension nw, nh, iw, ih;
    LjpTitulusWidgetClass tc = (LjpTitulusWidgetClass)XtClass(tw);

    if( !XtIsRealized((Widget)tw) )
	doit = False;

    find_max( tw, &nw, &nh );
    tc->primitive_class.calc_interior( (Widget)tw, &xl, &yt, &xr, &yb,
	&iw, &ih );

    if( doit == False )
    {
	tw->core.width = nw+xl+xr;
	tw->core.height = nh+yt+yb;
	return;
    }

    if( nw > iw )
    {
	XtGeometryResult rc;
	XtWidgetGeometry request, reply;

	request.request_mode = CWWidth;
	request.width = nw+xl+xr;
	rc = XtMakeGeometryRequest( (Widget)tw, &request, &reply );
	if( rc == XtGeometryAlmost )
	    XtMakeGeometryRequest( (Widget)tw, &reply, &request );
    }
    if( nh > ih )
    {
	XtGeometryResult rc;
	XtWidgetGeometry request, reply;

	request.request_mode = CWHeight;
	request.height = nh+yt+yb;
	rc = XtMakeGeometryRequest( (Widget)tw, &request, &reply );
	if( rc == XtGeometryAlmost )
	    XtMakeGeometryRequest( (Widget)tw, &reply, &request );
    }
}

static void draw_titulus( LjpTitulusWidget tw )
{
    LjpTitulusWidgetClass tc = (LjpTitulusWidgetClass)XtClass(tw);
    struct XljpTitulusChunk *chunk;
    Display *dpy = XtDisplay(tw);
    Window win = XtWindow(tw);
    GC gc = tw->primitive.gcTop;
    Position x, y, sx, sy;
    Dimension width;
    int direction;
    int ascent, descent;
    XCharStruct rec;

    tc->primitive_class.calc_interior( (Widget)tw, &sx, &sy, NULL, NULL,
	&width, NULL );

    /*
    fprintf( stdout, "LjpTitulus draw interior x %d shadow %d\n",
	sx, tw->primitive.shadow_type );
    */

    tc->primitive_class.draw_frame( (Widget)tw );

    y = sy;

    for( chunk=tw->titulus.chunks; chunk!=NULL; chunk=chunk->next )
    {
	XSetForeground( dpy, gc, chunk->color );
	XSetFont( dpy, gc, chunk->font );
	XQueryTextExtents( dpy, chunk->font,
	    chunk->format, strlen(chunk->format),
	    &direction, &ascent, &descent, &rec );

	if( y == sy )
	    y += ascent;

	x = sx;
	if( chunk->alignment == XljpAlignCenter )
	{
	    if( width > rec.width ) 
		x = (width-rec.width)/2;
	}
	if( chunk->alignment == XljpAlignEnd )
	{
	    x = width-rec.width;
	}

	/*
	fprintf( stdout, "LjpTitulus draw next chunk x%d y%d \"%s\"\n",
	    x, y, chunk->format );
	*/
	XDrawString( dpy, win, gc, x, y, chunk->format,
	    strlen(chunk->format) );

	y += (ascent+descent+chunk->line_spacing);
    }

    /* Restore to cannonical values. */
    XSetForeground( dpy, gc, tw->primitive.pixelTopShadow );
    XSetFont( dpy, gc, tw->primitive.font->fid );
}

static void find_max( LjpTitulusWidget tw, Dimension *w, Dimension *h )
{
    int direction, ascent, descent;
    Dimension maxw=0, maxh=0;
    XCharStruct rec;
    struct XljpTitulusChunk *c;

    for( c=tw->titulus.chunks; c!=NULL; c=c->next )
    {
	XQueryTextExtents( XtDisplay(tw), c->font,
	    c->format, strlen(c->format),
	    &direction, &ascent, &descent, &rec );
	if( rec.width > maxw )
	    maxw = rec.width;
	maxh += (ascent+descent+c->line_spacing);
    }

    if( w != NULL )
	*w = maxw;
    if( h != NULL )
	*h = maxh;
}

/*
** Auxilary routines.
*/

int ljpTitulusAddChunk( Widget w, struct XljpTitulusChunk *chunk )
{
    LjpTitulusWidget tw = (LjpTitulusWidget)w;
#if 0
    {
	int i = 0;
	struct XljpTitulusChunk *c;

	for( c=chunk; c!=NULL; c=c->next )
	    c->font = tw->primitive.font->fid;
    }
#endif

    if( tw->titulus.chunks == NULL )
    {
	tw->titulus.chunks = chunk;
    }
    else
    {
	struct XljpTitulusChunk *c;

	for( c=tw->titulus.chunks; c->next!=NULL; c=c->next );
	c->next = chunk;
    }

    layout( tw, True );
    if( XtIsRealized((Widget)tw) )
	draw_titulus( tw );

    return 1;
}

/*
** ljpTitulusReplaceChunks
**
** Free existing chunks (chunks and format element must have been allocated
** with malloc. Call ljpTitulusAddChunk.
*/
int ljpTitulusReplaceChunks( Widget w, struct XljpTitulusChunk *chunks )
{
    LjpTitulusWidget tw = (LjpTitulusWidget)w;

    ljpTitulusFreeChunks( tw->titulus.chunks );
    tw->titulus.chunks = NULL;
    ljpTitulusAddChunk( w, chunks );
}

struct XljpTitulusChunk *ljpTitulusIndexChunk( struct XljpTitulusChunk *root,
    int index )
{
    int i;
    struct XljpTitulusChunk *c;

    for( i=0,c=root; i<index; ++i,c=c->next )
    {
	if( c->next == NULL )
	    break;
    }

    if( i == index )
	return c;
    return NULL;
}

struct XljpTitulusChunk *ljpTitulusMkChunks( const char *text[],
   struct XljpTitulusChunk *default_chunk )
{
    int i;
    struct XljpTitulusChunk *c, *cp, *cs;

    c = cp = cs = NULL;
    for( i=0; text[i]!=NULL; ++i )
    {
	c = (struct XljpTitulusChunk*)malloc(sizeof(struct XljpTitulusChunk));
	if( default_chunk != NULL )
	    *c = *default_chunk;
	if( cs == NULL )
	    cs = c;
	c->format = strdup(text[i]);
	if( cp != NULL )
	    cp->next = c;
	cp = c;
    }
    /*
    fprintf( stdout, "LjpTitulus made %d chunks\n", i );
    */

    if( c != NULL )
	c->next = NULL;

    return cs;
}

struct XljpTitulusChunk *ljpTitulusMkChunksFromString( const char *str,
   struct XljpTitulusChunk *default_chunk )
{
    const char *p1, *p2;
    char *n;
    int len;
    struct XljpTitulusChunk *c=NULL, *cs=NULL, *cp=NULL;

    p1 = str;
    while( (p2=strchr(p1,(int)'\n')) != NULL )
    {
	if( (len=p2-p1) == 0 )
	    len = 1;
	n = (char*)malloc( len+1 );
	strncpy( n, p1, len );
	n[len] = '\0';
	p1 = p2+1;
	c = (struct XljpTitulusChunk*)malloc(sizeof(struct XljpTitulusChunk));
	if( default_chunk != NULL )
	    *c = *default_chunk;
	c->format = (const char*)n;
	if( cs == NULL )
	    cs = c;
	if( cp != NULL )
	    cp->next = c;
	cp = c;
    }

    if( c != NULL )
	c->next = NULL;

    return cs;
}

/*
** ljpTitulusFreeChunks
**
** Assumes all chunks and thier format elements are dynamically allocated.
*/
int ljpTitulusFreeChunks( struct XljpTitulusChunk *chunks )
{
    struct XljpTitulusChunk *c, *c_prev;

    if( chunks == NULL )
	return 1;

    for( c=chunks; c!=NULL; )
    {
	if( c->format != NULL )
	    free( c->format );
	c_prev = c;
	free( c );
	c = c_prev->next;
    }

    return 0;
}
