/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

#include <stdio.h>
#include <stdlib.h>

#include <X11/X.h>
#include <X11/Xproto.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/xpm.h>
#include "LjpStringDefs.h"
#include "Button.h"
#include "Dial.h"
#include "Frame.h"
#include "GameWindow.h"
#include "Label.h"
#include "Larry.h"
#include "Row.h"
#include "Scrollbar.h"
#include "ScrollWin.h"
#include "Titulus.h"
#include "backing1.xpm"

struct XljpTitulusChunk *chunk1, *chunk2;

const char *S_Text1[] =
{
    "IMP CAESAR",
    "DIVI NERVAE FIL NERVA",
    "TRAIANVS AVG GERM DAC",
    "ICVS PONTIFEX MAXIMVS TRIBV",
    "NICIAE POTESTATIS XII IMP VI COS V P P",
    "PORTAM PER LEG VIIII HISP FECIT",
    NULL
};

static const char *S_FontFixed1[] =
{
    "-adobe-Courier-Medium-r-normal--0-0-0-0-m-0-iso8859-1",
    "-adobe-courier-medium-r-normal--17-120-100-100-m-100-iso8859-1",
    "8x13",
    "6x13",
    "fixed",
    NULL
};

static const char *S_FontVariable1[] =
{
    "-adobe-Times-Roman-r-normal--0-0-0-0-p-0-iso8859-1",
    "-adobe-times-medium-r-normal--17-120-100-100-p-84-iso8859-1",
    "variable",
    NULL
};

static const char *S_FontVariable1small[] =
{
    "-adobe-Times-Roman-r-normal--0-0-0-0-p-0-iso8859-1",
    "variable",
    NULL
};

static const char *S_FontVariable2[] =
{
    "-adobe-Helvetica-Medium-r-normal--0-0-0-0-p-0-iso8859-1",
    "-adobe-helvetica-medium-r-normal--17-120-100-100-p-88-iso8859-1",
    "variable",
    NULL
};

static Display *display;
static Widget wShell;
static Widget wTop;
static Widget wBox;
static Widget wQuit, wTry;
static Widget wLarry = NULL;
static Widget wLarryOut = NULL;
static Widget wLarryRelief = NULL;
static XFontStruct *fonts[3];

static Pixel pxBlack;
static Pixel pxWhite;
static Pixel pxLGray;
static Pixel pxDGray;

static Widget mkLarry( Widget );
static Widget mkLarryFrame( Widget, enum XljpShadowType );
static void quit_cb( Widget, XtPointer, struct ljpButtonCallbackStruct* );
static void popdown_cb( Widget, XtPointer, struct ljpButtonCallbackStruct* );
static void try_cb( Widget, XtPointer, struct ljpButtonCallbackStruct* );
static void dialValue_cb( Widget, XtPointer,
    struct ljpGenericCallbackStruct* );
static void scrollValue_cb( Widget, XtPointer,
    struct ljpGenericCallbackStruct* );
static void popup_frame_cb( Widget, XtPointer,
    struct ljpButtonCallbackStruct* );
static void unmap_cb( Widget, XtPointer, XEvent*, Boolean* );
static XFontStruct *font_available( const char *[] );

int main( int argc, char *argv[] )
{
    Display *dpy;
    XtAppContext app;
    Pixmap pixBacking;
    Widget wFrame1;
    Widget wFrame2;

    /*
    ** Open display.
    */
    if( (dpy=XOpenDisplay(":0.0")) == NULL )
    {
	fprintf( stdout, "ERROR. Can not open display.\n" );
	return 0;
    }
    display = dpy;

    /* 24 bit true color. */
    pxBlack = 0x000000;
    pxWhite = 0xffffff;
    pxLGray = 0xcbcbcb;
    pxDGray = 0xababab;

#if 0
    /* Pseudo color. */
    pxBlack = 0;
    pxWhite = 1;
    pxLGray = 2;
    pxDGray = 3;
#endif


    XtToolkitInitialize();
    app = XtCreateApplicationContext();
    XtDisplayInitialize(app,dpy,argv[0],"Test",NULL,0,argc,argv);
    {
	int i;
	Arg args[16];

	i = 0;
	XtSetArg( args[i], XtNinput, True ); i++;
	if( (wShell=XtAppCreateShell(argv[0],"Test",
	    applicationShellWidgetClass,dpy,args,i)) == NULL )
	{
	    fputs( "ERROR. Bad XtAppCreateShell.\n", stdout );
	    return 0;
	}
    }

    {
	int rc;

	if( (rc=XpmCreatePixmapFromData(display,
	    DefaultRootWindow(display),backing_xpm,
	    &pixBacking,NULL,NULL)) != XpmSuccess )
	{
	    fprintf( stdout, "ERROR. Bad XpmCreatePixmapFromData. \"%s\".\n",
		XpmGetErrorString(rc) );
	}
    }

    fonts[0] = font_available(S_FontFixed1);
    fonts[1] = font_available(S_FontVariable1);
    fonts[2] = font_available(S_FontVariable2);

    {
	int i;
	Arg args[16];
	Widget w, wUpper, wUpper1, wUpper2, wLower;

	i = 0;
	XtSetArg( args[i], XtNbackground, pxLGray ); ++i;
	XtSetArg( args[i], XtNbackgroundPixmap, pixBacking ); ++i;
	wTop = XtCreateManagedWidget(
	    "game_window", ljpGameWindowWidgetClass, wShell, args, i );

	i = 0;
	XtSetArg( args[i], XtNbackground, pxLGray ); ++i;
	XtSetArg( args[i], XljpNorientation, XljpOrientVertical ); ++i;
	XtSetArg( args[i], XljpNhfit, XljpFitExpand ); ++i;
	XtSetArg( args[i], XljpNspacing, 4 ); ++i;
	XtSetArg( args[i], XljpNmarginWidth, 2 ); ++i;
	XtSetArg( args[i], XljpNmarginHeight, 2 ); ++i;
	wUpper = XtCreateManagedWidget(
	    "upper_menu", ljpRowWidgetClass, wTop, args, i );
	/*
	XtVaSetValues( wUpper, XtNborderWidth, 1, NULL );
	*/

	/*
	** Upper row. Top half.
	*/

	i = 0;
	XtSetArg( args[i], XtNbackground, pxDGray ); ++i;
	XtSetArg( args[i], XljpNorientation, XljpOrientHorizontal ); ++i;
	XtSetArg( args[i], XljpNhalignment, XljpAlignCenter ); ++i;
	XtSetArg( args[i], XljpNspacing, 2 ); ++i;
	XtSetArg( args[i], XljpNmarginWidth, 2 ); ++i;
	XtSetArg( args[i], XljpNmarginHeight, 2 ); ++i;
	wUpper1 = XtCreateManagedWidget(
	    "upper_menu1", ljpRowWidgetClass, wUpper, args, i );
	XtVaSetValues( wUpper1, XtNborderWidth, 0, NULL );

	i = 0;
	XtSetArg( args[i], XljpNlabel, "label beginning" ); ++i;
	XtSetArg( args[i], XljpNhalignment, XljpAlignBeginning ); ++i;
	XtSetArg( args[i], XtNbackground, pxBlack ); ++i;
	XtSetArg( args[i], XtNforeground, pxWhite ); ++i;
	XtSetArg( args[i], XtNborderWidth, 0 ); ++i;
	w = XtCreateManagedWidget(
	    "label_title", ljpLabelWidgetClass, wUpper1, args, i );
	/* Smaller than above so that alignment shows. */
	XtVaSetValues( w, XljpNlabel, "beginning", NULL );

	i = 0;
	XtSetArg( args[i], XljpNlabel, "label center" ); ++i;
	XtSetArg( args[i], XljpNhalignment, XljpAlignCenter ); ++i;
	XtSetArg( args[i], XtNbackground, pxBlack ); ++i;
	XtSetArg( args[i], XtNforeground, pxWhite ); ++i;
	XtSetArg( args[i], XtNborderWidth, 0 ); ++i;
	w = XtCreateManagedWidget(
	    "label_label", ljpLabelWidgetClass, wUpper1, args, i );
	/* Smaller than above so that alignment shows. */
	XtVaSetValues( w, XljpNlabel, "center", NULL );

	i = 0;
	XtSetArg( args[i], XljpNlabel, "label end" ); ++i;
	XtSetArg( args[i], XljpNhalignment, XljpAlignEnd ); ++i;
	XtSetArg( args[i], XtNbackground, pxBlack ); ++i;
	XtSetArg( args[i], XtNforeground, pxWhite ); ++i;
	XtSetArg( args[i], XtNborderWidth, 0 ); ++i;
	w = XtCreateManagedWidget(
	    "label_label", ljpLabelWidgetClass, wUpper1, args, i );
	/* Smaller than above so that alignment shows. */
	XtVaSetValues( w, XljpNlabel, "end", NULL );

	/*
	** Upper row. Bottom half.
	*/

	i = 0;
	XtSetArg( args[i], XtNbackground, pxWhite ); ++i;
	XtSetArg( args[i], XljpNorientation, XljpOrientHorizontal ); ++i;
	XtSetArg( args[i], XljpNhalignment, XljpAlignEnd ); ++i;
	XtSetArg( args[i], XljpNspacing, 4 ); ++i;
	wUpper2 = XtCreateManagedWidget(
	    "upper_menu2", ljpRowWidgetClass, wUpper, args, i );
	XtVaSetValues( wUpper2, XtNborderWidth, 1, NULL );

	i = 0;
	XtSetArg( args[i], XljpNlabel, "Two h rows of labels in a v row." );
	++i;
	XtSetArg( args[i], XtNbackground, pxLGray ); ++i;
	XtSetArg( args[i], XtNborderWidth, 0 ); ++i;
	XtCreateManagedWidget(
	    "label_ltitle", ljpLabelWidgetClass, wUpper2, args, i );

	/*
	*/

	i = 0;
	XtSetArg( args[i], XtNwidth, 600 ); ++i;
	XtSetArg( args[i], XtNheight, 400 ); ++i;
	XtSetArg( args[i], XljpNorientation, XljpOrientHorizontal ); ++i;
	XtSetArg( args[i], XljpNspacing, 8 ); ++i;
	wBox = XtCreateManagedWidget(
	    "mainBox", ljpRowWidgetClass, wTop, args, i );

	{
	    Widget sbw, w;
	    struct XljpTitulusChunk default_chunk;
	    struct XljpTitulusChunk *c;

	    i = 0;
	    XtSetArg( args[i], XtNwidth, 100 ); ++i;
	    XtSetArg( args[i], XtNheight, 100 ); ++i;
	    XtSetArg( args[i], XtNbackground, pxLGray ); ++i;
	    sbw = XtCreateManagedWidget(
		"scrollbox", ljpScrollWinWidgetClass, wBox, args, i );

	    i = 0;
	    XtSetArg( args[i], XljpNhorizontalScrollBar, &w ); ++i;
	    XtGetValues( sbw, args, i );

	    i = 0;
	    XtSetArg( args[i], XtNforeground, pxLGray ); ++i;
	    XtSetArg( args[i], XtNbackground, pxDGray ); ++i;
	    XtSetArg( args[i], XtNborderWidth, 0 ); ++i;
	    XtSetArg( args[i], XljpNtopShadowColor, pxWhite ); ++i;
	    XtSetArg( args[i], XljpNbottomShadowColor, pxBlack ); ++i;
	    XtSetArg( args[i], XljpNshadowThickness, 1 ); ++i;
	    XtSetArg( args[i], XtNheight, 15 ); ++i;
	    XtSetValues( w, args, i );

	    i = 0;
	    XtSetArg( args[i], XljpNverticalScrollBar, &w ); ++i;
	    XtGetValues( sbw, args, i );

	    i = 0;
	    XtSetArg( args[i], XtNforeground, pxLGray ); ++i;
	    XtSetArg( args[i], XtNbackground, pxDGray ); ++i;
	    XtSetArg( args[i], XtNborderWidth, 0 ); ++i;
	    XtSetArg( args[i], XljpNtopShadowColor, pxWhite ); ++i;
	    XtSetArg( args[i], XljpNbottomShadowColor, pxBlack ); ++i;
	    XtSetArg( args[i], XljpNshadowThickness, 1 ); ++i;
	    XtSetArg( args[i], XtNwidth, 15 ); ++i;
	    XtSetValues( w, args, i );

#if 0
	    i = 0;
	    XtSetArg( args[i], XtNbackground, pxLGray ); ++i;
	    XtSetArg( args[i], XtNborderWidth, 0 ); ++i;
	    XtSetArg( args[i], XtNwidth, 128 ); ++i;
	    XtSetArg( args[i], XtNheight, 128 ); ++i;
	    XtSetArg( args[i], XtNbackgroundPixmap, pixBacking ); ++i;
	    XtCreateManagedWidget(
		"scrolled_child", widgetClass, sbw, args, i );
#endif

	    i = 0;
	    XtSetArg( args[i], XtNwidth, 500 ); ++i;
	    XtSetArg( args[i], XtNheight, 180 ); ++i;
	    XtSetArg( args[i], XtNforeground, pxBlack ); ++i;
	    XtSetArg( args[i], XtNbackground, pxLGray ); ++i;
	    XtSetArg( args[i], XtNbackgroundPixmap, pixBacking ); ++i;
	    w = XtCreateManagedWidget(
		"titulus", ljpTitulusWidgetClass, sbw, args, i );

	    default_chunk.alignment = XljpAlignCenter;
	    default_chunk.color = pxLGray;
	    default_chunk.line_spacing = 5;
	    default_chunk.font = fonts[0]->fid;

	    chunk1 = ljpTitulusMkChunks( S_Text1, &default_chunk );
	    c = ljpTitulusIndexChunk( chunk1, 0 );
	    c->color = pxWhite;
	    ljpTitulusAddChunk( w, chunk1 );
	}

	i = 0;
	XtSetArg( args[i], XtNbackground, pxLGray ); ++i;
	XtSetArg( args[i], XljpNshadowThickness, 2 ); ++i;
	w = XtCreateManagedWidget(
	    "labelbar_frame", ljpFrameWidgetClass, wTop, args, i );
	i = 0;
	XtSetArg( args[i], XtNbackground, pxLGray ); ++i;
	XtSetArg( args[i], XljpNorientation, XljpOrientHorizontal ); ++i;
	XtSetArg( args[i], XljpNspacing, 8 ); ++i;
	XtSetArg( args[i], XljpNmarginWidth, 4 ); ++i;
	XtSetArg( args[i], XljpNhalignment, XljpAlignBeginning ); ++i;
	XtSetArg( args[i], XljpNvalignment, XljpAlignCenter ); ++i;
	wLower = XtCreateManagedWidget(
	    "labelbar", ljpRowWidgetClass, w, args, i );

	i = 0;
	XtSetArg( args[i], XtNforeground, pxWhite ); ++i;
	XtSetArg( args[i], XtNbackground, pxLGray ); ++i;
	XtSetArg( args[i], XljpNlabel, "Quit" ); ++i;
	XtSetArg( args[i], XljpNarmColor, pxDGray ); ++i;
	XtSetArg( args[i], XljpNshadowThickness, 1 ); ++i;
	XtSetArg( args[i], XljpNmarginTop, 0 ); ++i;
	XtSetArg( args[i], XljpNmarginBottom, 0 ); ++i;
	XtSetArg( args[i], XljpNmarginLeft, 0 ); ++i;
	XtSetArg( args[i], XljpNmarginRight, 0 ); ++i;
	wQuit = XtCreateManagedWidget(
	    "quit_button", ljpButtonWidgetClass, wLower, args, i);

	i = 0;
	XtSetArg( args[i], XtNforeground, pxBlack ); ++i;
	XtSetArg( args[i], XtNbackground, pxLGray ); ++i;
	XtSetArg( args[i], XljpNlabel, "Popup" ); ++i;
	XtSetArg( args[i], XljpNarmColor, pxDGray ); ++i;
	XtSetArg( args[i], XljpNshadowThickness, 2 ); ++i;
	XtSetArg( args[i], XljpNmarginTop, 4 ); ++i;
	XtSetArg( args[i], XljpNmarginBottom, 4 ); ++i;
	XtSetArg( args[i], XljpNmarginLeft, 4 ); ++i;
	XtSetArg( args[i], XljpNmarginRight, 4 ); ++i;
	wTry = XtCreateManagedWidget(
	    "try_button", ljpButtonWidgetClass, wLower, args, i );

	i = 0;
	XtSetArg( args[i], XtNforeground, pxBlack ); ++i;
	XtSetArg( args[i], XtNbackground, pxLGray ); ++i;
	XtSetArg( args[i], XljpNlabel, "ShadowOut" ); ++i;
	XtSetArg( args[i], XljpNarmColor, pxDGray ); ++i;
	XtSetArg( args[i], XljpNshadowThickness, 2 ); ++i;
	XtSetArg( args[i], XljpNmarginTop, 4 ); ++i;
	XtSetArg( args[i], XljpNmarginBottom, 4 ); ++i;
	XtSetArg( args[i], XljpNmarginLeft, 4 ); ++i;
	XtSetArg( args[i], XljpNmarginRight, 4 ); ++i;
	XtSetArg( args[i], XljpNfont, fonts[2] ); ++i;
	wFrame1 = XtCreateManagedWidget(
	    "frame_button", ljpButtonWidgetClass, wLower, args, i );

	i = 0;
	XtSetArg( args[i], XtNforeground, pxBlack ); ++i;
	XtSetArg( args[i], XtNbackground, pxLGray ); ++i;
	XtSetArg( args[i], XljpNlabel, "ShadowRelief" ); ++i;
	XtSetArg( args[i], XljpNarmColor, pxDGray ); ++i;
	XtSetArg( args[i], XljpNshadowThickness, 2 ); ++i;
	XtSetArg( args[i], XljpNmarginTop, 4 ); ++i;
	XtSetArg( args[i], XljpNmarginBottom, 4 ); ++i;
	XtSetArg( args[i], XljpNmarginLeft, 4 ); ++i;
	XtSetArg( args[i], XljpNmarginRight, 4 ); ++i;
	XtSetArg( args[i], XljpNfont, fonts[2] ); ++i;
	wFrame2 = XtCreateManagedWidget(
	    "frame_button", ljpButtonWidgetClass, wLower, args, i );

#if 0

	i = 0;
	XtSetArg( args[i], XljpNshadowThickness, 4 ); ++i;
	w = XtCreateManagedWidget(
	    "label_frame", ljpFrameWidgetClass, wBox, args, i );

	i = 0;
	XtSetArg( args[i], XljpNlabel, "LABEL" ); ++i;
	XtSetArg( args[i], XtNborderWidth, 0 ); ++i;
	XtCreateManagedWidget(
	    "label_label", ljpLabelWidgetClass, w, args, i );
#endif
    }

    XtRealizeWidget( wShell );

    XtAddCallback( wQuit, XljpNactivateCallback,
	(XtCallbackProc)quit_cb, NULL );
    XtAddCallback( wTry, XljpNactivateCallback,
	(XtCallbackProc)try_cb, NULL );
    XtAddCallback( wFrame1, XljpNactivateCallback,
	(XtCallbackProc)popup_frame_cb, (void*)XljpShadowOut );
    XtAddCallback( wFrame2, XljpNactivateCallback,
	(XtCallbackProc)popup_frame_cb, (void*)XljpShadowRelief );
    XtAddEventHandler( wShell, StructureNotifyMask, 0, unmap_cb, NULL );

    XtAppMainLoop( app );

    return 1;
}

static void unmap_cb( Widget w, XtPointer client_data,
    XEvent *event, Boolean *bool )
{
    if( event->type != UnmapNotify )
	return;
    if( wLarry != NULL )
	XtPopdown( wLarry );
}

void quit_cb( Widget w, XtPointer closure,
    struct ljpButtonCallbackStruct *rec )
{
    if( rec->reason != XljpCR_RELEASE )
	return;
    exit( 0 );
}

void try_cb( Widget w, XtPointer closure,
    struct ljpButtonCallbackStruct *rec )
{
    if( rec->reason != XljpCR_RELEASE )
	return;
    if( wLarry == NULL )
	wLarry = mkLarry( wShell );
    XtPopup( wLarry, XtGrabNone );
}

void popup_frame_cb( Widget w, XtPointer closure,
    struct ljpButtonCallbackStruct *rec )
{
    enum XljpShadowType type = (enum XljpShadowType)closure;

    if( rec->reason != XljpCR_RELEASE )
	return;

    if( type == XljpShadowOut )
    {
	if( wLarryOut == NULL )
	    wLarryOut = mkLarryFrame( wShell, XljpShadowOut );
	XtPopup( wLarryOut, XtGrabNone );
    }

    if( type == XljpShadowRelief )
    {
	if( wLarryRelief == NULL )
	    wLarryRelief = mkLarryFrame( wShell, XljpShadowRelief );
	XtPopup( wLarryRelief, XtGrabNone );
    }
}

void dialValue_cb( Widget w, XtPointer closure,
    struct ljpGenericCallbackStruct *rec )
{
    char str[15+1];
    Widget wLabel = (Widget)closure;

    sprintf( str, "%03d", rec->value );
    XtVaSetValues( wLabel, XljpNlabel, str, NULL );
}

void scrollValue_cb( Widget w, XtPointer closure,
    struct ljpGenericCallbackStruct *rec )
{
}

void popdown_cb( Widget w, XtPointer closure,
    struct ljpButtonCallbackStruct *rec )
{
    if( rec->reason != XljpCR_RELEASE )
	return;
    XtPopdown( (Widget)closure );
}

Widget mkLarry( Widget wParent )
{
    const int sw = 2*3;
    int i;
    Arg args[16];
    Widget w;
    Widget wBox;
    Widget wDialValue, wDial;
    Widget wL;

    i = 0;
    XtSetArg( args[i], XtNwidth, 100+(2*sw) ); ++i;
    /*XtSetArg( args[i], XtNborderWidth, 5 ); ++i;*/
    XtSetArg( args[i], XtNborderColor, pxBlack ); ++i;
    XtSetArg( args[i], XtNbackground, pxLGray ); ++i;
    wL = XtCreatePopupShell( "lw", ljpLarryWidgetClass, wParent, args, i );

    i = 0;
    XtSetArg( args[i], XljpNshadowType, XljpShadowRelief ); ++i;
    XtSetArg( args[i], XljpNshadowThickness, 2 ); ++i;
    XtSetArg( args[i], XtNbackground, pxLGray ); ++i;
    w = XtCreateManagedWidget(
	"popup_frame", ljpFrameWidgetClass, wL, args, i );

    i = 0;
    XtSetArg( args[i], XtNborderWidth, 0 ); ++i;
    XtSetArg( args[i], XtNbackground, pxLGray ); ++i;
    XtSetArg( args[i], XtNforeground, pxWhite ); ++i;
    XtSetArg( args[i], XljpNorientation, XljpOrientVertical ); ++i;
    XtSetArg( args[i], XljpNspacing, 6 ); ++i;
    wBox = XtCreateManagedWidget(
	"child", ljpRowWidgetClass, w, args, i );

    i = 0;
    XtSetArg( args[i], XtNborderWidth, 0 ); ++i;
    XtSetArg( args[i], XtNbackground, pxLGray ); ++i;
    XtSetArg( args[i], XljpNlabel, "Dial" ); ++i;
    XtSetArg( args[i], XljpNfont, fonts[1] ); ++i;
    w = XtCreateManagedWidget(
	"title", ljpLabelWidgetClass, wBox, args, i );

    i = 0;
    XtSetArg( args[i], XtNborderWidth, 0 ); ++i;
    XtSetArg( args[i], XtNbackground, pxLGray ); ++i;
    XtSetArg( args[i], XljpNdialColor, pxLGray ); ++i;
    wDial = XtCreateManagedWidget(
	"dial", ljpDialWidgetClass, wBox, args, i );

    i = 0;
    XtSetArg( args[i], XtNforeground, pxWhite ); ++i;
    XtSetArg( args[i], XtNbackground, pxDGray ); ++i;
    XtSetArg( args[i], XtNborderWidth, 0 ); ++i;
    XtSetArg( args[i], XljpNlabel, "000" ); ++i;
    XtSetArg( args[i], XljpNmarginTop, 4 ); ++i;
    XtSetArg( args[i], XljpNmarginBottom, 4 ); ++i;
    XtSetArg( args[i], XljpNmarginLeft, 8 ); ++i;
    XtSetArg( args[i], XljpNmarginRight, 8 ); ++i;
    wDialValue = XtCreateManagedWidget(
	"dial_value", ljpLabelWidgetClass, wBox, args, i );

    XtAddCallback( wDial, XljpNactivateCallback,
	(XtCallbackProc)dialValue_cb, wDialValue );

    i = 0;
    XtSetArg( args[i], XtNforeground, pxLGray ); ++i;
    XtSetArg( args[i], XtNbackground, pxDGray ); ++i;
    XtSetArg( args[i], XtNborderWidth, 0 ); ++i;
    XtSetArg( args[i], XljpNtopShadowColor, pxWhite ); ++i;
    XtSetArg( args[i], XljpNbottomShadowColor, pxBlack ); ++i;
    XtSetArg( args[i], XljpNshadowThickness, 1 ); ++i;
    w = XtCreateManagedWidget(
	"scrollbar", ljpScrollbarWidgetClass, wBox, args, i );

    XtAddCallback( w, XljpNactivateCallback,
	(XtCallbackProc)scrollValue_cb, wL );

    i = 0;
    XtSetArg( args[i], XtNforeground, pxWhite ); ++i;
    XtSetArg( args[i], XtNbackground, pxLGray ); ++i;
    XtSetArg( args[i], XljpNlabel, "Popdown" ); ++i;
    XtSetArg( args[i], XljpNarmColor, pxDGray ); ++i;
    XtSetArg( args[i], XljpNshadowThickness, 2 ); ++i;
    XtSetArg( args[i], XljpNmarginTop, 4 ); ++i;
    XtSetArg( args[i], XljpNmarginBottom, 4 ); ++i;
    XtSetArg( args[i], XljpNmarginLeft, 4 ); ++i;
    XtSetArg( args[i], XljpNmarginRight, 4 ); ++i;
    w = XtCreateManagedWidget(
	"popdown_button", ljpButtonWidgetClass, wBox, args, i);

    XtAddCallback( w, XljpNactivateCallback,
	(XtCallbackProc)popdown_cb, wL );

    return wL;
}

Widget mkLarryFrame( Widget wParent, enum XljpShadowType type )
{
    int i;
    Arg args[16];
    Widget wButton;
    Widget wBox;
    Widget wL;
    Widget w;

    i = 0;
    XtSetArg( args[i], XtNwidth, 100+4 ); ++i;
    XtSetArg( args[i], XtNborderWidth, 1 ); ++i;
    XtSetArg( args[i], XtNborderColor, pxBlack ); ++i;
    XtSetArg( args[i], XtNbackground, pxLGray ); ++i;
    XtSetArg( args[i], XljpNlabel,"Popdown" ); ++i;
    XtSetArg( args[i], XljpNpopdownHeight, 38 ); ++i;
    wL = XtCreatePopupShell( "lw_frame", ljpLarryWidgetClass,
	wParent, args, i );

    i = 0;
    XtSetArg( args[i], XtNforeground, pxWhite ); ++i;
    XtSetArg( args[i], XtNbackground, pxLGray ); ++i;
    XtSetArg( args[i], XljpNlabel, "Popdown" ); ++i;
    XtSetArg( args[i], XljpNarmColor, pxDGray ); ++i;
    XtSetArg( args[i], XljpNshadowThickness, 2 ); ++i;
    XtSetArg( args[i], XljpNmarginTop, 4 ); ++i;
    XtSetArg( args[i], XljpNmarginBottom, 4 ); ++i;
    wButton = XtCreateManagedWidget(
	"popdown_button", ljpButtonWidgetClass, wL, args, i);

    i = 0;
    XtSetArg( args[i], XljpNshadowType, type ); ++i;
    XtSetArg( args[i], XljpNshadowThickness, 3 ); ++i;
    XtSetArg( args[i], XljpNtopShadowColor, pxWhite ); ++i;
    XtSetArg( args[i], XljpNbottomShadowColor, pxBlack ); ++i;
    XtSetArg( args[i], XtNbackground, pxLGray ); ++i;
    w = XtCreateManagedWidget(
	"popup_frame", ljpFrameWidgetClass, wL, args, i );

    XtAddCallback( wButton, XljpNactivateCallback,
	(XtCallbackProc)popdown_cb, wL );

    return wL;
}

static XFontStruct *font_available( const char *list[] )
{
    int i;
    XFontStruct *fs = NULL;

    for( i=0; list[i]!=NULL; ++i )
    {
	if( (fs=XLoadQueryFont(display,list[i])) != NULL )
	    break;
    }
    return fs;
}
