/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

/*
** C++ class AnimList.
**
** This class, a superclass of TileList, maintains a list of tiles for
** the purposes of animation. A member function is called for each
** tick of the animation clock. On each tick, this class will draw each
** tile according to the sync data found in the tile.
*/

#include "animList.h"

AnimList::AnimList( Imago *nimago )
{
    imago = nimago;
    sync_tags[0] = 0;
    sync_tags[1] = 2;
    sync_tags[2] = 4;
}

int AnimList::incanim()
{
    int cntAnim, cntSet, cntStatic, cntOther, cntPipe;
    struct tinode *p = root;
    XImage *fimg = NULL;
    Tile *tile;

    //fprintf( stdout, "TMP al anim begin sync2 %d \n", sync_tags[0] );
    cntAnim = cntSet = cntStatic = cntOther = cntPipe = 0;
    while( p != NULL )
    {
	tile = p->tile;
	if( tile->tobj->tclass == tobj_tclass_pipe )
	    ++cntPipe;

	switch( tile->state )
	{
	    case tile_state_set:
		fimg = tile->tobj->incanim( tile->state_info );
		imago->copy( fimg, tile->x, tile->y, tile->width,
		    tile->height );
		tile->state = tile_state_static;
		p = pop( p );
		++cntSet;
		break;

	    case tile_state_anim:
		/*
		if( tile->tobj->tclass == tobj_tclass_meter )
		    fprintf( stdout, "TMP al anim meter tag %d\n",
			tile->sync_tag );
		*/
		if( (tile->sync_tag!=-1) && (sync_tags[tile->sync_tag]==0) )
		    tile->sync_tag = -1;
		if( tile->sync_tag == -1 )
		    fimg = tile->tobj->incanim( tile->state_info );
		else
		   fimg = tile->tobj->latest( tile->state_info );
		imago->copy( fimg, tile->x, tile->y, tile->width,
		    tile->height );
		p = p->next;
		++cntAnim;
		break;

	    case tile_state_static:
		// State may change after being added to list.
		p = pop( p );
		++cntStatic;
		break;

	    case tile_state_null:
	    default:
		// FUTURE. Error.
		p = pop( p );
		++cntOther;
		break;
	}
    }
    /*
    fprintf( stdout,
	"TMP al anim end anim:%d set:%d static:%d -  pipe:%d\n",
	cntAnim, cntSet, cntStatic, cntPipe );
    */
    --sync_tags[1];
    if( sync_tags[1] < 0 )
	sync_tags[1] = 2;
    --sync_tags[2];
    if( sync_tags[2] < 0 )
	sync_tags[2] = 4;

    return 1;
}

void AnimList::set_imago( Imago *i )
{
    imago = i;
}
