/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

#ifndef ARTWORK_H
#define ARTWORK_H

/*
** Artwork C++ class.
**
** This class holds the data found in an XPipeman Artwork glob. These
** globs contain all of the information (images, sound, tile data) for
** one Artwork block. These are not levels, as Artwork may be reused
** for several XPipeman levels.
**
** This class is tightly coupled with the Game object and has no use
** outside of XPipeman.
**
** When creating an instance, the filename of a Glob file is passed in.
** This object will set the current cursor, read the artwork config
** component and set up the globally accessable tcache, register sounds,
** etc.
*/

#define ARTWORK_MAX_CONFLINE  127
#define ARTWORK_MAX_OBSTACLES   8
#define ARTWORK_MAX_PIPES      16
#define ARTWORK_MAX_SYNCS       1
#define ARTWORK_MAX_PIXMAPNAME 31
#define ARTWORK_MAX_NAME       31

#define ARTWORK_STANZA_NULL       0
#define ARTWORK_STANZA_OBSTACLES  1
#define ARTWORK_STANZA_SOUNDS     2
#define ARTWORK_STANZA_PIPES      3
#define ARTWORK_STANZA_SYNCS      4

#define ARTWORK_SND_NULL      0
#define ARTWORK_SND_FIRST     5
#define ARTWORK_SND_LEVELOVER (ARTWORK_SND_FIRST+0)
#define ARTWORK_SND_BADPLACE  (ARTWORK_SND_FIRST+1)
#define ARTWORK_SND_CROSS     (ARTWORK_SND_FIRST+2)
#define ARTWORK_SND_FLOWCNT   (ARTWORK_SND_FIRST+3)
#define ARTWORK_SND_FLOWSTART (ARTWORK_SND_FIRST+4)
#define ARTWORK_SND_PLACE     (ARTWORK_SND_FIRST+5)
#define ARTWORK_SND_GAMEOVER  (ARTWORK_SND_FIRST+6)
#define ARTWORK_SND_SHIFT     (ARTWORK_SND_FIRST+7)
#define ARTWORK_MAX_SOUNDS    ARTWORK_SND_SHIFT

#include <ctype.h>
#include <string.h>
#include "glob.h"

/* Config data from glob. */
struct intlevel
{
    int obscnt;
    char *obstacles[ARTWORK_MAX_OBSTACLES]; /* TObj ids. */
    int pipecnt;
    char *pipes[ARTWORK_MAX_PIPES];   /* TObj ids. */
    char *sounds[ARTWORK_MAX_SOUNDS];
};

class Artwork
{
    struct intlevel *parse_conf( const char*, int );
    char *parse_identifier( char* );
    void free_intlevel( struct intlevel* );
    int level;
    int nameToTag( const char* );
public:
    Glob *glob;
    int obscnt;
    int pipecnt;
    int synccnt;
    int obstacles[ARTWORK_MAX_OBSTACLES];
    int pipes[ARTWORK_MAX_PIPES];
    int syncs[ARTWORK_MAX_SYNCS];
    Artwork( const char*, int );
    ~Artwork();
    int initialized( void );
};

#endif
