/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

#ifndef BOARD_H
#define BOARD_H

/*
** Board C++ class.
**
** Lays out arrangement of Tile objects in form seen on screen. Contains
** an AnimList instance to manage animation of tiles. Contains an Imago
** instance. Given a Colores instance, X11 settings, and a TCache to
** manage the TObjs.
*/

#include <X11/Intrinsic.h>
#include "colores.h"
#include "imago.h"
#include "tcache.h"
#include "tile.h"
#include "animList.h"

enum board_mode
{
    board_mode_null = 0,
    board_mode_title,
    board_mode_backdraw,
    board_mode_initmelt,
    board_mode_melt,
    board_mode_animate,
    board_mode_pause
};

#define BOARD_WIDTH  12
#define BOARD_HEIGHT 10

#define BOARD_TILE_NULL   -1
#define BOARD_TILE_EMPTY   0
#define BOARD_TILE_TBAR    1
#define BOARD_TILE_BBAR    2
#define BOARD_TILE_RBAR    3
#define BOARD_TILE_METER   4
#define BOARD_TILE_START   5
#define BOARD_TILE_DEATH   6

// FUTURE. Interface to be decided.
extern struct xface xface;

class Board
{
    int locality; /* Non zero indicates local X server. */
    int metercount;
    Tile *placed;
    TObj *backtobj;
    Imago *imago;
    void *effect;
    void *effectMelt;
    enum board_mode mode;
    AnimList *animList;
    XVisualInfo *vinfo;
    Window win;
    Display *display;
    Colores *colores;
    TCache *tcache;
public:
    static void boardExpose_cb( Widget, XtPointer, XEvent* );
    Board( Display*, XVisualInfo*, Window, int, TCache*, Colores* );
    ~Board();
    int (*externSetAnim)( int, Board* );
    int set( Tile*, int );
    int set( int, int, int );
    int set( Tile*, TObj* );
    void reset( int );
    int title( const char* );
    int pause( void );
    int place( int, int );
    void shift( TObj* );
    int incanim( void );
    int incflow( Tile*, tobj_dir*, struct tobj_gain* );
    int incmeter( void );
    Tile playfield[BOARD_WIDTH][BOARD_HEIGHT];
    Tile deck[10];
    Tile ntile;     // Next tile (top of deck).
    Tile meter[3];  // Flow meter. Bottom is index zero.
    Tile tbar;      // Above flowmeter.
    Tile bbar;      // Below flowmeter.
    Tile rbar;      // Right of deck.
    Tile *start;    // Start tile.
};

#endif
