/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

#ifndef GAME_H
#define GAME_H 1

/*
** Game C++ class.
**
** This object comprises the bulk of the logic of XPipeman. It relies
** heavily on the C code of main.cc and xface.cc.
**
** The object is largely interrupt/mode driven. "Signals" are transmitted
** to the object through the member new_mode(). Signals represent events
** such as "pause", "crash", etc.
**
** The class contains several static member functions which must be tied
** to X11 events such as button presses.
*/

#include "tcache.h"
#include "score.h"
#include "episode.h"
#include "board.h"
#include <sfx.h>

enum game_input
{
    game_input_null = 0,
    game_input_fast,
    game_input_place,
    game_input_shift
};

enum game_list
{
    game_list_null = 0,
    game_list_scores,
    game_list_stats
};

enum game_dialog
{
    game_dialog_null = 0,
    game_dialog_options,
    game_dialog_scores,
    game_dialog_level,
    game_dialog_credits
};

enum game_label
{
    game_label_null = 0,
    game_label_remain,
    game_label_level,
    game_label_levelTitle
};

enum game_button
{
    game_button_null = 0,
    game_button_pause
};

enum game_mode
{
    game_mode_null = 0,
    game_mode_title,
    game_mode_play,
    game_mode_levelover,
    game_mode_gameover,
    game_mode_pause,
    game_mode_quit
};

enum game_signal
{
    game_signal_null = 0,
    game_signal_start,
    game_signal_new,
    game_signal_pause,
    game_signal_continue,
    game_signal_crash,
    game_signal_quit,
    game_signal_error,
    game_signal_flow
};

enum game_larry
{
    game_larry_null = 0,
    game_larry_levelover,
    game_larry_credits
};

class Game
{
    short createdLevel;
    short createdInfo;
    short createdOptions;
    short createdScores;
    short flowcol;
    short flowrow;
    short ocol;
    short orow;
    short inFlow;
    game_mode mode;
    tobj_dir flowdir;
    unsigned flowrate;
    Score score;
    struct score_pars lstats; // Level statistics.
    char *sstats[16];
    char statstrs[16][31+1];
    int new_mode( game_signal );
    int incflow( void );
    int prep_level( void );
    int prep_intr( void );
    int prep_stop( void );
    int prep_board( void );
    int prep_popdown( void );
    static const char * const rtilenames[];
public:
    const char *scorefile;
    Episode episode;
    Board *board;
    Game( void );
    int new_episode( const char*, const char* );
    int event( enum game_signal );
    int event( enum game_input, int, int );
    int credits( void );
    int scores( void );
    int options( void );
    /* Set by outside. */
    int (*guiSetLabelNumber)( enum game_label, unsigned int );
    int (*guiSetLabel)( enum game_label, const char* );
    int (*guiSetList)( enum game_list, const char *[] );
    int (*guiSetStats)( int, struct score_pars*, struct score_pars* );
    int (*guiSetInput)( int, Game* );
    int (*guiSetFlow)( unsigned int, Game* );
    int (*guiDialog)( enum game_dialog, int );
    int (*guiCreateDialog)( enum game_dialog, const char*, Game* );
    /* Used as X11 callbacks. */
    static int gameButton_cb( int, int, int, void* );
    static int gameKey_cb( KeySym, int, int, void* );
    static int gameFlow_cb( void* );
};

#endif
