/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

#include <stdio.h>
#include <signal.h>
#if 0
#include <poll.h>
#endif

#include <X11/X.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/cursorfont.h>
#include <X11/Core.h>
#include <X11/Shell.h>

#include <Button.h>

#include "xpipeman.h"

const char * S_DefaultLevelsdir = XPIPE_LIB ;
const char * S_DefaultScorefile = XPIPE_LIB "/xpipescores";

struct xface xface;
Game game;
TCache tcache;
const char *levelsdir = S_DefaultLevelsdir;
const char *scorefile = S_DefaultScorefile;
const char *language = "C";
char *displayname = NULL;
#ifdef HAVE_OSS
int flagSound = 1;
#else
int flagSound = 0;
#endif

static const char * const S_StandardGlob = "standard.glob";

extern "C" {
static void intr_nosound( int );
}

static void usage( void )
{
    fprintf( stdout, "usage: xpipeman [-levelsdir <path>]"
	" [-scorefile <file>] [-lang <locale>]\n"
	" [-noshm]  [-display <X server>]\n"
	"   locales: C, de, fr, it, gr, la\n" );
}

main( int argc, char *argv[] )
{
    int i;
    int flagNoshm = 0;
    int locality = 0;

    for( i=1; i<argc; ++i )
    {
	if( !strcmp(argv[i],"-nosound") )
	{
	    flagSound = 0;
	    continue;
	}
	if( !strcmp(argv[i],"-levelsdir") )
	{
	    ++i;
	    if( i >= argc )
	    {
		fputs( "ERROR. No argument to \"-levelsdir\".\n", stdout );
		usage();
		exit( 1 );
	    }
	    levelsdir = argv[i];
	    continue;
	}
	if( !strcmp(argv[i],"-lang") )
	{
	    ++i;
	    if( i >= argc )
	    {
		fputs( "ERROR. No argument to \"-lang\".\n", stdout );
		usage();
		exit( 1 );
	    }
	    language = argv[i];
	    continue;
	}
	if( !strcmp(argv[i],"-display") )
	{
	    ++i;
	    if( i >= argc )
	    {
		fputs( "ERROR. No argument to \"-display\".\n", stdout );
		usage();
		exit( 1 );
	    }
	    displayname = argv[i];
	    continue;
	}
	if( !strcmp(argv[i],"-noshm") )
	{
	    flagNoshm = 1;
	    continue;
	}
    }

    if( flagSound )
    {
	signal( SIGUSR1, intr_nosound );
	if( !sfx_init() )
	{
	    flagSound = 0;
	    fputs( "ERROR. Could not intialize sound.\n", stdout );
	}
    }

    if( displayname == NULL )
    {
	if( (displayname=getenv("DISPLAY")) == NULL )
	    displayname = "unix:0.0";
    }
    if( !xfaceInit(displayname,&argc,argv,&game) )
    {
	fputs( "ERROR. Unable to initialize X11.\n", stdout );
	if( flagSound )
	    sfx_finis();
	exit( 1 );
    }

    if( (strncmp(displayname,"unix:",5)==0) || (displayname[0]==':') )
	locality = 1;
    if( flagNoshm )
	locality = 0;

    tcache.setColores( xface.colores );

    game.board = new Board( xface.display, &xface.vinfo,
	xface.winBoard, locality, &tcache, xface.colores );
    game.board->externSetAnim = xfaceSetAnim;

    game.guiSetLabelNumber = xfaceSetLabelNumber;
    game.guiSetLabel = xfaceSetLabel;
    game.guiSetList = xfaceSetList;
    game.guiSetStats = xfaceSetStats;
    game.guiSetInput = xfaceSetInput;
    game.guiSetFlow = xfaceSetFlow;
    game.guiDialog = xfaceDialog;
    game.guiCreateDialog = xfaceCreateDialog;
    game.scorefile = scorefile;

    XtAddEventHandler( xface.wPlayfield, ExposureMask, 0,
	(XtEventHandler)Board::boardExpose_cb, game.board );

    if( !game.new_episode(levelsdir,S_StandardGlob) )
    {
	fputs( "ERROR. Failed to load standard episode.\n", stdout );
	game.event( game_signal_error );
	exit( 1 );
    }

    /*
    ** Start the Game object engine. All further game events will
    ** come from the XFace.
    */
    if( !game.event(game_signal_start) )
    {
	game.event( game_signal_error );
	fputs( "ERROR. Failed to enter title mode.\n", stdout );
	exit( 1 );
    }

    XtAppMainLoop( xface.app );
}

/*
** SFX API call this routine via a signal to indicate that
** sound is no longer available.
*/
static void intr_nosound( int signum )
{
   flagSound = 0;
}
