/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

#include <sys/stat.h>
#include <stdio.h>
#ifndef linux
#include <libgen.h>
#endif
#include <pwd.h>
#include "score.h"

extern int errno;
extern char *sys_errlist[];

void Score::reset( void )
{
    emissa = 0;
    proposita = 0;
    tacta = 0;
    crossed = 0;
    obscura = 0;
    score = 0;
    skill = 0.0;
    level = 0;
}

/*
** Score::accipere()
**
** Include external statistics into the total.
*/
int Score::accipere( struct score_pars *pars )
{
    pars->score  = pars->tacta*4;
    pars->score += pars->crossed*16;
    pars->score -= pars->emissa*2;
    pars->score -= pars->obscura*8;

    proposita += pars->proposita;
    tacta += pars->tacta;
    crossed += pars->crossed;
    emissa += pars->emissa;
    obscura += pars->obscura;
    score += pars->score;

/*
    skill = ((float)proposita/(float)emissa)
	+ ((float)tacta/(float)proposita)
	- ((float)obscura/(float)proposita);
    if( tacta > 0 )
	skill += ((float)crossed/(float)tacta);
*/

    return 1;
}

/*
** Score::fetch()
**
*/
int Score::fetch( struct score_pars *pars )
{
    pars->tacta = tacta;
    pars->crossed = crossed;
    pars->emissa = emissa;
    pars->obscura = obscura;
    pars->proposita = proposita;
    pars->score = score;

    return 1;
}

/*
** Score::summa()
**
** Return a printable string containing the list of high scores.
*/
#if 0
char *Score::summa( const char *filename )
{
    char *p, *op;
    char *oEnd;
    char line[SCORE_MAX_LINE+1];
    char oline[SCORE_MAX_LINE+1];
    char user[31+1];
    unsigned int val, lev;
    unsigned int len;
    FILE *fp;
    static char obuf[511+1];

    if( (fp=fopen(filename,"r")) == NULL )
	return "";

    op = obuf;
    oEnd = obuf+511;

    while( fgets(line,SCORE_MAX_LINE+1,fp) != NULL )
    {
	// Skip leading whitespace.
	for( p=line; isspace(*p); ++p );

	if( *p=='\n' || *p=='#' )
	    continue;

	if( sscanf(p,"%d %s %d",&val,user,&lev) != 3 )
	    continue;

	sprintf( oline, " %5d %02d %-12s\n", val, lev, user );
	len = strlen( oline );

	if( (op+len) >= oEnd )
	    break;
	strcpy( op, oline );
	op += len;
    }

    return obuf;
}
#endif

const char **Score::summaByList( const char *filename )
{
    char *p, *op;
    char *oEnd;
    char line[SCORE_MAX_LINE+1];
    char oline[SCORE_MAX_LINE+1];
    char user[31+1];
    unsigned int val, lev;
    unsigned int len;
    int i = 0;
    FILE *fp;
    static char obuf[1023+1];
    static const char *list[SCORE_MAX_KEEP+1];

    if( (fp=fopen(filename,"r")) == NULL )
	return NULL;

    op = obuf;
    oEnd = obuf+511;

    while( fgets(line,SCORE_MAX_LINE+1,fp) != NULL )
    {
	// Skip leading whitespace.
	for( p=line; isspace(*p); ++p );

	if( *p=='\n' || *p=='#' )
	    continue;

	if( sscanf(p,"%d %s %d",&val,user,&lev) != 3 )
	    continue;

	sprintf( oline, "%5d %02d %-12s", val, lev, user );
	len = strlen( oline );

	if( (op+len+1) >= oEnd )
	    break;
	strcpy( op, oline );
	list[i] = op;
	++i;
	op += len;
	*op = '\0';
	++op;

	if( i >= SCORE_MAX_KEEP )
	    break;
    }

    list[i] = NULL;

    return list;
}

char *Score::summaByString( const char *filename )
{
    char *p, *op;
    char *oEnd;
    char line[SCORE_MAX_LINE+1];
    char oline[SCORE_MAX_LINE+1];
    char user[31+1];
    unsigned int val, lev;
    unsigned int len;
    FILE *fp;
    static char obuf[511+1];

    if( (fp=fopen(filename,"r")) == NULL )
	return "";

    op = obuf;
    oEnd = obuf+511;

    while( fgets(line,SCORE_MAX_LINE+1,fp) != NULL )
    {
	// Skip leading whitespace.
	for( p=line; isspace(*p); ++p );

	if( *p=='\n' || *p=='#' )
	    continue;

	if( sscanf(p,"%d %s %d",&val,user,&lev) != 3 )
	    continue;

	sprintf( oline, " %5d %02d %-12s\n", val, lev, user );
	len = strlen( oline );

	if( (op+len) >= oEnd )
	    break;
	strcpy( op, oline );
	op += len;
    }

    return obuf;
}

/*
** Score::referre()
**
** Add current score to the list of high scores.
**
** FUTURE. Limit number of scores from one user.
**
** Returns.
**    -1 Error.
**     0 Score not high enough (not implemented).
**     1 Score added.
*/
int Score::referre( const char *filename )
{
    char line[SCORE_MAX_LINE+1];
    char *p;
    char *cpy, *path;
    char *outname;
    const char *username;
    unsigned linecount = 0;
    int val;
    int flagCopy = 0;
    struct passwd *pwrec;
    FILE *fp, *fpOut;

    cpy = strdup( filename );
    path = dirname( cpy );

    if( (pwrec=getpwuid(getuid())) != NULL )
	username = pwrec->pw_name;
    else
	username = "[unknown]";

    outname = tempnam( path, "xpipe" );

    if( access(filename,F_OK) == -1 )
    {
	int fd;

	if( (fd=creat(filename,0666)) == -1 )
	    return -1;
	close( fd );
    }

    if( (fp=fopen(filename,"r")) == NULL )
    {
	fprintf( stdout, "ERROR. Bad fopen of \"%s\". %s.\n",
	    filename, sys_errlist[errno] );
	free( cpy );
	free( outname );
	return -1;
    }

    unlink( outname );
    if( (fpOut=fopen(outname,"w")) == NULL )
    {
	fprintf( stdout, "ERROR. Bad fopen of \"%s\". %s.\n",
	    filename, sys_errlist[errno] );
	fclose( fp );
	free( cpy );
	free( outname );
	return -1;
    }

    flagCopy = 0;
    while( fgets(line,SCORE_MAX_LINE+1,fp) != NULL )
    {
	// Skip leading whitespace.
	for( p=line; isspace(*p); ++p );

	if( *p=='\n' || *p=='#' )
	{
	    fputs( line, fpOut );
	    continue;
	}

	++linecount;

	if( linecount > SCORE_MAX_KEEP )
	    break;

	if( flagCopy )
	{
	    fputs( line, fpOut );
	    continue;
	}

	if( sscanf(p,"%d %*s %*d",&val) != 1 )
	    continue;

	if( score > val )
	{
	    ++flagCopy;
	    fprintf( fpOut, "%5d %s %2d\n", score, username, level );
	    fprintf( stdout, "%5d %s %2d\n", score, username, level );
	}
	fputs( line, fpOut );
    }

    if( !flagCopy && (linecount<SCORE_MAX_KEEP) )
	fprintf( fpOut, "%5d %s %2d\n", score, username, level );

    fclose( fp );
    fclose( fpOut );
    free( cpy );
    free( outname );

    if( unlink(filename) == -1 )
    {
	unlink( outname );
	return -1;
    }

    link( outname, filename );
    unlink( outname );

    return 1;
}

void Score::scoreInitPars( struct score_pars *pars )
{
    pars->proposita = 0;
    pars->emissa = 0;
    pars->tacta = 0;
    pars->obscura = 0;
    pars->crossed = 0;
}
