/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

#include "tile.h"

Tile::Tile( void )
{
    state = tile_state_null;
    tobj = NULL;
    sync_tag = -1;
}

int Tile::set( TObj *newtobj )
{
    // FUTURE. Is this possible?
    if( newtobj->type == -1 )
	return 0;

    sync_tag = -1;

    tobj = newtobj;
    if( (state_info=newtobj->initstate(&sync_tag)) == NULL )
	return 0;

    if( sync_tag > 0 )
    {
	state = tile_state_anim;
	/*
	fprintf( stdout, "TMP ti set add t%d %d,%d to sync%d\n",
	    newtobj->type, x, y, sync_tag );
	*/
    }
    else if( sync_tag == 0 )
	state = tile_state_anim;
    else
	state = tile_state_set;

    return 1;
}

void Tile::setpos( int nx, int ny, int nw, int nh )
{
    x = nx;
    y = ny;
    width = nw;
    height = nh;
}

int Tile::incflow( Imago *img, tobj_dir *flowdir, struct tobj_gain *gain )
{
    int flagTerm = 0;
    XImage *fimg;
    struct tobj_state_pipe *s = (struct tobj_state_pipe*)state_info;

    fimg = tobj->incflow( flowdir, &sync_tag, s, gain );

    /* Change in band to out-of-band. */
    flagTerm = (fimg==NULL?1:0);

    if( flagTerm && (sync_tag<0) )
	return 0;

    /*
    ** An obstacle can cause termination, but may also want to display
    ** a frame or sequence. Even if it's just a single frame, the
    ** sync flag must be set. Grab the current frame.
    */
    if( flagTerm && (fimg==NULL) )
	fimg = tobj->latest(s);

    img->copy( fimg, x, y, width, height );

    if( sync_tag > 0 )
    {
	state = tile_state_anim;
	/*
	fprintf( stdout, "TMP ti set add t%d %d,%d to sync%d\n",
	    tobj->type, x, y, sync_tag );
	*/
    }
    else if( sync_tag == 0 )
	state = tile_state_anim;
    else
	state = tile_state_set;

    return TILE_FLOW_DRAW|(flagTerm?0:TILE_FLOW_CONT);
}

int Tile::incmeter( Imago *img )
{
    int rc;
    XImage *fimg;
    struct tobj_state_meter *s = (struct tobj_state_meter*)state_info;

    rc = tobj->incmeter( &sync_tag, s, &fimg );

    // Error.
    if( rc == 0 )
	return 0;

    // Signify that this tile is exhausted.
    if( rc == 2 )
    {
	//fprintf( stdout, "TMP ti meter new tile\n" );
	//fflush( stdout );
	return 2;
    }

    img->copy( fimg, x, y, width, height );

    state = tile_state_anim;

    if( sync_tag > 0 )
    {
	/*
	fprintf( stdout, "TMP ti flow add t%d %d,%d to sync%d\n",
	    tobj->type, x, y, sync_tag );
	*/
    }
    else if( sync_tag == 0 )
	state = tile_state_anim;
    else
	state = tile_state_set;

    return 1;
}

int Tile::canplace( void )
{
    switch( tobj->tclass )
    {
	case tobj_tclass_pipe:
	{
	    struct tobj_state_pipe *s = (struct tobj_state_pipe*)state_info;

	    // FUTURE. How accurate is this?
	    if( s->base_id != 0 )
		return 0;
	}
	case tobj_tclass_obstacle:
	    return (tobj->tinfo.obstacle.flags&0x01);

	case tobj_tclass_start:
	    return 0;
    }
    return 0;
}
