/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

#include "tileList.h"

TileList::TileList()
{
    root = NULL;
}

int TileList::clear( void )
{
    struct tinode *p = root;
    struct tinode *hold;

    while( p != NULL )
    {
	hold = p->next;
	free( p );
	p = hold;
    }
    root = NULL;
    count = 0;
    return 1;
}

int TileList::push( Tile *ntile )
{
    struct tinode *p;

    /*
    ** See if it is already in the list. Hopefully this can
    ** be removed at some point.
    */
    if( root != NULL )
    {
	for( p=root; p->next!=NULL; p=p->next )
	{
	    if( p->tile == ntile )
		return 1;
	}
    }

    if( (p=(struct tinode*)malloc(sizeof(struct tinode))) == NULL )
    {
	fprintf( stdout, "TMP. Bad malloc. %s.\n", sys_errlist[errno] );
	return 0;
    }

    // check( "before push" );
    p->next = root;
    p->prev = NULL;
    p->tile = ntile;
    if( root != NULL )
	root->prev = p;
    root = p;
    // check( "after push" );

    ++count;

    return 1;
}

Tile *TileList::pop( Tile *target )
{
    struct tinode *p = root;

    while( p != NULL )
    {
	if( p->tile == target )
	{
	    Tile *tile;

	    if( p->prev != NULL )
		p->prev->next = p->next;
	    if( p->next != NULL )
		p->next->prev = p->prev;
	    tile = p->tile;
	    --count;
	    if( root == p )
		root = p->next;
	    free( p );
	    return tile;
	}
	p = p->next;
    }
    return NULL;
}

struct tinode *TileList::pop( struct tinode *p )
{
    struct tinode *np;

    if( p->prev != NULL )
	p->prev->next = p->next;
    if( p->next != NULL )
	p->next->prev = p->prev;
    np = p->next;
    --count;
    if( root == p )
	root = p->next;
    free( p );

    return np;
}

void *TileList::grab( void )
{
    struct tinode *nroot = root;

    count = 0;
    root = NULL;

    return (void*)nroot;
}

// FUTURE. Count is not updated.
int TileList::merge( void *handle )
{
    struct tinode *p;
    struct tinode *nroot = (struct tinode*)handle;

    // dump( "TMP tl merge", nroot ); // TMP

    if( nroot == NULL )
	return 1;

    if( root == NULL )
    {
	root = nroot;
	return 1;
    }

    for( p=root; p->next!=NULL; p=p->next );

    p->next = nroot;
    nroot->prev = p;

    return 1;
}

int TileList::check( const char *comment )
{
    struct tinode *p = root;

    fprintf( stdout, "Check %s\n", comment );
    while( p != NULL )
    {
	fprintf( stdout, "node %08x - %08x %08x\n",
	    p, p->prev, p->next );
	fflush( stdout );
	if( (p->prev!=NULL) && (p->prev<((struct tinode*)0x1000)) )
	{
	    fprintf( stdout, "TMP found prev %d 0x%x,0x%x\n",
		count, p->prev, p->next ); fflush( stdout );
	    break;
	}
	if( (p->next!=NULL) && (p->next<((struct tinode*)0x1000)) )
	{
	    fprintf( stdout, "TMP found next %d 0x%x,0x%x\n",
		count, p->prev, p->next );
	    if( p == root )
		fputs( "TMP found is root\n", stdout );
	    fflush( stdout );
	    break;
	}
	if( (p->prev==p) || (p->next==p) )
	{
	    fprintf( stdout, "TMP found2\n" ); fflush( stdout );
	    break;
	}
	p = p->next;
    }
    return 1;
}

int TileList::dump( const char *comment, struct tinode *nroot )
{
    struct tinode *p = nroot;
    Tile *ti;

    fprintf( stdout, "Dump %s\n", comment );
    while( p != NULL )
    {
	ti = p->tile;
	fprintf( stdout, "node t%02d %04d,%04d\n",
	    ti->tobj->type, ti->x, ti->y );
	p = p->next;
    }
    return 1;
}
