/*
** 
**  Copyright (c) 1998 by Larry Plona
** 
**  This distribution  of XPipeman is copyright by Larry Plona.
**  The  package  and  its  component  files  may  be  freely
**  distributed as long as  all copyright notices remain intact
**  and this distribution is not sold  as  commercial software.
** 
**  No warranty of any kind is attached with this software.  It
**  is supplied as is.
** 
** ----------------------------------------------------------------
*/

#ifndef XFACE_H
#define XFACE_H 1

#include <stdio.h>
#include <stdlib.h>
#include <nl_types.h>
/* nl_types defines this. So do the X11 includes. */
#ifdef MALLOC
#undef MALLOC
#endif

#include <X11/X.h>
#include <X11/Xproto.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/keysym.h>
#include <Button.h>
#include <Dial.h>
#include <Frame.h>
#include <GameWindow.h>
#include <Label.h>
#include <Larry.h>
#include <Row.h>
#include <Titulus.h>
#include <X11/xpm.h>

#include "colores.h"
#include "game.h"

extern int errno;
extern char *sys_errlist[];

/*
** Valid for default colormap and all artwork colormaps.
*/
#define XFACE_PIXEL_BLACK  0
#define XFACE_PIXEL_WHITE  1
#define XFACE_PIXEL_LGREY  2
#define XFACE_PIXEL_DGREY  3

/* Offsets into set 1 of message catalog. */
#define XFACE_MSG_QUIT 1
#define XFACE_MSG_NEWGAME 2
#define XFACE_MSG_SCORES 3
#define XFACE_MSG_PAUSE 4
#define XFACE_MSG_CREDITS 5
#define XFACE_MSG_MUTE 6
#define XFACE_MSG_OPTIONS 7
#define XFACE_MSG_GAMEOVER 8
#define XFACE_MSG_LEVELOVER 9
#define XFACE_MSG_REMAIN 10
#define XFACE_MSG_LEVEL 11
#define XFACE_MSG_POPDOWN 12
#define XFACE_MSG_CONTINUE  13
#define XFACE_MSG_AUTHORS   14
#define XFACE_MSG_INENGLISH 15
#define XFACE_MSG_VOLUME    16
#define XFACE_MSG_DISPENSED 17
#define XFACE_MSG_PLACED    18
#define XFACE_MSG_OBSCURED  19
#define XFACE_MSG_TOUCHED   20
#define XFACE_MSG_CROSSED   21
#define XFACE_MSG_SCORE     22
#define XFACE_MSG_TOTAL     23
#define XFACE_MSG_START     24

/* Offsets into set 2 of message catalog. */
#define XFACE_FONT_FIXED     1
#define XFACE_FONT_BUTTONS   6
#define XFACE_FONT_TITLES   11
#define XFACE_FONT_TEXT     16

struct xface
{
    XtAppContext app;
    Display *display;
    Widget wShell;
    Widget wTop;
    Widget wPlayfield;
    Widget wLevelText;
    Widget wLevelTitle;
    Widget wScoresText;
    /* Buttons */
    Widget wPause;
    Widget wMute;
    /* Larry windows */
    Widget wLCredits;
    Widget wLLevel;
    Widget wLScores;
    Widget wLOptions;
    /* Main window fields. */
    Widget wRemain;
    Widget wLevel;
    /* */
    Window winBoard;
    XVisualInfo vinfo;
    GC gc;
    GC gcClear;
    Colores *colores;
    Colormap colormap;
    Cursor cursor;
    Pixmap pixTile;
    /* Fonts */
    XFontStruct *fontButtons;
    XFontStruct *fontFixedText;
    XFontStruct *fontTitle;
    XFontStruct *fontText;
    /* Colors. */
    Pixel pxBlack;
    Pixel pxWhite;
    Pixel pxLGray;
    Pixel pxDGray;
    /* */
    XtIntervalId i_anim;
    XtIntervalId i_flow;
    /* */
    nl_catd cd;
    Game *inputClosure; /* Needed only to cancel button/key handler. */
};

/*
** External API.
*/

extern struct xface xface;

extern int xfaceInit( char*, int*, char*[], Game* );
extern int xfaceSetCursor( const char* );

extern int xfaceSetLabelNumber( enum game_label, unsigned int );
extern int xfaceSetLabel( enum game_label, const char* );
extern int xfaceSetList( enum game_list, const char*[] );
extern int xfaceSetStats( int, struct score_pars*, struct score_pars* );
extern int xfaceSetInput( int, Game* );
extern int xfaceSetFlow( unsigned int, Game* );
extern int xfaceDialog( enum game_dialog, int );
extern int xfaceCreateDialog( enum game_dialog, const char*, Game* );
extern void xfaceFinis( void );

extern int xfaceSetAnim( int, Board* );

#endif
